/* ************************************************************************************
 *	Das Modul beinhaltet Funktionen zur Berechnung des Mond Auf-
 *	und Untergangs.
 * Die Funktionen und Algorithmen haben keinen Anspruch auf
 *	hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module includes functions for calculating the moons
 * rise and set.
 * The functions and algorithms do not claim 
 * high accuracy, completeness and correctness.
 *
 *	Funktionsliste - Function list:
 *
 * - nutationLong()				- 	Berechnet den Lngengrad der Nutation
 *											Calculates the longitude of the nutation
 *  - moonHP()					- 	Berechnet die Parallaxe des Mondes
 *										   	 Calculates the parallax of the moon
 *  - moonLatitude()			- 	Berechnet den Breitengrad des Mondes
 *										   	Calculates the latitude of the moon
 *  - moonLongitude()		- 	Berechnet den Lngengrad des Mondes
 *										   	Calculates the longitude of the moon
 * - moonriseLocalTime()	- 	Berechnet die Lokale Zeit des Mond Aufgangs
 *										   	 Calculates the local time of the moonrise
 * - moonsetLocalTime()	- 	Berechnet die Lokale Zeit des Mond Untergangs
 *										   	Calculates the local time of the moonset
 * - moonRiseAz()				- 	Berechnet Azimuth und Hhe des Mondaufgangs
 *											Calculates azimuth and altitude of moon rise
 * - moonSetAz()				- 	Berechnet Azimuth und Hhe des Monduntergangs
 *											Calculates azimuth and altitude of moon set
 *
 *
 *	12.12.2020, Dieter Steffan
 *
 *********************************************************************************** */	

 
 
 /* *********************************************************************************
 * Die Funktion berechnet den ekliptischen Lngegrad der Nutation. 
 * Wird in der Funktion moonriseLocalTime() verwendet.
 *	
 *	bergabewert:
 * 		gday		= Greenwich Tag
 *		gmonth	= Greenwich Monat
 *		gyear	= Greenwich Jahr
 *	
 *	Returnwert:
 *		DP		= 	Der Lngengrad als dezimaler Wert und als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the ecliptic degree of length of the nutation. Used in the 
 * moonriseLocalTime() function. 
 * 
 * Transfer value: 
 * 		gday = Greenwich day 
 * 		gmonth = Greenwich month 
 * 		gyear = Greenwich year 
 * 
 * Return value: 
 * 		DP = The longitude as decimal value and as Number
  *************************************************************************************** */
function  nutationLong(gday, gmonth, gyear){
	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var GDAY = Number(gday);				// Greenwich Tag - Greenwich Day
 	var GMONTH = Number(gmonth);	// Greenwich Monat - Greenwich month
	var GYEAR = parseInt(gyear);			// Greenwich Jahr - Greenwich year
	var DJ = 0.0 ;											// MDJ
	var T = 0.0;												// Vergangene Jahrhunderte - Centuries past
	var T2 = 0.0;												// Vergangene Jahrhunderte zum Quadrat
																		// Centuries past squared
	var DP = 0.0;												// Returnwert - Return value
	// Hilfsvariablen
	// Auxilary variables
	var A = 0.0; var B = 0.0; var L1 = 0.0; var L2 = 0.0; var D1 = 0.0; var D2 = 0.0;
	var M1 = 0.0; var M2 = 0.0; var N1 = 0.0; var N2 = 0.0;
	// Berechne das MJD (Modifiziertes julianisches Datum) ...
	// Find the MJD (Modified Julian Date). . .
	DJ = calDate2JulDate(GDAY, GMONTH, GYEAR) - 2415020.0;
	// ... dann daraus das bereits vergangene Jahrhundert ...
	// . . . then from it the century already past . . .
	T = DJ / 36525.0;
	// ... und das zum Quadrat ...
	// . . . and that squared . . .
	T2 = T * T;
	A = 100.0021358 * T; B = 360.0 * (A - Math.floor(A));
	L1 = 279.6967 + 0.000303 * T2 + B; L2 = 2.0 * (L1/RAD);
	A = 1336.855231 * T; B = 360.0 * (A - Math.floor(A));
    D1 = 270.4342 - 0.001133 * T2 + B; D2 = 2.0 * (D1/RAD);
	A = 99.99736056 * T;  B = 360.0 * (A - Math.floor(A));
	M1 = 358.4758 - 0.00015 * T2 + B;  M1 = (M1/RAD);
	A = 1325.552359 * T; B = 360.0 * (A - Math.floor(A));
    M2 = 296.1046 + 0.009192 * T2 + B; M2 = (M2/RAD)
    A = 5.372616667 * T; B = 360.0 * (A - Math.floor(A));
    N1 = 259.1833 + 0.002078 * T2 - B; N1 = (N1/RAD);
	N2 = 2 * N1;
	
    DP = (-17.2327 - 0.01737 * T) * Math.sin(N1);
    DP = DP + (-1.2729 - 0.00013 * T) * Math.sin(L2) + 0.2088 * Math.sin(N2);
    DP = DP - 0.2037 * Math.sin(D2) + (0.1261 - 0.00031 * T) * Math.sin(M1);
    DP = DP + 0.0675 * Math.sin(M2) - (0.0497 - 0.00012 * T) * Math.sin(L2 + M1);
    DP = DP - 0.0342 * Math.sin(D2 - N1) - 0.0261 * Math.sin(D2 + M2);
    DP = DP + 0.0214 * Math.sin(L2 - M1) - 0.0149 * Math.sin(L2 - D2 + M2);
    DP = DP + 0.0124 * Math.sin(L2 - N1) + 0.0114 * Math.sin(D2 - M2);
	DP = DP / 3600.0;
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(DP);
	
}// End of nutationLong()



function moonDiameter( lh , lm ,ls , dst, tz, dy, mn, yr){
	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LH = Number(lh);						// Berechnung Stunde - Calculated hour
	var LM = Number(lm);						// Berechnung Minute - Calculated minute
	var LS = Number(ls);						// Berechnung Sekunde - Calculated second
	var DY = Number(dy);						// Berechnung Tag - Calculated day
 	var MN = Number(mn);					// Berechnung Monat - Calculated month
	var YR = parseInt(yr);						// Berechnung Jahr - Calculated year
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var dia= 0.0;											// Winkeldurchmesser des Mondes - Moon's diameter
	
	var HP = (moonHP(LH, LM, LS, DST, TZ, DY, MN, YR))/ RAD;

    var R = 6378.14 / Math.sin(HP);
    dia = 384401.0 * (0.5181 / R);
        
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(dia);
	
 }// End of Diameter()



function moonDistance( lh , lm ,ls , dst, tz, dy, mn, yr){
	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LH = Number(lh);						// Berechnung Stunde - Calculated hour
	var LM = Number(lm);						// Berechnung Minute - Calculated minute
	var LS = Number(ls);						// Berechnung Sekunde - Calculated second
	var DY = Number(dy);						// Berechnung Tag - Calculated day
 	var MN = Number(mn);					// Berechnung Monat - Calculated month
	var YR = Number(yr);						// Berechnung Jahr - Calculated year
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var dist = 0.0;										// Abstand des Mondes - Moon's distance
	
	var HP = (moonHP(LH, LM, LS, DST, TZ, DY, MN, YR))/ RAD;
	//console.log(HP);
	dist = 6378.14 / Math.sin(HP);
	//console.log(dist);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(dist);
	
 }// End of moonDistance()
 


    /* *********************************************************************************
 * Die Funktion berechnet den Parallaxe des Mondes zum gegebenen 
 * Datum. Verwendet dieselben Parameter wie moonLongitude().
 *	
 *	bergabewert:
 *		dst		=	Daylight saving time
 *		tz			=	Zeitzone
 * 		gday		= Greenwich Tag
 *		gmonth	= Greenwich Monat
 *		gyear	= Greenwich Jahr
 *		ut			= Weltzeit
 *		jd			= Julianisches Datum
 *	
 *	Returnwert:
 *		MM		= 	Die Parallaxe des Mondes in Grad und als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the parallax of the Moon at the given
 * Date. Uses the same parameters as moonLongitude (). 
 *
 * Transfer value:
 * 		dst 		= Daylight saving time
 * 		tz 		= Time zone
 * 		gday 	= Greenwich day
 * 		gmonth = Greenwich month
 * 		gyear 	= Greenwich year
 * 		ut 		= World Time
 * 		jd 			= Julian date
 *
 * Return value:
 * 		MM 		= The latitude of the Moon in degrees and as a number
  *************************************************************************************** */
 function moonHP( lh , lm ,ls , dst, tz, dy, mn, yr){
	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LH = Number(lh);						// Berechnung Stunde - Calculated hour
	var LM = Number(lm);						// Berechnung Minute - Calculated minute
	var LS = Number(ls);						// Berechnung Sekunde - Calculated second
	var DY = Number(dy);						// Berechnung Tag - Calculated day
 	var MN = Number(mn);						// Berechnung Monat - Calculated month
	var YR = Number(yr);						// Berechnung Jahr - Calculated year
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var UTAR = new Array();			// Ergebnis Array der UT - Result array of UT
	var UT = 0.0;										// Weltzeit - Universal Time
	var GD = 0;										// Greenwich Tag, Monat, Jahr
	var GM = 0;										// Greenwich day, month, year
	var GY = 0;
	var T = 0.0; 										//  	-"-		als Dezimalwert - as decimal value
	var T2 = 0.0;										// T^2 - T squared
	var Q = 0.0;
	var M1 = 0.0;										// Hilfsvariablen der Berechnungsreihe
																// Auxiliary variables of the calculation series
	var M2 = 0.0;var M3 = 0.0;var M4 = 0.0;var M5 = 0.0;var M6 = 0.0;var ML = 0.0;var MS = 0.0;
	var MD = 0.0;var ME1 = 0.0;var MF = 0.0;var NA = 0.0;var A = 0.0;var S1 = 0.0;var S2 = 0.0;
	var B = 0.0;var S3 = 0.0;var C = 0.0;var S4 = 0.0;var E = 0.0;var E1 = 0.0;var PM = 0.0;
	// Den Tag des Berechnungsdatums um 00:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 10:00 . . .
	GD = localtime2GreenwichDay(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... berechne nun die dezimale Zahl der UT aus der Unhrzeit und Datum mit TZ und DST...
	// . . . now find the decimal number of UT from the time and date with TZ and DST. . .
	UT = localtime2UT(LH, LM, LS, DST, TZ, DY, MN, YR);
	// ... den Teil des Jahrhunderts berechnen ...
	// . . . calculate the part of the century . . .
	T = ((calDate2JulDate(GD, GM, GY) - 2415020.0) /  36525.0) + (UT / 876600.0);
	// ... und T zum Quadrat ...
	// ... and T squared ...
	T2 = Math.pow(T, 2);
	// .. einige Hilfsvariablen der Berechnungsreihe ...
	// ... some auxiliary variables of the calculation series ...
	M1 = 27.32158213;
	M2 = 365.2596407;
	M3 = 27.55455094;
	M4 = 29.53058868;
	M5 = 27.21222039;
	M6 = 6798.363307;
	// ... und das modifizieret julianische Datum (MJD)berechnen ...
	// . . . and calculate the modified Julian date (MJD).
	Q = calDate2JulDate(GD, GM, GY)- 2415020.0 + (UT / 24.0);
	// ... und die vorher definierten Hilfsvariablen mit dem MJD verrechnen ...
	// . . . and calculate the previously defined auxiliary variables with the MJD . . .
	M1 = Q / M1;
	M2 = Q / M2;
	M3 = Q / M3;
	M4 = Q / M4;
	M5 = Q / M5;
	M6 = Q / M6;
	// ... nun die Nachkommaanteile der Hilfsvariablen mit 360 verrechnen ...
	// . . . now calculate the decimal parts of the auxiliary variables with 360 . . .
	M1 = 360 * (M1 - parseInt(M1) );
	M2 = 360 * (M2 - parseInt(M2) );
	M3 = 360 * (M3 - parseInt(M3) );
	M4 = 360 * (M4 - parseInt(M4) );
	M5 = 360 * (M5 - parseInt(M5) );
	M6 = 360 * (M6 - parseInt(M6) );
	// ... weitere Hilfavraiablen ...
	// ... further auxilary variables ...
	ML = 270.434164 + M1 - (0.001133 - 0.0000019 * T) * T2;
	MS = 358.475833 + M2 - (0.00015 + 0.0000033 * T) * T2;
	MD = 296.104608 + M3 + (0.009192 + 0.0000144 * T) * T2;
	ME1 = 350.737486 + M4 - (0.001436 - 0.0000019 * T) * T2;
	MF = 11.250889 + M5 - (0.003211 + 0.0000003 * T) * T2;
	NA = 259.183275 - M6 + (0.002078 + 0.0000022 * T) * T2;
	A = (51.2 + 20.2 * T)/ RAD; S1 = Math.sin(A ); S2 = Math.sin(NA / RAD);
	B = 346.56 + (132.87 - 0.0091731 * T) * T;
	S3 = 0.003964 * Math.sin(B / RAD);
	C = (NA + 275.05 - 2.3 * T)/ RAD; S4 = Math.sin(C);
	ML = ML + 0.000233 * S1 + S3 + 0.001964 * S2;
	MS = MS - 0.001778 * S1;
	MD = MD + 0.000817 * S1 + S3 + 0.002541 * S2;
	MF = MF + S3 - 0.024691 * S2 - 0.004328 * S4;
	ME1 = ME1 + 0.002011 * S1 + S3 + 0.001964 * S2;
	E = 1.0 - (0.002495 + 0.00000752 * T) * T; E2 = E * E;
	ML = ML / RAD; MS = MS / RAD; NA = NA / RAD;
	ME1 = ME1 / RAD;  MF = MF / RAD;  MD = MD / RAD;
    PM = 0.950724 + 0.051818 * Math.cos(MD) + 0.009531 * Math.cos(2 * ME1 - MD);
    PM = PM + 0.007843 * Math.cos(2 * ME1) + 0.002824 * Math.cos(2 * MD);
    PM = PM + 0.000857 * Math.cos(2 * ME1 + MD) + E * 0.000533 * Math.cos(2 * ME1 - MS);
    PM = PM + E * 0.000401 * Math.cos(2 * ME1 - MD - MS);
    PM = PM + E * 0.00032 * Math.cos(MD - MS) - 0.000271 * Math.cos(ME1);
    PM = PM - E * 0.000264 * Math.cos(MS + MD) - 0.000198 * Math.cos(2 * MF - MD);
    PM = PM + 0.000173 * Math.cos(3 * MD) + 0.000167 * Math.cos(4 * ME1 - MD);
    PM = PM - E * 0.000111 * Math.cos(MS) + 0.000103 * Math.cos(4 * ME1 - 2 * MD);
    PM = PM - 0.000084 * Math.cos(2 * MD - 2 * ME1) - E * 0.000083 * Math.cos(2 * ME1 + MS);
    PM = PM + 0.000079 * Math.cos(2 * ME1 + 2 * MD) + 0.000072 * Math.cos(4 * ME1);
    PM = PM + E * 0.000064 * Math.cos(2 * ME1 - MS + MD) - E * 0.000063 * Math.cos(2 * ME1 + MS - MD);
    PM = PM + E * 0.000041 * Math.cos(MS + ME1) + E * 0.000035 * Math.cos(2 * MD - MS);
    PM = PM - 0.000033 * Math.cos(3 * MD - 2 * ME1) - 0.00003 * Math.cos(MD + ME1);
    PM = PM - 0.000029 * Math.cos(2 * (MF - ME1)) - E * 0.000029 * Math.cos(2 * MD + MS);
    PM = PM + E2 * 0.000026 * Math.cos(2 * (ME1 - MS)) - 0.000023 * Math.cos(2 * (MF - ME1) + MD);
    PM = PM + E * 0.000019 * Math.cos(4 * ME1 - MS - MD);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(PM);
	
 }// End of moonHP()
 
 
 
 
   /* *********************************************************************************
 * Die Funktion berechnet den Breitengrad des Mondes zum gegebenen 
 * Datum. Verwendet dieselben Parameter wie moonLongitude().
 *	
 *	bergabewert:
 *		dst		=	Daylight saving time
 *		tz			=	Zeitzone
 * 		gday		= Greenwich Tag
 *		gmonth	= Greenwich Monat
 *		gyear	= Greenwich Jahr
 *		ut			= Weltzeit
 *		jd			= Julianisches Datum
 *	
 *	Returnwert:
 *		MM		= 	Der Breitengrad des Mondes in Grad und als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the latitude of the Moon at the given
 * Date. Uses the same parameters as moonLongitude (). 
 *
 * Transfer value:
 * 		dst 		= Daylight saving time
 * 		tz 		= Time zone
 * 		gday 	= Greenwich day
 * 		gmonth = Greenwich month
 * 		gyear 	= Greenwich year
 * 		ut 		= World Time
 * 		jd 			= Julian date
 *
 * Return value:
 * 		MM 		= The latitude of the Moon in degrees and as a number
  *************************************************************************************** */
 function moonLatitude( lh , lm ,ls , dst, tz, dy, mn, yr){
	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LH = Number(lh);						// Berechnung Stunde - Calculated hour
	var LM = Number(lm);						// Berechnung Minute - Calculated minute
	var LS = Number(ls);						// Berechnung Sekunde - Calculated second
	var DY = Number(dy);						// Berechnung Tag - Calculated day
 	var MN = Number(mn);						// Berechnung Monat - Calculated month
	var YR = Number(yr);						// Berechnung Jahr - Calculated year
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var UTAR = new Array();				// Ergebnis Array der UT - Result array of UT
	var UT = 0.0;											// Weltzeit - Universal Time
	var GD = 0;											// Greenwich Tag, Monat, Jahr
	var GM = 0;											// Greenwich day, month, year
	var GY = 0;
	var T = 0.0; 											//  	-"-		als Dezimalwert - as decimal value
	var T2 = 0.0;											// T^2 - T squared
	var Q = 0.0;
	var M1 = 0.0;											// Hilfsvariablen der Berechnungsreihe
																	// Auxiliary variables of the calculation series
	var M2 = 0.0;var M3 = 0.0;var M4 = 0.0;var M5 = 0.0;var M6 = 0.0;var ML = 0.0;var MS = 0.0;
	var MD = 0.0;var ME1 = 0.0;var MF = 0.0;var NA = 0.0;var A = 0.0;var S1 = 0.0;var S2 = 0.0;
	var B = 0.0;var S3 = 0.0;var C = 0.0;var S4 = 0.0;var E = 0.0;var E1 = 0.0;var G = 0.0;var BM = 0.0;
	var W1 = 0.0;var W2 = 0.0;
	// Den Tag des Berechnungsdatums um 00:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 10:00 . . .
	GD = localtime2GreenwichDay(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(LH, LM, LS, DST, TZ, DY, MN, YR);
	// ... berechne nun die dezimale Zahl der UT aus der Unhrzeit und Datum mit TZ und DST...
	// . . . now find the decimal number of UT from the time and date with TZ and DST. . .
	UT = localtime2UT(LH, LM, LS, DST, TZ, DY, MN, YR);
	// ... den Teil des Jahrhunderts berechnen ...
	// ... calculate the part of the century ...
	T = ((calDate2JulDate(GD, GM, GY) - 2415020.0) /  36525.0) + (UT / 876600.0);
	// ... und T zum Quadrat ...
	// ... and T squared ...
	T2 = Math.pow(T, 2);
	// .. einige Hilfsvariablen der Berechnungsreihe ...
	// ... some auxiliary variables of the calculation series ...
	M1 = 27.32158213;
	M2 = 365.2596407;
	M3 = 27.55455094;
	M4 = 29.53058868;
	M5 = 27.21222039;
	M6 = 6798.363307;
	// ... und das modifizieret julianische Datum (MJD)berechnen ...
	// . . . and calculate the modified Julian date (MJD).
	Q = calDate2JulDate(GD, GM, GY)- 2415020.0 + (UT / 24.0);
	// ... und die vorher definierten Hilfsvariablen mit dem MJD verrechnen ...
	// . . . and calculate the previously defined auxiliary variables with the MJD . . .
	M1 = Q / M1;
	M2 = Q / M2;
	M3 = Q / M3;
	M4 = Q / M4;
	M5 = Q / M5;
	M6 = Q / M6;
	// ... nun die Nachkommaanteile der Hilfsvariablen mit 360 verrechnen ...
	// . . . now calculate the decimal parts of the auxiliary variables with 360 . . .
	M1 = 360 * (M1 - parseInt(M1) );
	M2 = 360 * (M2 - parseInt(M2) );
	M3 = 360 * (M3 - parseInt(M3) );
	M4 = 360 * (M4 - parseInt(M4) );
	M5 = 360 * (M5 - parseInt(M5) );
	M6 = 360 * (M6 - parseInt(M6) );
	// ... weitere Hilfavraiablen ...
	// ... further auxilary variables ...
	ML = 270.434164 + M1 - (0.001133 - 0.0000019 * T) * T2;
	MS = 358.475833 + M2 - (0.00015 + 0.0000033 * T) * T2;
	MD = 296.104608 + M3 + (0.009192 + 0.0000144 * T) * T2;
	ME1 = 350.737486 + M4 - (0.001436 - 0.0000019 * T) * T2;
	MF = 11.250889 + M5 - (0.003211 + 0.0000003 * T) * T2;
	NA = 259.183275 - M6 + (0.002078 + 0.0000022 * T) * T2;
	A = (51.2 + 20.2 * T)/ RAD; S1 = Math.sin(A ); S2 = Math.sin(NA / RAD);
	B = 346.56 + (132.87 - 0.0091731 * T) * T;
	S3 = 0.003964 * Math.sin(B / RAD);
	C = (NA + 275.05 - 2.3 * T)/ RAD; S4 = Math.sin(C);
	ML = ML + 0.000233 * S1 + S3 + 0.001964 * S2;
	MS = MS - 0.001778 * S1;
	MD = MD + 0.000817 * S1 + S3 + 0.002541 * S2;
	MF = MF + S3 - 0.024691 * S2 - 0.004328 * S4;
	ME1 = ME1 + 0.002011 * S1 + S3 + 0.001964 * S2;
	E = 1.0 - (0.002495 + 0.00000752 * T) * T; E2 = E * E;
	ML = ML / RAD; MS = MS / RAD; NA = NA / RAD;
	ME1 = ME1 / RAD;  MF = MF / RAD;  MD = MD / RAD;
    G = 5.128189 * Math.sin(MF) + 0.280606 * Math.sin(MD + MF);
    G = G + 0.277693 * Math.sin(MD - MF) + 0.173238 * Math.sin(2 * ME1 - MF);
    G = G + 0.055413 * Math.sin(2 * ME1 + MF - MD) + 0.046272 * Math.sin(2 * ME1 - MF - MD);
    G = G + 0.032573 * Math.sin(2 * ME1 + MF) + 0.017198 * Math.sin(2 * MD + MF);
    G = G + 0.009267 * Math.sin(2 * ME1 + MD - MF) + 0.008823 * Math.sin(2 * MD - MF);
    G = G + E * 0.008247 * Math.sin(2 * ME1 - MS - MF) + 0.004323 * Math.sin(2 * (ME1 - MD) - MF);
    G = G + 0.0042 * Math.sin(2 * ME1 + MF + MD) + E * 0.003372 * Math.sin(MF - MS - 2 * ME1);
    G = G + E * 0.002472 * Math.sin(2 * ME1 + MF - MS - MD);
    G = G + E * 0.002222 * Math.sin(2 * ME1 + MF - MS);
    G = G + E * 0.002072 * Math.sin(2 * ME1 - MF - MS - MD);
    G = G + E * 0.001877 * Math.sin(MF - MS + MD) + 0.001828 * Math.sin(4 * ME1 - MF - MD);
    G = G - E * 0.001803 * Math.sin(MF + MS) - 0.00175 * Math.sin(3 * MF);
    G = G + E * 0.00157 * Math.sin(MD - MS - MF) - 0.001487 * Math.sin(MF + ME1);
    G = G - E * 0.001481 * Math.sin(MF + MS + MD) + E * 0.001417 * Math.sin(MF - MS - MD);
    G = G + E * 0.00135 * Math.sin(MF - MS) + 0.00133 * Math.sin(MF - ME1);
    G = G + 0.001106 * Math.sin(MF + 3 * MD) + 0.00102 * Math.sin(4 * ME1 - MF);
    G = G + 0.000833 * Math.sin(MF + 4 * ME1 - MD) + 0.000781 * Math.sin(MD - 3 * MF);
    G = G + 0.00067 * Math.sin(MF + 4 * ME1 - 2 * MD) + 0.000606 * Math.sin(2 * ME1 - 3 * MF);
    G = G + 0.000597 * Math.sin(2 * (ME1 + MD) - MF);
    G = G + E * 0.000492 * Math.sin(2 * ME1 + MD - MS - MF) + 0.00045 * Math.sin(2 * (MD - ME1) - MF);
    G = G + 0.000439 * Math.sin(3 * MD - MF) + 0.000423 * Math.sin(MF + 2 * (ME1 + MD));
    G = G + 0.000422 * Math.sin(2 * ME1 - MF - 3 * MD) - E * 0.000367 * Math.sin(MS + MF + 2 * ME1 - MD);
    G = G - E * 0.000353 * Math.sin(MS + MF + 2 * ME1) + 0.000331 * Math.sin(MF + 4 * ME1);
    G = G + E * 0.000317 * Math.sin(2 * ME1 + MF - MS + MD);
    G = G + E2 * 0.000306 * Math.sin(2 * (ME1 - MS) - MF) - 0.000283 * Math.sin(MD + 3 * MF);
	W1 = 0.0004664 * Math.cos(NA); W2 = 0.0000754 * Math.cos(C);
	// Breitengrad in Radiant ...
	// Latitude in radians ...
	BM = G / RAD * (1.0 - W1 - W2);
	// Breitengrad in Grad umrechnen ...
	// Convert llatitude to degrees ...
	BM = BM * RAD;
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(BM);
	
 }// End of moonLatitude()
 
 
 
 
  /* *********************************************************************************
 * Die Funktion berechnet den Lngengrad des Mondes zum gegebenen 
 * Datum.
 *	
 *	bergabewert:
 *		dst		=	Daylight saving time
*		tz			=	Zeitzone
* 		gday		= Greenwich Tag
*		gmonth	= Greenwich Monat
*		gyear	= Greenwich Jahr
*		ut			= Weltzeit
*		jd			= Julianisches Datum
 *	
 *	Returnwert:
 *		MM		= 	Der Lngengrad des Mondes in Grad und als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the longitude of the moon at the given 
 * date. 
 * 
 * Transfer value: 
 * 		dst 		= Daylight saving time 
 * 		tz 		= Time zone 
 * 		gday 	= Greenwich day 
 * 		gmonth = Greenwich month 
 * 		gyear 	= Greenwich year 
 * 		ut 		= World time 
 *		 jd 		= Julian date 
 * 
 * Return value: 
 * 		MM 		= The longitude of the moon in degrees and as Number
  *************************************************************************************** */
 function moonLongitude( lh , lm ,ls , dst, tz, dy, mn, yr){
	// Transfer parameters Save data type-safe
	var LH = Number(lh);						// Berechnung Stunde - Calculated hour
	var LM = Number(lm);						// Berechnung Minute - Calculated minute
	var LS = Number(ls);						// Berechnung Sekunde - Calculated second
	var DY = Number(dy);						// Berechnung Tag - Calculated day
 	var MN = Number(mn);					// Berechnung Monat - Calculated month
	var YR = Number(yr);						// Berechnung Jahr - Calculated year
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var JD = 0.0; 										// Julianisches Datum - Julian date
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var UTAR = new Array();				// Ergebnis Array der UT - Result array of UT
	var UT = 0.0;											// Weltzeit - Universal Time
	var GD = 0;											// Greenwich Tag, Monat, Jahr
	var GM = 0;											// Greenwich day, month, year
	var GY = 0;
	var T = 0.0; 											//  	-"-		als Dezimalwert - as decimal value
	var T2 = 0.0;											// T^2 - T squared
	var Q = 0.0;
	var M1 = 0.0;											// Hilfsvariablen der Berechnungsreihe
																	// Auxiliary variables of the calculation series
	var M2 = 0.0;var M3 = 0.0;var M4 = 0.0;var M5 = 0.0;var M6 = 0.0;var ML = 0.0;var MS = 0.0;
	var MD = 0.0;var ME1 = 0.0;var MF = 0.0;var NA = 0.0;var A = 0.0;var S1 = 0.0;var S2 = 0.0;
	var B = 0.0;var S3 = 0.0;var C = 0.0;var S4 = 0.0;var E = 0.0;var E1 = 0.0;var L = 0.0;var MM = 0.0; var MM2 = 0.0;
		// Den Tag des Berechnungsdatums um 00:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 10:00 . . .
	GD = localtime2GreenwichDay(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(LH, LM, LS, DST, TZ, DY, MN, YR);
	// ... berechne nun die dezimale Zahl der UT aus der Uhrzeit und Datum mit TZ und DST...
	// . . . now find the decimal number of UTs from the time and date with TZ and DST. . .
	UT = localtime2UT(LH, LM, LS, DST, TZ, DY, MN, YR);
	// ... den Teil des vergangenen Jahrhunderts berechnen ...
	// . . . calculate the part of the last century . . .
	T = ((calDate2JulDate(GD, GM, GY) - 2415020.0) /  36525.0) + (UT / 876600.0);
	// ... und T zum Quadrat ...
	// ... and T squared ...
	T2 = Math.pow(T, 2);
	// .. einige Hilfsvariablen der Berechnungsreihe ...
	// ... some auxiliary variables of the calculation series ...
	M1 = 27.32158213;
	M2 = 365.2596407;
	M3 = 27.55455094;
	M4 = 29.53058868;
	M5 = 27.21222039;
	M6 = 6798.363307;
	// ... und das modifizieret julianische Datum (MJD)berechnen ...
	// . . . and calculate the modified Julian date (MJD).
	Q = calDate2JulDate(GD, GM, GY)- 2415020.0 + (UT / 24.0);
	// ... und die vorher definierten Hilfsvariablen mit dem MJD verrechnen ...
	// . . . and calculate the previously defined auxiliary variables with the MJD . . .
	M1 = Q / M1;
	M2 = Q / M2;
	M3 = Q / M3;
	M4 = Q / M4;
	M5 = Q / M5;
	M6 = Q / M6;
	// ... nun die Nachkommaanteile der Hilfsvariablen mit 360 verrechnen ...
	// . . . now calculate the decimal parts of the auxiliary variables with 360 . . .
	M1 = 360 * (M1 - parseInt(M1) );
	M2 = 360 * (M2 - parseInt(M2) );
	M3 = 360 * (M3 - parseInt(M3) );
	M4 = 360 * (M4 - parseInt(M4) );
	M5 = 360 * (M5 - parseInt(M5) );
	M6 = 360 * (M6 - parseInt(M6) );
	// ... weitere Hilfsvraiablen ...
	// ... further auxilary variables ...
	ML = 270.434164 + M1 - (0.001133 - 0.0000019 * T) * T2;
	MS = 358.475833 + M2 - (0.00015 + 0.0000033 * T) * T2;
	MD = 296.104608 + M3 + (0.009192 + 0.0000144 * T) * T2;
	ME1 = 350.737486 + M4 - (0.001436 - 0.0000019 * T) * T2;
	MF = 11.250889 + M5 - (0.003211 + 0.0000003 * T) * T2;
	NA = 259.183275 - M6 + (0.002078 + 0.0000022 * T) * T2;
	A = (51.2 + 20.2 * T)/ RAD; S1 = Math.sin(A ); S2 = Math.sin(NA / RAD);
	B = 346.56 + (132.87 - 0.0091731 * T) * T;
	S3 = 0.003964 * Math.sin(B / RAD);
	C = (NA + 275.05 - 2.3 * T)/ RAD; S4 = Math.sin(C);
	ML = ML + 0.000233 * S1 + S3 + 0.001964 * S2;
	MS = MS - 0.001778 * S1;
	MD = MD + 0.000817 * S1 + S3 + 0.002541 * S2;
	MF = MF + S3 - 0.024691 * S2 - 0.004328 * S4;
	ME1 = ME1 + 0.002011 * S1 + S3 + 0.001964 * S2;
	E = 1.0 - (0.002495 + 0.00000752 * T) * T; E2 = E * E;
	ML = ML / RAD; MS = MS / RAD; NA = NA / RAD;
	ME1 = ME1 / RAD;  MF = MF / RAD;  MD = MD / RAD;
	
    L = 6.28875 * Math.sin(MD) + 1.274018 * Math.sin(2 * ME1 - MD);
    L = L + 0.658309 * Math.sin(2 * ME1) + 0.213616 * Math.sin(2 * MD);
    L = L - E * 0.185596 * Math.sin(MS) - 0.114336 * Math.sin(2 * MF);
    L = L + 0.058793 * Math.sin(2 * (ME1 - MD));
    L = L + 0.057212 * E * Math.sin(2 * ME1 - MS - MD) + 0.05332 * Math.sin(2 * ME1 + MD);
    L = L + 0.045874 * E * Math.sin(2 * ME1 - MS) + 0.041024 * E * Math.sin(MD - MS);
    L = L - 0.034718 * Math.sin(ME1) - E * 0.030465 * Math.sin(MS + MD);
    L = L + 0.015326 * Math.sin(2 * (ME1 - MF)) - 0.012528 * Math.sin(2 * MF + MD);
    L = L - 0.01098 * Math.sin(2 * MF - MD) + 0.010674 * Math.sin(4 * ME1 - MD);
    L = L + 0.010034 * Math.sin(3 * MD) + 0.008548 * Math.sin(4 * ME1 - 2 * MD);
    L = L - E * 0.00791 * Math.sin(MS - MD + 2 * ME1) - E * 0.006783 * Math.sin(2 * ME1 + MS);
    L = L + 0.005162 * Math.sin(MD - ME1) + E * 0.005 * Math.sin(MS + ME1);
    L = L + 0.003862 * Math.sin(4 * ME1) + E * 0.004049 * Math.sin(MD - MS + 2 * ME1);
    L = L + 0.003996 * Math.sin(2 * (MD + ME1)) + 0.003665 * Math.sin(2 * ME1 - 3 * MD);
    L = L + E * 0.002695 * Math.sin(2 * MD - MS) + 0.002602 * Math.sin(MD - 2 * (MF + ME1));
    L = L + E * 0.002396 * Math.sin(2 * (ME1 - MD) - MS) - 0.002349 * Math.sin(MD + ME1);
    L = L + E2 * 0.002249 * Math.sin(2 * (ME1 - MS)) - E * 0.002125 * Math.sin(2 * MD + MS);
    L = L - E2 * 0.002079 * Math.sin(2 * MS) + E2 * 0.002059 * Math.sin(2 * (ME1 - MS) - MD);
    L = L - 0.001773 * Math.sin(MD + 2 * (ME1 - MF)) - 0.001595 * Math.sin(2 * (MF + ME1));
    L = L + E * 0.00122 * Math.sin(4 * ME1 - MS - MD) - 0.00111 * Math.sin(2 * (MD + MF));
    L = L + 0.000892 * Math.sin(MD - 3 * ME1) - E * 0.000811 * Math.sin(MS + MD + 2 * ME1);
    L = L + E * 0.000761 * Math.sin(4 * ME1 - MS - 2 * MD);
    L = L + E2 * 0.000704 * Math.sin(MD - 2 * (MS + ME1));
    L = L + E * 0.000693 * Math.sin(MS - 2 * (MD - ME1));
    L = L + E * 0.000598 * Math.sin(2 * (ME1 - MF) - MS);
    L = L + 0.00055 * Math.sin(MD + 4 * ME1) + 0.000538 * Math.sin(4 * MD);
    L = L + E * 0.000521 * Math.sin(4 * ME1 - MS) + 0.000486 * Math.sin(2 * MD - ME1);
    L = L + E2 * 0.000717 * Math.sin(MD - 2 * MS);
	MM2 = ML + (L/RAD);
	MM = unwindRad(MM2);
	MM = MM * RAD;
	// ... und zurck zum Aufrufer ...
	// ... and back to the caller ...
	return(MM);
	
 }// End of moonLongitude()
 
 
 
 
 /* *********************************************************************************
 * Die Funktion berechnet das lokale Datum des Mond Aufgangs in
 * kompakter Form in einem Array. Siehe bei Returnwerte.
 *	
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0] ist der Mondaufgang in Dezimaler Form als String
 *		RET[1] ist der Status der Konvertierung GST zu UT als String
 *		RET[2] ist der Status des Mondaufganges als String
 *		RET[3 - 5] Tag, Monat und Jahr des Mondaufgangs
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the local date of the Moon rise in
 * compact form in an array. See at Return values.
 * 
 * Transfer value: 
 * 		day 		= The calculation day 
 * 		month 	= The calculation month 
 * 		year 	= The calculation year 
 * 		dst 		= Daylight saving time 
 * 		tz 		= Time zone 
 * 		glong 	= Geographical longitude of the location 
 * 		glat 		= Geographical latitude of the location 
 * 
 * Return value: 
 *		RET[0] is the moonrise in decimal form as string
 *		RET[1] the status of the conversion is GST to UT as string
 *		RET[2] the state of the moon rise is as string
 *		RET[3 - 5] Day, month and year of moon rise
  *************************************************************************************** */
 function moonriseLocalTime(day, month, year, dst, tz, glong, glat){
	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var DAY = parseInt(day);				// Tag, Monat, Jahr Berechnungsdatum
 	var MONTH = parseInt(month);		// Day, month, year Calculation date
	var YEAR = parseInt(year);
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var GLONG = Number(glong);		// Lngengrad des Orts - Longitude of the place
	var GLAT = Number(glat);				// Breitengrad des Orts - Latitude of the place
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GDAY = 0;						// Greenwich Tag, Monat, Jahr
	var GMONTH = 0;				// Greenwich day, month, year
	var GYEAR = 0;
	var JD = 0.0;							// Julianisches Datum 12:00 Uhr des Datums
	var DATE = [];						// Ergebnisarray der Umrechnung JD -> Datum
	var DATE1 = [];
	var LCT = 12.0;					// Returnwert local Time
	var UT = 0.0;							// Grennwich UT
	var eGSTUT =  "";				// String Grennwich UT mit Warnungsmeldung
	var arrUT = [];						// Grennwich UT Array
	var LU = 0.0;							// In sub6700() verwendet
	var K = 0;								// Zhlvariable
	var X = 0.0;							// Greenwich Sternzeit in der Iteration
	var G1 = 0.0;							// Hilfvariablen der Iteration
	var AU = 0.0;
	var LA = 0.0;
	var AA = 0.0;
	var GU = 0.0;
	var DY1 = 0;							// Lokales Datum aus UT
	var MN1 = 0;
	var YR1 = 0;
	var GDY = 0;							// Grennwich Datum aus lokalem Datum
	var GMN = 0;
	var GYR = 0;
	var eRS = "";							// Sichtbarkeitsstatus des Mondes als String
	var RET = [];							// Return array mit 5 Werten
	// Vorbelegung von Variablen aus Parametern
	// Pre-setting variables from parameters
	var DY1 = Number(DAY);				// Hilfsvariablen Tag, Monat Jahr
	var MN1 = Number(MONTH);		// Auxiliary variables day, month year
	var YR1 = Number(YEAR);
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... nun gehe zur Unterfunktion Mondparameter ...
	// ... now go to the subfunction Moon parameters ...
	sub6700();
	
	LA = LU;
	// Iterationslufe 8 x ...
	// Loop 8 times ...
	for (K = 1; K <= 8; K++){
		// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
		// Decimal Greenwich Determine stardate in hours from the local stardate . . .
		X = lokalST2GreenwichST(LA, 0, 0, GLONG);
		// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
		// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
		UT =  greenwichST2UT(X, 0, 0, GDY, GMN, GYR);
		// ... nur beim ersten mal ausfhren ...
		// . . . only run the first time . . .
        if (K == 1)
            G1 = UT;
		// ... und hier bei allen weiteren Durchlufen ...
		// . . . and here at all further runs . . .
        else
            G1 = GU;
        // ... GU + UT haben denselben Wert ...
		// . . . GU + UT have the same value . . .
        GU = UT;
        UT = GU;
		//... und damit ab zu sub6880() + sub6700() ...
		//. . . and thus from sub6880 ( ) + sub6700 () . . .
		sub6680();
		// ... und danach Variablenwerte tauschen und weiter ...
		// . . . and then swap variable values and continue . . .
		LA = LU; 
		AA = AU;
		
    }// End of K-Schleife (Loop)
	 
	AU = AA;
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GLONG);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	UT = greenwichST2UT(X, 0, 0, GDY, GMN, GYR);
	// Wenn bei der Berechnung der UT ein Fehler vorhanden ist,
	// einen Korrekturwert einrechnen ...
	// ... wenn in UT[0] "Warnung" steht ...
	// If there is an error in the calculation of the UT,
	// calculate a correction value . . .
	// . . . if UT[0] says Warning . . .
	if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GDY, GMN, GYR)) != "OK"){
		if (Math.abs(G1 - UT) > 0.5){
			// Korrekturwert zu UT hinzuaddieren ...
			// Add correction value to UT . . .
			UT = UT + 23.93447;
		}
	}	
	// Justiere den UT Tag, wenn es ntig ist auf einen Tag vorher oder spter ...
	// Adjust the UT tag if necessary to a day before or after . . .
	UT = adjustUTDay(UT, G1);
	// ... und aus der UT, Daylight saving, Zeitzone und Greenwich Datum die lokale Zeit berechnen ...
	// . . . and calculate the local time from the UT, daylight saving, time zone and Greenwich date . . .
	LCT = ut2LocalTime(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
	
	function sub6680(){
		// Wenn bei der Berechnung der UT ein Fehler vorhanden ist,
		// einen Korrekturwert einrechnen ...
		// ... wenn in UT[0] "Warnung" steht ...
		// If there is an error in the calculation of the UT,
		// calculate a correction value . . .
		// . . . if UT[0] says Warning . . .
		if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GDY, GMN, GYR)) != "OK"){
			if (Math.abs(G1 - UT) > 0.5){
				// Korrekturwert zu UT hinzuaddieren ...
				// Add correction value to UT . . .
				UT = UT + 23.93447;
			}
		}	
		// Justiere den UT Tag, wenn es ntig ist auf einen Tag vorher oder spter ...
		// Adjust the UT tag if necessary to a day before or after . . .
		UT = adjustUTDay(UT, G1);
		// ... und aus der UT, Daylight saving, Zeitzone und Greenwich Datum die lokale Zeit berechnen ...
		// . . . and calculate the local time from the UT, daylight saving, time zone and Greenwich date . . .
		LCT = ut2LocalTime(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		// Das lokale Datum Tag, Monat, Jahr aus dem UT, Daylight saving, Zeitzone und Greenwich Datum ermitteln ...
		// Determine the local date day, month, year from the UT, daylight saving, time zone and greenwich date . . .
		DATE1 = ut2LocalDate(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		// ... der lokale Tag ...
		// ... the local day ...
		DY1 = DATE1[0];
		// ... den lokalen Monat ...
		// ... the local month ...
		MN1 = DATE1[1];
		// ... das lokale Jahr ...
		// ... the local year ...
		YR1 = DATE1[2];
		// Das Greenwich Datum Tag, Monat, Jahr aus dem lokalen Datum Daylight saving, Zeitzone und lokalem Datum ermitteln ...
		// ... der Greenwich Tag ...
		// ... the Greenwich day ...
		GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... der Greenwich Monat ...
		// ... the Greenwich month ...
		GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... das Greenwich Jahr ...
		// ... the Greenwich year ...
		GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		UT = UT - 24.0 * Math.floor(UT / 24.0);
		// ... und zu sub6700() ...
		// ... call subroutine sub6700() ...
		sub6700();
		// ... und dann zum Aufrufer zurck ...
		// ... and then back to the caller ...
		return;
	}
	
	/* **************************************************************************************
		Unterfunktion berechnet die Mondparameter.
		Subfunction calculates the lunar parameters.
	************************************************************************************** */
	function sub6700(){
		// Berechne den Lngengrad des  Mondes ...
		// Get the longitude of the Moon ...
		var MM = moonLongitude( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		//console.log("MoonRiseLCT 6700 MM=" + MM +  " - LCT=" + LCT  + " - K=" + K);
		// Berechne den Breitengrad des  Mondes ...
		// Get the latitude of the Moon ...
		var BM = moonLatitude(  LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		//console.log("MoonRiseLCT 6700 Moon Latitude=" + BM);
		// Berechnet die Parallaxe des  Mondes ...
		// Get the parallax of the Moon ...
		var PM = moonHP( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1)/ RAD;
		//console.log("MoonRiseLCT 6700 Moon Parallaxe=" + PM);
		// Berechnet die Nutation der Lnge des  Mondes ...
		// Calculate the nutation of the length of the Moon ...
		var DP = nutationLong( GDY, GMN, GYR);
		//console.log("MoonRiseLCT 6700 Moon Nutation Longitude DP=" + DP);
		var TH = 0.27249 * Math.sin(PM); 
		var DI = TH + 0.0098902 - PM;
        //console.log ("MoonRiseLCT TH=" + TH + " - DI=" + DI);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		var P = eclipticCoords2EquatorialRA(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		// ... und dann die Grad in Stunden 
		P = P / 15.0;
		//console.log ("MoonRiseLCT P=" + P );		
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die Deklination in Grad ...
		var Q = eclipticCoords2EquatorialDec(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		//console.log ("MoonRiseLCT Q=" + Q );	
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		LU = riseLocalSiderealTime(P, 0, 0, Q, 0, 0, DI * RAD, GLAT);
		//console.log ("MoonRiseLCT 6700 LU=" + LU  + " - P=" + P + " - Q=" + Q + " - DI=" + DI * RAD + " - GLAT=" + GLAT);	
		// ... und dann die Sichtbarkeit des Mondes berprfen, ob er Auf- und Untergeht, niemals
		// zu sehen ist, oder immer zu sehen ist ...
		eRS = visibleStateMoonRiseSet(P, 0, 0, Q, 0, 0, (DI * RAD), GLAT);
		if( eRS != "OK"){
			LCT = -99;
			// Wenn Mond nie, oder immer sichtbar, dann beenden.
		}
		//console.log("End 6700 ---------------------------------------------------------------------");
		// ... von hier geht es wieder nach oben. Beim ersten Mal vor die Iteration,
		// danach immer ber sub6880() und sub6700().  Beide werden
		// immer hintereinander, aus der Iteration heraus, abgearbeitet ...
		return;
	}// End of sub6700()
	
	// RET[0] ist der Mondaufgang in Dezimaler Form als String ...
	// aus Rundungsgrnden werden 30s = 0.008333 hinzugerechnet ...
	// RET[0] is the moonrise in decimal form as string ...
	// for rounding reasons add 30s = 0. 008 333 . . .
	RET.push(String(LCT + 0.008333));
	// ... und in RET[1] eGSTUTist der Status der Konvertierung GST zu UT als String ...
	// . . . and in RET[1] eGSTUT the status of the conversion is GST to UT as string . . .
	RET.push(eGSTUT);
	// ... und in RET[2] eRS ist der Status des Mondaufganges als String ...
	// . . . and in RET[2] eRS the state of the moon rise is as string . . .
	RET.push(eRS);
	// ... und in RET[3-5] Tag, Monat und Jahr des Mondaufgangs ...
	// ... and in RET[3-5] Day, month and year of moon rise ...
	RET.push(DY1);
	RET.push(MN1);
	RET.push(YR1);
	//console.log("MoonRiseLCT Date=" + DY1 + "." + MN1 + "." + YR1)
	// .. und dann zurck zum Aufrufer.
	// ... and than back to the caller.
	return(RET);
	
 }// End of moonriseLocalTime()
 
 
 
 
  /* *********************************************************************************
 * Die Funktion berechnet das lokale Datum des Mond Untergangs in
 * kompakter Form in einem Array. Siehe bei Returnwerte.
 *
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0] ist der Mondaufgang in Dezimaler Form als String
 *		RET[1] ist der Status der Konvertierung GST zu UT als String
 *		RET[2] ist der Status des Mondaufganges als String
 *		RET[3 - 5] Tag, Monat und Jahr des Mondaufgangs
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the local date of the Moon set in
 * compact form in an array. See at Return values.
 * 
 * Transfer value: 
 * 		day 		= The calculation day 
 * 		month 	= The calculation month 
 * 		year 	= The calculation year 
 * 		dst 		= Daylight saving time 
 * 		tz 		= Time zone 
 * 		glong 	= Geographical longitude of the location 
 * 		glat 		= Geographical latitude of the location 
 * 
 * Return value: 
 *		RET[0] is the moonset in decimal form as string
 *		RET[1] the status of the conversion is GST to UT as string
 *		RET[2] the state of the moon rset is as string
 *		RET[3 - 5] Day, month and year of moon set
  *************************************************************************************** */
  function moonsetLocalTime(day, month, year, dst, tz, glong, glat){
  	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var DAY = parseInt(day);				// Tag, Monat, Jahr Berechnungsdatum
 	var MONTH = parseInt(month);		// Day, month, year Calculation date
	var YEAR = parseInt(year);
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var GLONG = Number(glong);		// Lngengrad des Orts - Longitude of the place
	var GLAT = Number(glat);				// Breitengrad des Orts - Latitude of the place
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GDAY = 0;						// Greenwich Tag, Monat, Jahr
	var GMONTH = 0;				// Greenwich day, month, year
	var GYEAR = 0;
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GDAY = 0;						// Greenwich Tag, Monat, Jahr
	var GMONTH = 0;				// Greenwich day, month, year
	var GYEAR = 0;
	var LCT = 12.0;					// Returnwert local Time
	var UT = 0.0;						// Grennwich UT
	var LU = 0.0;						// In sub6700() verwendet
	var K = 0;								// Zhlvariable
	var eGSTUT =  "";				// String Grennwich UT mit Warnungsmeldung
	var arrUT = [];						// Grennwich UT Array
	var LU = 0.0;						// In sub6700() verwendet
	var K = 0;								// Zhlvariable
	var X = 0.0;							// Greenwich Sternzeit in der Iteration
	var G1 = 0.0;						// Hilfvariablen der Iteration
	var AU = 0.0;
	var LA = 0.0;
	var AA = 0.0;
	var GU = 0.0;
	var DY1 = 0;							// Lokales Datum aus UT
	var MN1 = 0;
	var YR1 = 0;
	var GDY = 0;							// Grennwich Datum aus lokalem Datum
	var GMN = 0;
	var GYR = 0;
	var eRS = "";							// Sichtbarkeitsstatus des Mondes als String
	var RET = [];							// Return array mit 5 Werten
	
	// Vorbelegung von Variablen aus Parametern
	// Pre-setting variables from parameters
	var DY1 = Number(DAY);				// Hilfsvariablen Tag, Monat Jahr
	var MN1 = Number(MONTH);		// Auxiliary variables day, month year
	var YR1 = Number(YEAR);
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	//console.log("moonsetLocalTime Greenwichdatum: K="  + K  + " - "  + GDY + "." + GMN + "." + GYR );
	//console.log("moonriseLocalTime Berechnungsdatum: K=" + K + " - "  + DY1 + "." + MN1 + "." + YR1);
	// ... nun gehe zur Unterfunktion Mondparameter ...
	// ... now go to the subfunction Moon parameters ...
	//console.log("LCT vor sub6700():" + LCT);
	sub6700();
  
	LA = LU;
	//console.log ("moonriseLocalTime LA=" + LA);
	// Iterationslufe 8 x
	for (K = 1; K <= 8; K++){
		//console.log("K=" + K );
		// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
		X = lokalST2GreenwichST(LA, 0, 0, GLONG);
		//console.log("moonsetLocalTime X=" + X + " - LA=" + LA + " - K=" + K + " - Glong=" + GLONG);
		// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
		UT =  greenwichST2UT(X, 0, 0, GDY, GMN, GYR);
		//console.log("moonsetLocalTime UT=" + UT +  " - K=" + K + " - GDY=" + GDY + " - GMN=" + GMN + " - GYR=" + GYR);
		if (K==2) break;
        // ... nur beim ersten mal ausfhren ...
        if (K == 1)
            G1 = UT;
		// ... und hier bei allen weiteren Durchlufen ...
        else
            G1 = GU;
        // ... GU + UT haben denselben Wert ...
        GU = UT;
        UT = GU;
		//... und damit ab zu sub6880() + sub6700() ...
		sub6680();
		// ... und danach Variablenwerte tauschen und weiter ...
		LA = LU; 
		AA = AU;
		
    }// End of K-Schleife (Loop)
	AU = AA;
	//console.log("moonriseLocalTime nach Loop AU=" + AU);
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	X = lokalST2GreenwichST(LA, 0, 0, GLONG);
	//console.log("moonriseLocalTime nach Loop X=" + X + " - LA=" + LA + " - GLONG=" + GLONG);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	UT = greenwichST2UT(X, 0, 0, GDY, GMN, GYR);
	//console.log("moonriseLocalTime nach Loop UT=" + UT);
	
	// Wenn bei der Berechnung der UT ein Fehler vorhanden ist,
	// einen Korrekturwert einrechnen ...
	// ... wenn in UT[0] "Warnung" steht ...
	if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GDY, GMN, GYR)) != "OK"){
		if (Math.abs(G1 - UT) > 0.5){
			// Korrekturwert zu UT hinzuaddieren ...
			UT = UT + 23.93447;
		}
	}	
	// Justiere den UT Tag, wenn es ntig ist auf einen Tag vorher oder spter ...
	UT = adjustUTDay(UT, G1);
	//console.log("moonriseLocalTime nach Loop  adjustUT=" + UT);
	// ... und aus der UT, Daylight saving, Zeitzone und Greenwich Datum die lokale Zeit berechnen ...
	LCT = ut2LocalTime(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
	//console.log("moonriseLocalTime nach Loop LCT=" + LCT);
	
	function sub6680(){
		// Wenn bei der Berechnung der UT ein Fehler vorhanden ist,
		// einen Korrekturwert einrechnen ...
		// ... wenn in UT[0] "Warnung" steht ...
		if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GDY, GMN, GYR)) != "OK"){
			if (Math.abs(G1 - UT) > 0.5){
				// Korrekturwert zu UT hinzuaddieren ...
				UT = UT + 23.93447;
			}
		}
		//console.log("sub 6680 eGSTUT UT=" + UT + " - K=" + K);
		// Justiere den UT Tag, wenn es ntig ist auf einen Tag vorher oder spter ...
		UT = adjustUTDay(UT, G1);
		//console.log("sub6680 adjustUTDay UT=" + UT + " - K=" + K );
		// ... und aus der UT, Daylight saving, Zeitzone und Greenwich Datum die lokale Zeit berechnen ...
		LCT = ut2LocalTime(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		// Sichere die Lokalzeit in globaler Variablen fr die Abbruchbedingung der Iteration
		//console.log("sub6680 ut2LocalTime LCT=" + LCT + " - K=" + K);
		// Das lokale Datum Tag, Monat, Jahr aus dem UT, Daylight saving, Zeitzone und Greenwich Datum ermitteln ...
		DATE1 = ut2LocalDate(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		// ... der lokale Tag ...
		DY1 = DATE1[0];
		// ... den lokalen Monat ...
		MN1 = DATE1[1];
		// ... das lokale Jahr ...
		YR1 = DATE1[2];
		//console.log("sub6680 ut2LocalDate DY1=" + DY1 + " - " + MN1 + " - " + YR1);
		// Das Greenwich Datum Tag, Monat, Jahr aus dem lokalen Datum Daylight saving, Zeitzone und lokalem Datum ermitteln ...
		// ... der Greenwich Tag ...
		GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... der Greenwich Monat ...
		GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... das Greenwich Jahr ...
		GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		//console.log("sub6680 Greenwich GDY=" + GDY + " - " + GMN + " - " + GYR);
		UT = UT - 24.0 * Math.floor(UT / 24.0);
		//console.log("sub6680 UT=" + UT);
		sub6700();
		return;
	}
	
	/* **************************************************************************************
		Unterfunktion berechnet die Mondparameter.
		Subfunction calculates the lunar parameters.
	************************************************************************************** */
	function sub6700(){
		// Berechne den Lngengrad des  Mondes ...
		// Get the longitude of the Moon ...
		var MM = moonLongitude( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		// Berechne den Breitengrad des  Mondes ...
		// Get the latitude of the Moon ...
		var BM = moonLatitude(  LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		//console.log("MoonRiseLCT 6700 Moon Latitude=" + BM);
		// Berechnet die Parallaxe des  Mondes ...
		// Get the parallax of the Moon ...
		var PM = moonHP( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1)/ RAD;
		//console.log("MoonRiseLCT 6700 Moon Parallaxe=" + PM);
		// Berechnet die Nutation der Lnge des  Mondes ...
		// Calculate the nutation of the length of the Moon ...
		var DP = nutationLong( GDY, GMN, GYR);
		//console.log("MoonRiseLCT 6700 Moon Nutation Longitude DP=" + DP);
		var TH = 0.27249 * Math.sin(PM); 
		var DI = TH + 0.0098902 - PM;
        //console.log ("MoonRiseLCT TH=" + TH + " - DI=" + DI);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		var P = eclipticCoords2EquatorialRA(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		// ... und dann die Grad in Stunden 
		P = P / 15.0;
		//console.log ("MoonRiseLCT P=" + P );		
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die Deklination in Grad ...
		var Q = eclipticCoords2EquatorialDec(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		//console.log ("MoonRiseLCT Q=" + Q );	
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		LU = setLocalSiderealTime(P, 0, 0, Q, 0, 0, DI * RAD, GLAT);
		//console.log ("MoonRiseLCT 6700 LU=" + LU  + " - P=" + P + " - Q=" + Q + " - DI=" + DI * RAD + " - GLAT=" + GLAT);	
		// ... und dann die Sichtbarkeit des Mondes berprfen, ob er Auf- und Untergeht, niemals
		// zu sehen ist, oder immer zu sehen ist ...
		eRS = visibleStateMoonRiseSet(P, 0, 0, Q, 0, 0, (DI * RAD), GLAT);
		if( eRS != "OK"){
			LCT = -99;
			// Wenn Mond nie, oder immer sichtbar, dann beenden.
		}
		//console.log("End 6700 ---------------------------------------------------------------------");
		// ... von hier geht es wieder nach oben. Beim ersten Mal vor die Iteration,
		// danach immer ber sub6880() und sub6700().  Beide werden
		// immer hintereinander, aus der Iteration heraus, abgearbeitet ...
		return;
	}// End of sub6700()
  
	// RET[0] ist der Mondaufgang in Dezimaler Form als String ...
	// aus Rundungsgrnden werden 30s = 0.008333 hinzugerechnet ...
	// RET[0] is the moonrise in decimal form as string ...
	// for rounding reasons add 30s = 0. 008 333 . . .
	RET.push(String(LCT + 0.008333));
	// ... und in RET[1] eGSTUTist der Status der Konvertierung GST zu UT als String ...
	// . . . and in RET[1] eGSTUT the status of the conversion is GST to UT as string . . .
	RET.push(eGSTUT);
	// ... und in RET[2] eRS ist der Status des Mondaufganges als String ...
	// . . . and in RET[2] eRS the state of the moon rise is as string . . .
	RET.push(eRS);
	// ... und in RET[3-5] Tag, Monat und Jahr des Mondaufgangs ...
	// ... and in RET[3-5] Day, month and year of moon rise ...
	RET.push(DY1);
	RET.push(MN1);
	RET.push(YR1);
	// .. und dann zurck zum Aufrufer.
	// ... and than back to the caller.
	return(RET);
	
  }// End of moonsetLocalTime()
 
 
 

 /* *********************************************************************************
 * Die Funktion berechnet Berechnet den Azimuth und die Hhe des 
 *	Mondaufganges in dezimalen Grad.
 *
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0] ist der Azimut Mondaufgang in dezimalen Grad
 *		RET[1] ist die Hhe des Mondaufgangs als , ', "
 *		RET[2] ist die Hhe des Mondaufgangs in dezimalen Gard
 *	---------------------------------------------------------------------------------------------------
	* The function calculates the azimuth and the height of
	* Moon rise in decimal degrees.
	*
	* Transfer value:
	* 	day = The day of calculation
	* 	month = The month of calculation
	* 	year = The year of calculation
	* 	dst = Daylight saving time
	* 	tz = Time zone
	* 	glong = Geographical longitude of the place
	* 	glat = Geographical latitude of the place
	*
	* Return value:
	* 	RET[0] is the azimuth moonrise in decimal degrees
	* 	RET[1] is the altitude of the moon rise as , ', 
	* 	RET[2] is the height of the moon rise in decimal degrees
  *************************************************************************************** */
   function moonRiseAz(day, month, year, dst, tz, glong, glat){
  	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var DAY = parseInt(day);				// Tag, Monat, Jahr Berechnungsdatum
 	var MONTH = parseInt(month);		// Day, month, year Calculation date
	var YEAR = parseInt(year);
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var GLONG = Number(glong);		// Lngengrad des Orts - Longitude of the place
	var GLAT = Number(glat);				// Breitengrad des Orts - Latitude of the place
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GDAY = 0;										// Greenwich Tag, Monat, Jahr
	var GMONTH = 0;								// Greenwich day, month, year
	var GYEAR = 0;
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GDAY = 0;										// Greenwich Tag, Monat, Jahr
	var GMONTH = 0;								// Greenwich day, month, year
	var GYEAR = 0;
	var LCT = 12.0;									// Returnwert local Time
	var UT = 0.0;											// Grennwich UT
	var LU = 0.0;											// In sub6700() verwendet
	var K = 0;												// Zhlvariable
	var eGSTUT =  "";								// String Grennwich UT mit Warnungsmeldung
	var arrUT = [];										// Grennwich UT Array
	var LU = 0.0;											// In sub6700() verwendet
	var K = 0;												// Zhlvariable
	var X = 0.0;											// Greenwich Sternzeit in der Iteration
	var G1 = 0.0;											// Hilfvariablen der Iteration
	var AU = 0.0;
	var LA = 0.0;
	var AA = 0.0;
	var GU = 0.0;
	var DY1 = 0;											// Lokales Datum aus UT
	var MN1 = 0;
	var YR1 = 0;
	var GDY = 0;											// Grennwich Datum aus lokalem Datum
	var GMN = 0;
	var GYR = 0;
	var Q = 0.0;
	var P = 0.0;
	var eRS = "";											// Sichtbarkeitsstatus des Mondes als String
	var RET = new Array();					// Return array mit 2 Werten
	
	// Vorbelegung von Variablen aus Parametern
	// Pre-setting variables from parameters
	var DY1 = Number(DAY);				// Hilfsvariablen Tag, Monat Jahr
	var MN1 = Number(MONTH);		// Auxiliary variables day, month year
	var YR1 = Number(YEAR);
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	//console.log("moonsetLocalTime Greenwichdatum: K="  + K  + " - "  + GDY + "." + GMN + "." + GYR );
	//console.log("moonriseLocalTime Berechnungsdatum: K=" + K + " - "  + DY1 + "." + MN1 + "." + YR1);
	// ... nun gehe zur Unterfunktion Mondparameter ...
	// ... now go to the subfunction Moon parameters ...
	//console.log("LCT vor sub6700():" + LCT);
	sub6700();
  
	LA = LU;
	//console.log ("moonriseLocalTime LA=" + LA);
	// Iterationslufe 8 x
	for (K = 1; K <= 8; K++){
		//console.log("moonRiseAz K=" + K );
		// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
		X = lokalST2GreenwichST(LA, 0, 0, GLONG);
		//console.log("moonRiseAz X=" + X + " - LA=" + LA + " - K=" + K);
		// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
		UT =  greenwichST2UT(X, 0, 0, GDY, GMN, GYR);
		//console.log("moonRiseAz UT=" + UT + " - GDAY=" + GDAY + " - GMONTH=" + GMONTH + " - GYEAR=" + GYEAR);
        // ... nur beim ersten mal ausfhren ...
        if (K == 1)
            G1 = UT;
		// ... und hier bei allen weiteren Durchlufen ...
        else
            G1 = GU;
        // ... GU + UT haben denselben Wert ...
        GU = UT;
        UT = GU;
		//... und damit ab zu sub6880() + sub6700() ...
		sub6680();
		// ... und danach Variablenwerte tauschen und weiter ...
		LA = LU; 
		AA = AU;
		
    }// End of K-Schleife (Loop)
	AU = AA;
	
	function sub6680(){
		// Wenn bei der Berechnung der UT ein Fehler vorhanden ist,
		// einen Korrekturwert einrechnen ...
		// ... wenn in UT[0] "Warnung" steht ...
		if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GDY, GMN, GYR)) != "OK"){
			if (Math.abs(G1 - UT) > 0.5){
				// Korrekturwert zu UT hinzuaddieren ...
				UT = UT + 23.93447;
			}
		}	
		// Justiere den UT Tag, wenn es ntig ist auf einen Tag vorher oder spter ...
		UT = adjustUTDay(UT, G1);
		//console.log("moonRiseAz sub6680 UT=" + UT + " - K=" + K + " - G1=" + G1);
		// ... und aus der UT, Daylight saving, Zeitzone und Greenwich Datum die lokale Zeit berechnen ...
		LCT = ut2LocalTime(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		//console.log("sub6680 moonRiseAz LCT=" + LCT + " - K=" + K);
		// Das lokale Datum Tag, Monat, Jahr aus dem UT, Daylight saving, Zeitzone und Greenwich Datum ermitteln ...
		DATE1 = ut2LocalDate(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		// ... der lokale Tag ...
		DY1 = DATE1[0];
		// ... den lokalen Monat ...
		MN1 = DATE1[1];
		// ... das lokale Jahr ...
		YR1 = DATE1[2];
		//console.log("sub6680 moonRiseAz lokal DY1=" + DY1 + " - " + MN1 + " - " + YR1);
		// Das Greenwich Datum Tag, Monat, Jahr aus dem lokalen Datum Daylight saving, Zeitzone und lokalem Datum ermitteln ...
		// ... der Greenwich Tag ...
		GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... der Greenwich Monat ...
		GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... das Greenwich Jahr ...
		GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		//console.log("sub6680 moonRiseAz Greenwich GDY=" + GDY + " - " + GMN + " - " + GYR);
		UT = UT - 24.0 * Math.floor(UT / 24.0);
		//console.log("sub6680 moonRiseAz UT=" + UT);
		sub6700();
		return;
	}
	
	/* **************************************************************************************
		Unterfunktion berechnet die Mondparameter.
		Subfunction calculates the lunar parameters.
	************************************************************************************** */
	function sub6700(){
		// Berechne den Lngengrad des  Mondes ...
		// Get the longitude of the Moon ...
		var MM = moonLongitude( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		//console.log("moonRiseAz 6700 MM=" + MM +  " - LCT=" + LCT  + " - K=" + K);
		// Berechne den Breitengrad des  Mondes ...
		// Get the latitude of the Moon ...
		var BM = moonLatitude(  LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		//console.log("moonRiseAz 6700 Moon Latitude=" + BM);
		// Berechnet die horizontale Parallaxe des  Mondes ...
		// Get the horizontal parallax of the Moon ...
		var PM = moonHP( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1)/ RAD;
		//console.log("moonRiseAz 6700 Moon Parallaxe=" + PM);
		// Berechnet die Nutation der Lnge des  Mondes ...
		// Calculate the nutation of the length of the Moon ...
		var DP = nutationLong( GDY, GMN, GYR);
		//console.log("MoonRiseLCT 6700 Moon Nutation Longitude DP=" + DP);
		var TH = 0.27249 * Math.sin(PM); 
		var DI = TH + 0.0098902 - PM;
        //console.log ("moonRiseAz TH=" + TH + " - DI=" + DI);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		P = eclipticCoords2EquatorialRA(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		// ... und dann die Grad in Stunden 
		P = P / 15.0;
		//console.log ("moonRiseAz P=" + P );		
		// ... und dann konvertiere ekliptische Koordinaten zu dezimalen quatorialen Koordinaten 
		// fr die Deklination in Grad ...
		Q = eclipticCoords2EquatorialDec(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		//console.log ("moonRiseAz Q=" + Q + " - K=" + K );	
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		LU = riseLocalSiderealTimeAZ(P, 0, 0, Q, 0, 0, DI * RAD, GLAT);
		//console.log ("moonRiseAz 6700 LU=" + LU  + " - P=" + P + " - Q=" + Q + " - DI=" + DI * RAD + " - GLAT=" + GLAT);	
		// Den Azimuth der RA des Untergangs in dezimalen Grad berechnen ...
		AU = riseLocalSiderealDegreesAZ(P, 0, 0, Q, 0, 0, DI * RAD, GLAT);
		//console.log("moonRiseAz AU=" + AU + " - K=" + K);
		//console.log ("moonRiseAz 6700 AU=" + AU);	
		//console.log("End 6700 ---------------------------------------------------------------------");
		// ... von hier geht es wieder nach oben. Beim ersten Mal vor die Iteration,
		// danach immer ber sub6880() und sub6700().  Beide werden
		// immer hintereinander, aus der Iteration heraus, abgearbeitet ...
		return;
	}// End of sub6700()
	// Berechne aus Stundenwinkel, Deklination und Breitengrad die Hhe eines
	// Der Azimuth in dezimalen Grad
	RET.push(AU);
	// Objekts (Mond ... Sterne ....) ber dem Horizont ...
	// Die Hhe ber dem Horizont in , ', ''
	//console.log("objectHeight=" +decDegrees2AngleOfHours( objectHeight(AU ,  Q, GLAT)) + " - convertEquatorial2Horizontal=" + convertEquatorial2Horizontal(AU ,  Q, GLAT));
	RET.push(decDegrees2AngleOfHours(objectHeight(AU ,  Q, GLAT)));
	// Die Hhe (Deklination)ber dem Horizont in dezimalen Grad ...
	RET.push(objectHeight(AU ,  Q, GLAT));
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(RET);
	
  }// End of moonRiseAz()
 
 
 

   /* *********************************************************************************
 * Die Funktion berechnet Berechnet den Azimuth und die Hhe des 
 *	Mondunterganges in dezimalen Grad.
 *
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0] ist der Azimut Monduntergang in dezimalen Grad
 *		RET[1] ist die Hhe des Monduntergangs als , ', "
 *		RET[2] ist die Hhe des Monduntergangs in dezimalen Gard
 *	---------------------------------------------------------------------------------------------------
	* The function calculates the azimuth and the height of
	* Moon set in decimal degrees.
	*
	* Transfer value:
	* 	day = The day of calculation
	* 	month = The month of calculation
	* 	year = The year of calculation
	* 	dst = Daylight saving time
	* 	tz = Time zone
	* 	glong = Geographical longitude of the place
	* 	glat = Geographical latitude of the place
	*
	* Return value:
	* 	RET[0] is the azimuth moon set in decimal degrees
	* 	RET[1] is the altitude of the moon set as , ', 
	* 	RET[2] is the height of the moon set in decimal degrees
  *************************************************************************************** */
   function moonSetAz(day, month, year, dst, tz, glong, glat){
  	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var DAY = parseInt(day);				// Tag, Monat, Jahr Berechnungsdatum
 	var MONTH = parseInt(month);		// Day, month, year Calculation date
	var YEAR = parseInt(year);
	var DST = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var GLONG = Number(glong);		// Lngengrad des Orts - Longitude of the place
	var GLAT = Number(glat);				// Breitengrad des Orts - Latitude of the place
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GDAY = 0;						// Greenwich Tag, Monat, Jahr
	var GMONTH = 0;				// Greenwich day, month, year
	var GYEAR = 0;
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GDAY = 0;						// Greenwich Tag, Monat, Jahr
	var GMONTH = 0;				// Greenwich day, month, year
	var GYEAR = 0;
	var LCT = 12.0;					// Lokale zeit - local Time 
	var UT = 0.0;							// Grennwich UT - Greenwich UT
	var LU = 0.0;							// In sub6700() verwendet - use it in sub6700()
	var K = 0;								// Zhlvariable - counter variable
	var eGSTUT =  "";				// String Grennwich UT mit Warnungsmeldung
													// String Grennwich UT with warning
	var arrUT = [];						// Grennwich UT Array	- Greenwich UT array
	var LU = 0.0;							// In sub6700() verwendet - use it in sub6700()
	var K = 0;								// Zhlvariable - counter variable
	var X = 0.0;							// Greenwich Sternzeit in der Iteration
													// Greenwich star date in the loop
	var G1 = 0.0;							// Hilfvariablen der Iteration
	var AU = 0.0;							// Auxilary variables for loop
	var LA = 0.0;
	var AA = 0.0;
	var GU = 0.0;
	var DY1 = 0;							// Lokales Datum aus UT
	var MN1 = 0;							// Local date from UT
	var YR1 = 0;
	var GDY = 0;							// Grennwich Datum aus lokalem Datum
	var GMN = 0;						// Greenwich date from local date
	var GYR = 0;
	var Q = 0.0;
	var P = 0.0;
	var eRS = "";							// Sichtbarkeitsstatus des Mondes als String
													// Visibility status of the Moon as string
	var RET = new Array();	// Return array mit 3 Werten
													// Return array with 3 values
	// Vorbelegung von Variablen aus Parametern
	// Pre-setting variables from parameters
	var DY1 = Number(DAY);				// Hilfsvariablen Tag, Monat Jahr
	var MN1 = Number(MONTH);		// Auxiliary variables day, month year
	var YR1 = Number(YEAR);
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
	// ... nun gehe zur Unterfunktion Mondparameter ...
	// ... now go to the subfunction Moon parameters ...
	sub6700();
	LA = LU;
	// Iterationslufe 8 x
	// Iteration runs 8 x
	for (K = 1; K <= 8; K++){
		// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
		// Determine decimal Greenwich stardate in hours from the local stardate . . .
		X = lokalST2GreenwichST(LA, 0, 0, GLONG);
		// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
		// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
		UT =  greenwichST2UT(X, 0, 0, GDY, GMN, GYR);
        // ... nur beim ersten mal ausfhren ...
		// . . . only run the first time . . .
        if (K == 1)
            G1 = UT;
		// ... und hier bei allen weiteren Durchlufen ...
		// . . . and here at all further runs . . .
        else
            G1 = GU;
        // ... GU + UT haben denselben Wert ...
		// . . . GU + UT have the same value . . .
        GU = UT;
        UT = GU;
		//... und damit ab zu sub6880() + sub6700() ...
		//. . . and thus from sub6880 ( ) + sub6700 () . . .
		sub6680();
		// ... und danach Variablenwerte tauschen und weiter ...
		// . . . and then swap variable values and continue . . .
		LA = LU; 
		AA = AU;
		
    }// End of K-Schleife (Loop)
	AU = AA;
	
	function sub6680(){
		// Wenn bei der Berechnung der UT ein Fehler vorhanden ist,
		// einen Korrekturwert einrechnen ...
		// ... wenn in UT[0] "Warnung" steht ...
		// If there is an error in the calculation of the UT,
		// calculate a correction value . . .
		// . . . if UT[0] says Warning . . .
		if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GDY, GMN, GYR)) != "OK"){
			if (Math.abs(G1 - UT) > 0.5){
				// Korrekturwert zu UT hinzuaddieren ...
				// Add correction value to UT . . .
				UT = UT + 23.93447;
			}
		}	
		// Justiere den UT Tag, wenn es ntig ist auf einen Tag vorher oder spter ...
		// Adjust the UT tag if necessary to a day before or after . . .
		UT = adjustUTDay(UT, G1);
		// ... und aus der UT, Daylight saving, Zeitzone und Greenwich Datum die lokale Zeit berechnen ...
		// . . . and calculate the local time from the UT, daylight saving, time zone and Greenwich date . . .
		LCT = ut2LocalTime(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		// Das lokale Datum Tag, Monat, Jahr aus dem UT, Daylight saving, Zeitzone und Greenwich Datum ermitteln ...
		// Determine the local date day, month, year from the UT, daylight saving, time zone and greenwich date . . .
		DATE1 = ut2LocalDate(UT, 0, 0, DST,  TZ, GDY, GMN, GYR);
		// ... der lokale Tag ...
		// ... the local day ...
		DY1 = DATE1[0];
		// ... den lokalen Monat ...
		// ... the local month ...
		MN1 = DATE1[1];
		// ... das lokale Jahr ...
		// ... the local year ...
		YR1 = DATE1[2];
		// Das Greenwich Datum Tag, Monat, Jahr aus dem lokalen Datum Daylight saving, Zeitzone und lokalem Datum ermitteln ...
		// ... der Greenwich Tag ...
		// Determine the Greenwich date day, month, year from the local date daylight saving, time zone and local date . . .
		// . . . the Greenwich day . . .
		GDY = localtime2GreenwichDay(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... der Greenwich Monat ...
		// ... the Greenwich month ...
		GMN = localtime2GreenwichMonth(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... das Greenwich Jahr ...
		// ... the Greenwich year ...
		GYR = localtime2GreenwichYear(LCT, 0, 0,  DST, TZ, DY1, MN1, YR1);
		// ... und UT berechnen ...
		// ... and calculate UT ...
		UT = UT - 24.0 * Math.floor(UT / 24.0);
		// ... und zu sub6700() springen ...
		// ... and jump to sub6700() ...
		sub6700();
		// ...  und zurck zum Aufrufer.
		// ... and bach to the caller.
		return;
	}
	/* **************************************************************************************
		Unterfunktion berechnet die Mondparameter.
		Subfunction calculates the lunar parameters.
	************************************************************************************** */
	function sub6700(){
		// Berechne den Lngengrad des  Mondes ...
		// Get the longitude of the Moon ...
		var MM = moonLongitude( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		// Berechne den Breitengrad des  Mondes ...
		// Get the latitude of the Moon ...
		var BM = moonLatitude(  LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1);
		// Berechnet die horizontale Parallaxe des  Mondes ...
		// Get the horizontal parallax of the Moon ...
		var PM = moonHP( LCT ,0 ,0 , DST, TZ, DY1, MN1, YR1)/ RAD;
		// Berechnet die Nutation der Lnge des  Mondes ...
		// Calculate the nutation of the length of the Moon ...
		var DP = nutationLong( GDY, GMN, GYR);
		var TH = 0.27249 * Math.sin(PM); 
		var DI = TH + 0.0098902 - PM;
		// ... und dann konvertiere ekliptische Koordinaten zu dezimalen quatorialen Koordinaten 
		// fr die RA in Grad ...
		// . . . and then convert ecliptic coordinates to decimal equatorial coordinates
		// for the RA in degrees . . .
		P = eclipticCoords2EquatorialRA(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		// ... und dann die dezimalen Grad  der RA in Stunden umrechnen ...
		// . . . and then convert the decimal degrees of RA into hours . . .
		P = P / 15.0;
		// ... und dann konvertiere ekliptische Koordinaten zu dezimalen quatorialen Koordinaten 
		// fr die Deklination in Grad ...
		// . . . and then convert ecliptic coordinates to decimal equatorial coordinates
		// for the declination in degrees . . .
		Q = eclipticCoords2EquatorialDec(MM + DP, 0, 0, BM, 0, 0, GDY, GMN, GYR);
		// Die lokale siderische Zeit der RA fr den Azimuth des Untergangs berechnen ...
		// Calculate the local sidereal time of RA for the azimuth of sinking . . .
		LU = setLocalSiderealTimeAZ(P, 0, 0, Q, 0, 0, DI * RAD, GLAT);
		// Den Azimuth der RA des Untergangs in dezimalen Grad berechnen ...
		// Calculate the azimuth of the RA of the sinking in decimal degrees . . .
		AU = setLocalSiderealDegreesAZ(P, 0, 0, Q, 0, 0, DI * RAD, GLAT);
		// ... von hier geht es wieder nach oben. Beim ersten Mal vor die Iteration,
		// danach immer ber sub6880() und sub6700().  Beide werden
		// immer hintereinander, aus der Iteration heraus, abgearbeitet ...
		// . . . From here it goes up again. The first time before the iteration,
		// then always via sub6880 () and sub6700 (). Both will be
		// always one after the other, out of the iteration, processed . . .
		return;
	}// End of sub6700()
	// Der Azimut (RA) in dezimalen Grad
	// The azimuth (RA) in decimal degrees
	RET.push(AU);
	// Objekts (Mond ... Sterne ....) ber dem Horizont ...
	// Die Hhe (Deklination)ber dem Horizont in , ', '' ...
	// Object (Moon . . . Stars . . . . ) above the horizon . . .
	// The height (declination) above the horizon in , ',  . . .
	RET.push(decDegrees2AngleOfHours(objectHeight(AU ,  Q, GLAT)));
	// Die Hhe (Deklination)ber dem Horizont in dezimalen Grad ...
	// The height (declination) above the horizon in decimal degrees . . .
	RET.push(objectHeight(AU ,  Q, GLAT));
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(RET);
	
  }// End of moonSetAz()
  
 


 
 
 
 
 
 
 
 
 
 
 
 
 
 
 