/* ************************************************************************************
 *	Das Modul beinhaltet Funktionen zur Berechnung der Sternzeit.
 *	Die Funktionen und Algorithmen haben keinen Anspruch auf
 *	hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module contains functions for calculation and sidereal time.
 * The functions and algorithms do not claim 
 * high accuracy, completeness and correctness.
 *
 *	Funktionsliste:
 *
 *  - localDate()							- Berechnet die lokale reale Uhrzeit
 * 												  Calculates the local real time	  
 *	- lengthMeanEquinox()			- Berechnet die Differenz zwischen mittleren-
 *												  und wahren Frhlingspunkt
 * 												  Calculates the difference between mean-
 *												  and true springtime point
 * - meanGreenwichStardate()	- Mittlere Greenwich ST (GMST)
 *												  Mean Greenwich sidereal time (GMST)
 * - meanEclipticInclination()		- Berechnet Ekliptische Schiefe
 *												   Calculates the ecliptic obliquity
 *
 *
 *	12.12.2020, Dieter Steffan
 *
 *********************************************************************************** */	


function greenwichST2LocalST(ghour, gmin, gsec, glong){
	var GSTLST = 0.0;					// Lokale Sternzeit als Returnwert
	var GHOUR = Number(ghour);	// Greenwich Stunden
	var GMIN		= Number(gmin);	// Greenwich Minute
	var GSEC	= Number(gsec);	// Greenwich Sekunde
	var GLONG = Number(glong);	// Geografischer Lngenbrad des Beobachters
	
	var A = time2Dectime(GHOUR, GMIN, GSEC, 0);
	var B = GLONG / 15.0;
	var C = A + B;
	GSTLST = C - (24.0 * parseInt(C / 24.0));
	
	return(GSTLST);
	
}// End of greenwichST2LocalST()
 
					
/* *********************************************************************************
 * Die Funktion berechnet aus der geografischen Lnge, also dem Ort, an dem
 * man sich gerade befindet, und der Universal Time (UT) + der Daylight
 * saving time (Sommer/Winterzeit) die reale Uhrzeit an dem Ort. Dies ist dann
 * nicht die Uhrzeit der Zeitzone, sondern ein Wert innerhalb der Zeitzone.
 *	Oliver Montenbruck - Ephemeridenrechnung
 *
 *	bergabewert:
 * 		lamda 		= Geografischer Lngengrad dezimal als Number (Ost +, West -)
 * 		st				= UT dezimal + DST als Number
 *	
 *	Returnwert:
 *		LT			= 	Die lokale Zeit als Dezimalwert und Number
 *	---------------------------------------------------------------------------------------------------
 * The function is calculated from the geographical longitude, i. e. the place where
 * you are at the moment, and the Universal Time (UT) + the Daylight
 * saving time (summer/winter time) is the real time at the location. This is then
 * not the time of the time zone, but a value within the time zone. 
 * 	Oliver Montenbruck - Ephemeridenrechnung
 *
 * Transfer value:
 *
 * 		lamda 	= Geographical longitude decimal as number (East +, West -)
 * 		st 		= UT decimal + DST as number
 *
 * Return value:
 * 		LT 		= The local time as decimal value and number
  *************************************************************************************** */				
function localDate(lambda, st){
	var ST = Number(st);							// Die  Universal Time inklusive DST
																// Universal time include daylight saving
	var LT = 0.0;										// Returnwert
																// return value
	var LAMBDA = Number(lambda);			// Die geografische Lnge des Ortes
																// The geographical length of the place
	// Berechne Lokale Zeit und speichere sie in LT ...
	// Calculate local time and save it in LT. ..
	LT = ST + (LAMBDA *(1.0/15.0));
	// Ist die Zeit grer gleich 24.0, dann sind wir ber 00:00:00.000 hinaus
	// und mssen die 24 entfernen und durch 00 ersetzen. Somit elemenieren
	// wir die 24 Uhr ...
	//
	// If the time is greater than 24. 0, then we are beyond 00:00:00. 000
	// and must remove the 24 and replace it with 00. Thus elemenate
	// we the 24 oclock . . .
	if (LT >= 24.0) {
		// ... indem wird 0.0 + den dezimalen Nachkommaanteil berechnen ...
		//. . . by calculating 0. 0 + the decimal decimal percentage. . .
		LT = 0.0 + (LT - Math.floor(LT));
	}
	// ... und an den Aufrufer zurck.
	// ... and back to the caller.
	return(LT);
	
}// End of localDate()
					



/* *********************************************************************************
 * Die Funktion berechnet die Differenz des mittleren Frhlingspunktes 
 * bezogen auf den wahren Frhlingspunkt.
 *	Oliver Montenbruck - Ephemeridenrechnung	
 *
 *	bergabewert:
 * 		t 					= Teil des vergangenen Jahrhunderts als Dezimalwert
 *							   und Number
 *	
 *	Returnwert:
 *		deltaPsi		= 	Die Differenz der Frhlingspunkte als Dezimalwert
 *							    und Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the difference of the mean vernal equinox 
 * with respect to the true vernal equinox. 
 * 	Oliver Montenbruck - Ephemeridenrechnung
 *
 * Transfer value: 
 * 		t 					= Part of the past century as decimal value and number 
 * 
 * Return value: 
 * 		deltaPsi 		= The difference of the vernal equinoxes as decimal value 
 *							   and number
  *************************************************************************************** */		
function lengthMeanEquinox(t){
	var T = Number(t);													// Teil des Jahrhunderts seit 1.1.2000 12:00 Uhr
																					// Part of the century since 1. 1. 2000 12:00
	var L = Number(357.525 + 35999.049 * T);				// Mittlere Anomalie der Sonne	
																					// Medium anomaly of the Sun
	var F = Number(93.273 + 483202.019 * T);				// Mittlerer Knotenabstand des Mondes
																					// Mean nodal distance of the moon
	var D = Number(297.850 + 445267.111 * T);				// Mittlerer Abstand Mond - Sonne
																					// Medium distance Moon  Sun
	var Omega = Number(125.045 - (1934.136 * T));		// Mittlere Knotenlnge des Mondes
																					// Mean node length of the moon
	// Egebnis an den Aufrufer zurck.
	// Solution back to the caller.
	return( (-17.200 * Math.sin(Omega/RAD)) +  ( 0.206 * Math.sin((2 * Omega) / RAD )) - 
				( 1.319 * Math.sin ( (2 * (F-D+Omega))/RAD ) )+
				(0.143 * Math.sin (L/RAD) ) - ( 0.227 * Math.sin ((2 * (F+Omega))/RAD) )  );
				
}// End of lengthMeanEquinox()
					
					

					
/* *********************************************************************************
 * Die Funktion berechnet die mittlere Greenwich Sternzeit  in Stunden 
 * als Dezimalwert.
 *	Oliver Montenbruck - Ephemeridenrechnung
 *
 *	bergabewert:
 * 		ut 					= Universal Time (UT) als Dezimalwert
 *							       und Number
 *		jd0					= Julianisches Datum der Epoche ab 1.1. 12:00 Uhr
 *	
 *	Returnwert:
 *		theta					= 	Die mittlere Greenwich Sternzeit als Dezimalwert
 *							        und Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the Greenwich Mean Stardate in hours 
 * as a decimal value. 
 * 	Oliver Montenbruck - Ephemeridenrechnung
 *
 * Transfer value: 
 * 		ut 					= Universal Time (UT) as decimal value 
 * 								   and Number 
 * 		jd0 					= Julian date of the epoch from 1. 1. 12:00 o,clock
 * 
 * Return value: 
 * 		theta 				= The Greenwich Mean Stardate as decimal value 
 * 								   and Number
  *************************************************************************************** */		
function meanGreenwichStardate(ut,  jd0){
	var UT = Number(ut);			// UT als Number - Universal time as Number
	var JD0 = Number(jd0);		// Julianisches Datum der Epoche
												// Julian date of the epoch
	// Ergebnis zurck zum Aufrufer.
	// Solution back to the caller.
	return(6.664520 + 0.0657098244 * (JD0 - 2451544.5) + (1.0027379093 * UT));
	
}// End of meanGreenwichStardate()




/* *********************************************************************************
 * Die Funktion berechnet die ekliptische Schiefe in dezimalen Grad.
 * Oliver Montenbruck - Ephemeridenrechnung
 *
 *	bergabewert:
 * 		t 					= Vergangener Teil des Jahrhunderts der Epoche als
 *							   Dezimalwert und Number
 *	
 *	Returnwert:
 *		epsilon			= 	Die ekliptische Schiefe als Dezimalwert
 *							    und Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the ecliptic skew in decimal degrees. 
 * 	Oliver Montenbruck - Ephemeridenrechnung
 *
 * Transfer value: 
 * 		t 					= Past part of the century of the epoch as 
 * 							   Decimal value and Number 
 * 
 * Return value: 
 * 		epsilon 			= The ecliptic obliquity as decimal value 
 * 							   and Number
  *************************************************************************************** */		
function meanEclipticInclination(t){
	var T = Number(t);			// Teil des vergangenen Jahrhunderts als Number
											// Part of the century as Number
	// Ergebnis zurck zum Aufrufer.
	// Solution back to the caller.
	return(23.439291 - (0.013004 * T));
	
}// End of meanEclipticInclination()