/* ************************************************************************************
 *	Das Modul beinhaltet Funktionen zur Berechnung des Sonnen Auf-
 *	und Untergangs.
 * 	Die Funktionen und Algorithmen haben keinen Anspruch auf
 *	hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module includes functions for calculating the suns
 * rise and set.
 * The functions and algorithms do not claim 
 * high accuracy, completeness and correctness.
 *
 *
 *	Funktionsliste - Function list:
 *
 *	- twilightLCT()					-	Berechnet die lokale Zeit der Dmmerung
 *												Calculates the local time of twilight
 * - sunLong()						- 	Berechnet den ekliptischen Lngengrad der Sonne
 *												Calculates the longitude of the nutation
 * - sunriseLocalTime()			-	Berechnet den Sonnenaufgang, Zeiten und Status
 *												Calculates the sunrise, times and state
 * - sunsetLocalTime()			-	Berechnet den Sonnenuntergang, Zeiten und Status
 *												Calculates the sunset, times and state
 *	- sunriseAZ()					-	Berechnet den Azimuth/Hhe des Sonnenaufgangs
 *												Calculates the azimuth/height of the sunrise
 *	- sunsetAZ()						-	Berechnet den Azimuth/Hhe des Sonnenauntergangs
 *												Calculates the azimuth/height of the sunset
 * - precSunHorAZHeight()	-	Berechnet Az + H der Sonnenposition im horizontalen. Koordinatensystem
 *												Calculates Az + H of the sun's horizontal position coordinate system
 *
 *	Erstellt:	06.05.2021	Dieter Steffan
 *
 *********************************************************************************** */	
 
 
 
 /* *********************************************************************************
 *	Die Funktion berechnet die Dmmerungsdaten fr den Sonnenaufgang
 *	und Sonnenuntergang, auch fr die gewnschte Dmmerungsart an einem
 *	bestimmten Datum mit Sommer/Winterzeit und Zeitzone und beliebigen
 *	Standort auf der Erde..
 * 
 *	bergabewert:
 *
 * 		ld					=	Der Berechnungstag	als Integer
 *		lm					=	Der Berechnungsmonat als Integer
 *		ly					=	Das Berechnungsjahr als Integer
 *		ds					=	Daylight saving time als Integer 0/1
 *		zc					=	Zeitzone als Ganzzahl und Integer -12 bis + 12
 *		glong			=	Lngengrad als dezimaler Wert -180 bis + 180
 *		glat				= 	Breitengrad als dezimaler Wert -90 bis +90
 *		typ				=  Art der Dmmerung Astronomisch(a)/Nautisch(n)/Brgerlich(c) als Zeichen
 *		what				=	Morgens = am und Abends = pm als String
 *	
 *	Returnwert:
 * 		LCT[0]			=	Dmmerung Morgens/Abends als dezimaler Wert und Number
 *		LCT[1]			=	Status der Konvertierung GST zu UT als String "GST zu UT Warnung"/"OK"
 *		LCT[2]			=	Status der Sichtbarkeit der Sonne als String "OK"/"Nie sichtbar"/"Immer sichtbar"
 *		LCT[3]			=	Status der Dmmerung selber als String "OK"/"Sonne immer sichtbar"/"Sonne zu weit unter dem Horizont"
 *	---------------------------------------------------------------------------------------------------
 / * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * The function calculates the twilight data for the sunrise
 * and sunset and for the desired type of twilight at a
 * certain date with summer/winter time and time zone and any
 * Place on Earth . .
 *
 * Transfer value:
 *
 * 		ld 				= 	The calculation day as integer
 *		lm 			= 	The calculation month as integer
 * 		ly  			= 	The year of calculation as integer
 * 		ds 			= 	Daylight saving time as integer 0/1
 * 		zc 			= Time zone as integer and integer -12 to + 12
 * 		glong 		= 	Longitude as decimal value -180 to + 180
 * 		sleek 		= 	Latitude as decimal value -90 to +90 
 * 		type 			= 	Type of twilight Astronomical(a)/Nautical(n)/Civic(c) as sign
 * 		what 		= 	Morning = am and pm = pm as string
 *
 *	
 *	 Return value:
 * 		 LCT[0]		=	 Twilight  Morning/evening as decimal value and number
 *		 LCT[1]		=	 Convert status GST to UT as string  GST to UT Warning/OK "
 *		 LCT[2]		=	 Sun visibility status as string OK/Never visible/Always visible
 *		 LCT [3]	=	 Status of twilight itself as string OK/Sun always visible/
 *							 Sun too far below the horizon
 *************************************************************************************** */
 function twilightLCT(ld, lm, ly, ds, zc, glong, glat, typ, what){
  	// bergabeparameter Datentypensicher speichern
 	// Transfer parameters Save data type-safe
	var LD = parseInt(ld);					// Tag, Monat, Jahr Berechnungsdatum
 	var LM = parseInt(lm);					// Day, month, year of the calculation date
	var LY	= parseInt(ly);
	var DS = parseInt(ds);					// Sommer/Winterzeit - Daylight saving time
	var ZC = parseInt(zc);					// Zeitzone - Time zone
	var GL = Number(glong);				// Geographische Position Beobachter Lngengrad
														// Observers position longitude
	var GP = Number(glat);				// Geographische Position Beobachter Latitude
														// Observers position latitude
	var TYP = String(typ).toLowerCase;// Art der Dmmerung astronom., nautisch, brgerlich
														// Kind of twilight, Astronom., nautical, civil
	var WHAT = String(what).toLowerCase;// Fr wann die Berechnung durchgefhrt wird, AM = Sonnenaufgang
														// PM = Sonnenuntergang
														// For when the calculation is performed, AM =  Sunrise
 														// PM  =  Sunset
	var LCT = new Array();				// Startzeit/Endzeit der Dmmerung als dez. Zeit
														// Start time /End time of twilight as decimal  time
	var GD = 0;									// Grennwich Datum aus lokalem Datum
	var GM = 0;
	var GY = 0;
	var X = 0.0;									// Greenwich Sternzeit - Greenwich siderial time
	var Y = 0.0;									// quatoriale Deklination der Sonne
														//  Equatorial declination of the Sun
	var A = 0.0;									// Korregierte Longitude der Sonne
														// Corrected longitude of the sun
	var SR = 0.0;								// Longitude der Sonne
	var LA = 0.0;								// Die lokale siderische Zeit der RA des Aufgangs
														//  The local sidereal time of the RA of the rise
														// Longitude of the Sun
	var S = "";									// Sichtbarkeitsstatus der Sonne als String
														// Visibility of the Sun as string
	var eGSTUT = "";							// Fehler bei der Konvertierung 
	var XX = 0.0;								// Returnwerte ist die dez. Zeit des Aufgangs
														// Return value is the sunrise time
	var DI	= 0.0;								// Das horizontale Shifting mit allen Korrekturen der Sonne
														// The horizontal shifting with all corrections of the sun
	var ISTWILIGHT = "";					// Status der Dmmerung selber - State of the twilight self
	// Whle die Art der Dmmerung ...
	// ... astronomisch ...
	DI = 18;
	// ... brgerlich ...
    if (TYP == "c")
		DI = 6;
	// ... nautisch ...
    else  if (TYP == "n")
		DI = 12;											
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GD = localtime2GreenwichDay(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... berechne die Longitude der Sonne zum Berechnungszeitraum ...
	SR = sunLong(12, 0, 0, DS, ZC, LD, LM, LY);
	// ... Springe zur Unterfunktion und berechne die Werte fr 12:00 Uhr am
	// Berechnungstag ...
	// ... Jump to the subfunction and calculate the values for 12:00 on ...
 	// Calculation day
	sub3710();
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	// .  .  .  and calculate the Universal Time (UT) from the Greenwich Stardate .  .  .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... Ist bei der Konvertierung der lokalen siserischen Zeit zu Greenwich siderischer
	// Zeit ein Fehler aufgetreten, dann Abbruch ...
	// . . . is sidereal when converting local siseric time to Greenwich more sidereal
	// Time an error occurred, then abort . . .
	if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GD, GM, GY)) != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// ... und jetzt mit der UT in Greenwich, ohne DST und TZ, die Longitude
	// der Sonne neu berechnen ...
	// ...  and now with the UT in Greenwich, without DST and TZ, the longitude
 	// recalculate the Sun ...
	SR = 0.0;
	SR = sunLong(UT, 0, 0, 0, 0, GD, GM, GY);
	// ... und gehe dann zur Unterfunktion sub3710() ...
	// ... and then go to function sub3710() ...
	sub3710();
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	// .  .  .  and calculate the Universal Time (UT) from the Greenwich Stardate .  .  .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... dann noch die lokale Zeit aus der UT und Datum, DST und TZ berechnen ...
	//  ... then calculate the local time from the UT and date, DST and TZ ...
	XX = ut2LocalTime(UT, 0, 0, DS, ZC, GD, GM, GY);
	// ... und ans Ende der Funktion springen ...
	// ... and jump to the end of the function ...
	sub3740();
	
	function sub3710(){
		A = SR + nutationLong(GD, GM, GY) - 0.005694;
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for the RA in degrees ...
		X = eclipticCoords2EquatorialRA(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die Deklination in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for declination in degrees ...
		Y = eclipticCoords2EquatorialDec(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		//  Calculate the local sidereal time of the RA of the rise ...
		if (what == "am")
			LA = riseLocalSiderealTime((X/15.0), 0, 0, Y, 0, 0, DI , GP);
		// ... hier der Untergang bei Parameter "pm" ist es der Untergang ...
		else
			LA = setLocalSiderealTime((X/15.0), 0, 0, Y, 0, 0, DI , GP);
		// ... und dann die Sichtbarkeit der Sonne berprfen, ob er Auf- und Untergeht, niemals
		// zu sehen ist, oder immer zu sehen ist ... kann auch fr die Sonne verwendet werden ...
		// ...  and then check the visibility of the sun to see if it rises and sets, never
 		// can be seen, or can always be seen ... can also be used for the Sun ...
		S = visibleStateMoonRiseSet((X / 15.0), 0, 0, Y, 0, 0, DI, GP);
		// Der Status der Dmmerung selber ...
		//  The status of the twilight itself ...
		ISTWILIGHT = "OK";
		if (S.charAt(0) == "I") 
			// ... wenn die Sonne immer sichtbar ist ...
		// . . . when the sun is always visible . . .
			ISTWILIGHT = "Sonne immer sichtbar";
		else if (S.charAt(0) == "N")
			// ... oder sehr tief unter den Horizont sinkt ...
		// . . . or sinking very deep below the horizon . . .
			ISTWILIGHT = "Sonne zu weit unter dem Horizont";
		// ... zurck zum Aufrufer.
		// ... back to the caller
		return;
	}
	
	// Abbruchfunktion ...
	// Abort function . . .
	function sub3740(){
		// ... und mit Fehler raus aus der Funktion ...
		// ... and with error out of function ...
	}
	// .. die Twilight Uhrzeit als Dezimalwert ins Array ...
	LCT.push(XX);
	// ... Status der Konvertierung GST zu UT ins Array ...
	LCT.push(eGSTUT);
	// ... Status Sichtbarkeit der Sonne ins Array ...
	LCT.push(S);
	// ... Status der Dmmerung selber ins Array ...
	LCT.push(ISTWILIGHT);
	 // ... und zurck zum Aufrufer.
	 // ... and back to the caller.
	 return(LCT);
	 
 } // End of twilightLCT()
 
 


 /* *********************************************************************************
 * Die Funktion berechnet den Azimut und die Hhe, RA und Deklination, 
 * des Sonnenstandes im horizontelen Koordinatensystem am Standort 
 * des Beobachters zum Beobachtungszeitpunkt. Der Azimut und Hhe, 
 * RA und Dec, werden in dezimalen Grad
 * berechnet. 
 * Der Azimut wird hier so berechnet das Sden 0 sind. Nach Osten 
 * sind es - Grade (0 - -90)und nach Westen +Grade  ( 0 - +90)
 *
 * Quelle : https://physik.cosmos-indirekt.de/Physik-Schule/Sonnenstand.
 * 
 *	bergabewert:
 *		lch			= Lokale Stunde
 *		lcm			= Lokale Minute
 *		lcs			= Lokale Sekunde
 *		ds			=	Daylight saving time
 *		zc			=	Zeitzone
 * 		ld				=	Der Berechnungstag
 *		lm			=	Der Berechnungsmonat
 *		ly				=	Das Berechnungsjahr
 *	
 *	Returnwert:
 * 		RET[0]			=	Azimut (Horizontal)in dezimaler Form und Number
 *		RET[1]			=	Hhe (Horizontal) in dezimaler Form und Number
 *		RET[2]			=	RA (quatorial)in dezimaler Form und Number
 *		RET[3]			=	Dec (quatorial) dezimaler Form und Number
 *
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the azimuth and the height, RA and declination, 
 *  of the Suns position in the horizontal coordinate system at the observers location
 *  at the time of observation.  The azimuth and height, RA and Dec, 
 *  are expressed in decimal degrees calculated. 
 *  The azimuth here will be calculated so that south is 0.  To the East it is  
 *  grades (0  -90)and to the West +Grade (0  +90).
 *
 * Source : https://physik.cosmos-indirekt.de/Physik-Schule/Sonnenstand.
 * 
 *	 Transfer value:
 *		Ich			=	Local time
 *		lcm			=	Local minute
 *		lcs			=	Local second
 *		ds			=	Daylight saving  time
 *		zc			=	Time zone
 *		ld				=	The day of calculation
 *		lm			=	The month of calculation
 *		ly 			=	The year of calculation
 *	
 *	 Return value:
 * 	 	RET[0]			 =	 Azimuth (horizontal) in decimal form and number
 *		 RET[1]			 =  Height (horizontal) in decimal form and number
 *		 RET[2]			 =  RA (equatorial)  in decimal form and number
 *		 RET[3]			 =  Dec (equatorial)   decimal form and number
 *
  *************************************************************************************** */
 function precSunHorAZHeight(lch, lcm, lcs, ds, zc, ld, lm, ly, glong, glat){
  	// bergabeparameter Datentypensicher speichern
 	// Transfer parameters Save data type-safe
	var LCH = parseInt(lch);				// Stunde, Minute, Sekunde des
 	var LCM = parseInt(lcm);				// Berechnungsdatums
	var LCS= parseInt(lcs);				// Hour, minute, second of the calculation date
	var LD = parseInt(ld);					// Tag, Monat, Jahr Berechnungsdatum
 	var LM = parseInt(lm);					// Day, month, year of the calculation date
	var LY= parseInt(ly);
	var DS = parseInt(ds);					// Sommer/Winterzeit - Daylight saving time
	var ZC = parseInt(zc);					// Zeitzone - Time zone
	var GLONG = Number(glong);		// Geographische Position Beobachter Lngengrad
														// Observers position longitude
	var GLAT = Number(glat);			// Geographische Position Beobachter Latitude
														// Observers position latitude
	var JD = 0.0;								// Julianisches Datum zurm Berechnungszeitpunkt UT
														// Julian date at UT local time
	var UT = 0.0;								// Weltzeit (UT)- Universal Time
	var T0 = new Array();					// Teil des vergangenen Jahrhunderts - Part of the Century
	var L = 0.0;									// Mittlere ekliptische Lnge der Sonne ohne Geschwindigkeitsschwankungen
																	// also idealisiert ohne die elliptik des Orbits
																	//  Mean ecliptic length of the Sun without velocity fluctuations
 																	// i.e. idealized without the elliptic of the orbit
	var g = 0.0;											// Mittelpunktsgleichung zur Korrektur von L zur wahren ekliptischen Lnge und
																	// wird auch mittlere Anomalie genannt
																	// center equation for the correction of L to true ecliptic longitude and
 																	// is also called medium anomaly
	var n = 0.0;											// Anzahl der Tage seit dem quinoptikum 0.5 Januar 2000 (J2000)
																	// Number of days since the equinoptic 0.5 January 2000 (J2000)
	var lambda = 0.0;									// Die wahre ekliptische Lnge der Sonne
																	//  The True Ecliptic Longitude of the Sun
	var epsilon = 0.0;									// Die ekliptische Schiefe -  The ecliptic slant
	var RA = 0.0;											// quatorial Koordinaten RA und Dec der Sonne
	var Dec = 0.0;										//  Equatorial Coordinates  RA and Dec of the Sun
	var thetaG = 0.0;									// Die Greenwich Sternzeit des gesuchten Datums UT
																	//  The Greenwich stardate time of the searched date UT
	var theta = 0.0;										// Die Sternzeit Stundenwinkel fr den Frhlingspunkt im Gradma
																	//  The stardate hour angle for the spring point in degrees
	var tau = 0.0;											// Stundnwinkel der Sonne fr den Beobachtungsort
																	// Hour angle of the Sun for the observation site
	var AZ = 0.0;											// Der Azimut der Sonne als Horizontalkoordinate
																	//  The Azimuth of the Sun as Horizontal Coordinate
	var H = 0.0;											// Die Hhe der Sonne als Horizontalkoordinate
																	//  The height of the Sun as a horizontal coordinate
	var HR = 0.0;										// Die Hhe der Sonne mit der Refraktion korregiert
																	//  The height of the Sun corrected with the refraction
	var RET = new Array();					// Returnwerte - Return values
	// Berechne das Julianische Datum zum Berechnungsdatum und Uhrzeit, inklusive Zeitzone, Sommer/
	// Winterzeit ...
	// Calculate  the Julian date for the calculation date and time, including time zone, summer/
 	// Winter time ...
	JD = calcJD(LD, LM, LY, LCH, LCM, LCS, 0,  DS, ZC);
	// ... und den Teil des vergangenen Jahrhunderts ab J2000 bis zum Berechnungsjahr ...
	// ... and the part of the last century from J2000 to the year of calculation ...
	T0 = partOfTheCentury(LY, JD);
	// ... und dann die Anzahl der Tage seit dem quinoptikum J2000 = 0.5 Januat 2000 ...
	// ...  and then the number of days since the equinoptic J2000  = 0.5 January 2000 ...
	n = JD - 2451545.0;
	// ... und danach die Mittlere ekliptische Lnge der Sonne ohne Geschwindigkeitsschwankungen ...
	// wobei das Ergebnis in den richtigen Bereich 0 - 360 gebracht wird. Es funktioniert auch mit
	// den greren Werten ...
	// ...  and then the Mean Ecliptic Longitude of the Sun without velocity fluctuations ...
 	//  where the result is placed in the correct range 0  360.  It also works with
 	//  the larger values ...
	L = unwindDeg(280.460 + 0.9856474 * n);
	// ... nun die mittlere Anomalie berechnen, um die mittlere Lnge der Sonne zur wahren lnge der Sonne
	// zu korregieren ... hier auch wieder den Wert in den Bereich 0 - 360 bringen ...
	//  ... now calculate the mean anomaly to change the mean length of the Sun to the true length of the Sun
 	// to correct ...  here again the value in the range 0  360 ...
	g = unwindDeg(357.528 + 0.9856003 * n);
	// ... und die wahre ekliptische Lnge der Sonne berechnen ... und wieder in den richtigen Bereich bringen ...
	// ...  and calculate the true ecliptic length of the Sun ... and bring it back to the right range ...
	lambda = unwindDeg(L + 1.915 * Math.sin(g / RAD)+ 0.01997 * Math.sin(( 2 * g)/ RAD));
	// ... und nun als nchstes die ekliptische Schiefe berechnen ...
	// ...  and now calculate the ecliptic slant ...
	epsilon = 23.439 - 0.0000004 * n;
	// ... jetzt knnen wir die quatorial Koordinaten RA und Dec berechnen ...
	// ... Rektaszension in dezimalen Grad
	//  ... now we can calculate the equatorial coordinates RA and Dec ...
 	//  ... Rectascension in decimal degrees
	if  (Math.cos( lambda / RAD) > 0.0)
		RA = Math.atan( Math.cos(epsilon / RAD) *  Math.tan(lambda / RAD) )* RAD;
	else if (Math.cos( lambda / RAD) < 0.0)
		RA = (Math.atan( Math.cos(epsilon / RAD) *  Math.tan(lambda / RAD) )+ 4 * Math.atan(1)) * RAD;
	// ... Deklination in dezimalen Grad ...
	//  ... Declination in decimal degrees ...
	Dec = Math.asin( Math.sin(epsilon / RAD)* Math.sin(lambda / RAD) )* RAD;
	// ... wir bentigen aber die Horizontal Koordinaten der Sonne fr den Azimut und die Hhe ber dem Horizont
	// und dafr die Greenwich Sternzeit zum gesuchten Zeitpunkt  als(UT)in Stunden ...
	//  ... but we need the horizontal coordinates of the Sun for the azimuth and the altitude above the horizon
 	// and for this the Greenwich  Stardate time at the desired time   as (UT) in hours ...
	UT = localtime2UT(LCH, LCM, LCS, DS, ZC, LD, LM, LY);
	// ... UT nach TT korregieren - Correct UT to TT ...
	UT = ut2TT(UT);
	thetag = 6.697376 + 2400.05134 * Number(T0[1]) + 1.002738 * UT;
	// ... und hier die Sternzeit in den richtigen 24 Stunden Bereich legen ...
	// ...  and put the stardate time in the correct 24 hour range ...
	thetag = thetag - (24.0 * Math.floor(thetag / 24.0));
	// ... und den Stundenwinkel des Frhlingspunkts fr den Ort der geographischen Lnge berechnen ...
	// ... and calculate the hour angle of the spring point for the place of the geographical longitude ...
	theta = thetag * 15.0 + GLONG;
	// ... und durch Subtraktion der RA den Stundenwinkel der Sonne fr den Beobachtungsort
	// ... and by subtracting the RA the hour angle of the Sun for the observation site
	tau = theta - RA;
	// ... und dann den Azimuth berechnen ...
	// ... and then calculate the azimuth ...
	AZ = Math.atan(Math.sin(tau / RAD)/ (Math.cos(tau / RAD)* Math.sin(GLAT / RAD)- Math.tan(Dec / RAD)* Math.cos(GLAT / RAD) ))* RAD;
	// ... und ber den Nenner der Division in den richtigen Bereich bringen ...
	// ... and use the denominator of the division in the correct area ...
	var nenner =  Math.cos(tau / RAD)* Math.sin(GLAT / RAD)- Math.tan(Dec / RAD)* Math.cos(GLAT / RAD);
	if (nenner < 0.0)AZ += 180.0; 
	// Den Azimut speichern - Store the azimuth
	RET.push(AZ);
	// ... und zum Schlu noch die Hhe ber dem Horizont ...
	// ... and finally the height above the horizon ...
	H = Math.asin( Math.cos(Dec / RAD)* Math.cos(tau / RAD)* Math.cos(GLAT / RAD)+ Math.sin(Dec / RAD)* Math.sin(GLAT /  RAD) ) * RAD;
	// ... und dazu noch die Refraktion der Atmosphre einrechnen ...
	// ...  and also calculate the refraction of the atmosphere ...
	var R = 1.02 / Math.tan(H + (10.3/ (H + 5.11))/  RAD );
	HR = H + R / 60.0;
	// Die Hhe speichern - Store the height ...
	RET.push(HR);
	// Die RA im quatorialen Koordinatensystem
	//  The RA in the Equatorial Coordinate System
	RET.push(RA);
	// Die Deklination im quatorialen Koordinatensystem
	//  Declination in the Equatorial Coordinate System
	RET.push(Dec);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(RET);
	
 } //End of precSunPosition()
 
 
 

 /* *********************************************************************************
 * Die Funktion berechnet die ekliptische Longitude der Sonne  zum 
 * Berechnungsdatum und Uhrzeit. Dabei werden die wahre Anomalie
  * der Sonne und die exzentrische Anomalie des Orbits mit eingerechnet.
 * 
 *	
 *	bergabewert:
 *		lch			= Lokale Stunde
 *		lcm			= Lokale Minute
 *		lcs			= Lokale Sekunde
 *		ds			=	Daylight saving time
 *		zc			=	Zeitzone
 * 		ld				=	Der Berechnungstag
 *		lm			=	Der Berechnungsmonat
 *		ly				=	Das Berechnungsjahr
 *	
 *	Returnwert:
 * 		SR 			=	Ekliptische Longitude der Sonne in dezimaler Form und Number
 *
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the ecliptic longitude of the Sun to 
 *  Calculation date and time.  In this case, the true anomaly is
  *  the Sun and the eccentric anomaly of the orbit.
  * 
  *	
  *	 Transfer value:
  *		Ich			=	Local time
  *		lcm			=	Local minute
  *		lcs			=	Local second
  *		ds			=	Daylight saving  time
  *		zc			=	Time zone
  *		ld				=	The day of calculation
  *		lm			=	The month of calculation
  *		ly 			=	The year of calculation
  *	
  *	 Return value:
  * 	SR 			=	Ecliptic Longitude of the Sun in Decimal Form and Number
  *
  *************************************************************************************** */
function sunLong(lch, lcm, lcs, ds, zc, ld, lm, ly){
 	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LCH = Number(lch);				// Stunde, Minute, Sekunde desr
 	var LCM = Number(lcm);				// Berechnungsuhrzeit
	var LCS= Number(lcs);
	var LD = Number(ld);					// Tag, Monat, Jahr Berechnungsdatum
 	var LM = Number(lm);					// Day, month, year Calculation date
	var LY= Number(ly);
	var DS = parseInt(ds);					// Sommer/Winterzeit - Daylight saving time
	var ZC = parseInt(zc);					// Zeitzone - Time zone
	var SR = 0.0;								// Longitude der Sonne - Longitude of the sun
	var UT = 0.0;								// Universal Time
	var DJ = 0.0;								// Modified JD aus dem Datum mit Epoche J2000
														// Modified JD from the date with epoch J2000
	var T = 0.0;									// Teil des Jahrhunderts + Anteil der Zeit am Tag
														// Part of the century + Part of the time per day
	
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	var AA = localtime2GreenwichDay(LCH, LCM, LCS, DS, ZC, LD, LM, LY);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	var BB = localtime2GreenwichMonth(LCH, LCM, LCS, DS, ZC, LD, LM, LY);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	var CC = localtime2GreenwichYear(LCH, LCM, LCS, DS, ZC, LD, LM, LY);
	//console.log("sunLong Datum =" + AA + "." + BB + "." + CC);
	// ... Universal Time zur Berechnungszeit und -Datum ...
	// ... Universal Time for calculation time and date ...
	UT = localtime2UT(LCH, LCM, LCS, DS, ZC, LD, LM, LY);
	//console.log("sunLong UT =" + UT);
	// ... und aus dem Berechnungsdatum das MJD berechnen ...
	// ... and calculate the MJD from the calculation date ...
	DJ = calDate2JulDate(AA, BB, CC) - 2415020.0;
	//console.log("sunLong DJ =" + DJ);
	// ... berechne dann mit dem MJD den Teil des Jahrhundert 
	// + den Anteil des Berechnungsdatums am Tag ...
	// ... then calculate with the MJD the part of the century 
 	// + the percentage of the calculation date on the day ...
	T = (DJ / 36525.0) + (UT / 876600.0);
	// ... und das Quadrat dazu ...
	// . . . and the square to it . . .
	T2 = T * T;
	//console.log("sunLong T =" + T + " T2=" + T2);
	// ... Hilfsvariablen fr die Polynomapproximation berechnen ...
	//  ... Calculate auxiliary variables for the polynomial approximation ...
	var A = 100.0021359 * T;
	var B = 360.0 * (A - Math.floor(A));
	//console.log("sunLong A =" + A + " B=" + B);
	// nun die Berechnung des Lngengrad es der Sonne ohne Korrekturen ...
	// now the calculation of the longitude  it of the sun without corrections ...
	var L = 279.69668 + 0.0003025 * T2 + B;
	//console.log("sunLong L =" + L );
	// ... Hilfsvariablen fr die Polynomapproximation berechnen ...
	//  ... Calculate auxiliary variables for the polynomial approximation ...
	A = 99.99736042 * T;
	B = 360.0 * (A - Math.floor(A));
	// ... Anomaly der Sonne berechnen ...
	//  ... Calculate anomaly of the Sun ...
	var M1 = 358.47583 - (0.00015 + 0.0000033 * T) * T2 + B;
	// ... Exzentrizitt des Sonne - Erd Orbits berechnen ...
	var EC = 0.01675104 - 0.0000418 * T - 0.000000126 * T2;
	//console.log("sunLong M1 =" + M1+ " EC=" + EC);
	var AM = M1 / RAD;
	// ... und dann die wahre Anomalie der Sonne berechnen ...
	// ...  and then calculate the true anomaly of the Sun ...
	var AT = trueAnomaly(AM, EC);
	//console.log("sunLong AT =" + AT);
	// ... und die exzentrische Anomalie ...
	// ...  and the eccentric anomaly...
	var AE = eccentricAnomaly(AM, EC);
	//console.log("sunLong AE =" + AE);
	// ... dann die mathematische Reihenentwicklung ...
	// . . . then the mathematical series development . . .
	A = 62.55209472 * T;
	B = 360.0 * (A - Math.floor(A));
    var A1 = (153.23 + B)/ RAD;
	A = 125.1041894 * T;
	B = 360.0* (A - Math.floor(A));
    var B1 = (216.57 + B)/ RAD;
	A = 91.56766028 * T;
	B = 360.0 * (A - Math.floor(A));
    var C1 = (312.69 + B)/ RAD;
	A = 1236.853095 * T;
	B = 360.0 * (A - Math.floor(A));
    var D1 = (350.74 - 0.00144 * T2 + B)/ RAD;
	var E1 = (231.19 + 20.2 * T) / RAD;
	A = 183.1353208 * T;
	B = 360.0 * (A - Math.floor(A));
    var H1 = (353.4 + B)/ RAD;
	var D2 = 0.00134 * Math.cos(A1) + 0.00154 * Math.cos(B1) + 0.002 * Math.cos(C1);
	D2 = D2 + 0.00179 * Math.sin(D1) + 0.00178 * Math.sin(E1);
	var D3 = 0.00000543 * Math.sin(A1) + 0.00001575 * Math.sin(B1);
	D3 = D3 + 0.00001627 * Math.sin(C1) + 0.00003076 * Math.cos(D1);
	D3 = D3 + 0.00000927 * Math.sin(H1);
	// ... und das Ergebnis in Grad berechnen ...
	// ... and calculate the result in degrees ...
	SR = AT +  ((L - M1 + D2)/ RAD);
	var TP = 6.283185308;
	SR = SR - TP * Math.floor(SR / TP);
	SR = SR * RAD;
	//console.log("sunLong UT =" + UT);
	// ... und dann zurck zum Aufrufer.
	// ... and ten back to the caller.
	return(SR);
	
}// End of sunLong()







 /* *********************************************************************************
 * Die Funktion berechnet den Azimut und die Hhe der Sonne am Horizont
 *	bei Sonnenaufgang. Ebenso wird der Status der Berechnung ermittelt.
 *	
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0]	=	Der Azimut in Dezimaler Form und Number
 *		RET[1]	= Die Hhe der Sonne am Horizont in , ', ''
 *		RET[2]	= Die Hhe der Sonne am Horizont in dezimalen Graden und als Number
 *		RET[3]	=	Status der Konvertierung von GST nach UT. Werte sind "Warnung" + "OK"
 *							und als String
 *		RET[4]	=	Status des Sonnenaufgangs als String . Werte sind "OK",
 *							"Nie Sichtbar" und "Immer Sichtbar" ...
 *	---------------------------------------------------------------------------------------------------
*  The function calculates the azimuth and the height of the Sun at the horizon
  *	 at sunrise.  The status of the calculation is also determined.
  *	
  *	 Transfer value:
  * 	day			=	 The day of calculation
  *		 month		=	 The month of calculation
  *		 year		=	 The year of calculation
  *		 dst			=	 Daylight saving  time
 *		 tz			=	 Time zone
 *		 glong		=  Geographical longitude of the place
 *		 glat			=  Geographical latitude of the place
 *	
 *	 Return value:
 * 		RET[0]	=	 The azimuth in decimal form and number
 *		RET[1]	=  The height of the Sun at the horizon in , ',  
 *		RET[2]	=  The height of the Sun at the horizon in decimal degrees and as a number
 *		RET[3]	=	 Convert status from GST to UT.  Values are "Warning" + "OK"
 *							and as a string
 *		RET[4 ]	=	 Status of sunrise as a string .  Values are "OK",
							"Never Visible"  and  "Always Visible"  ...
  *************************************************************************************** */
 function sunriseAZ(day, month, year, dst, tz, glong, glat){
 	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LD = parseInt(day);					// Tag, Monat, Jahr Berechnungsdatum
 	var LM = parseInt(month);				// Day, month, year Calculation date
	var LY= parseInt(year);
	var DS = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var ZC = parseInt(tz);						// Zeitzone - Time zone
	var GL = Number(glong);					// Lngengrad des Orts - Longitude of the place
	var GP = Number(glat);					// Breitengrad des Orts - Latitude of the place
	var GD = 0;										// Grennwich Datum aus lokalem Datum
	var GM = 0;
	var GY = 0;
	var UT = 0.0;										// Grennwich UT
	var eGSTUT = "";									// Fehler bei der Konvertierung 
	var X = 0.0;											// Greenwich Sternzeit - Greenwich siderial time
	var Y = 0.0;											// quatoriale Deklination der Sonne
																//  Equatorial declination of the Sun
	var A = 0.0;											// Korregierte Longitude der Sonne
																	// Corrected longitude of the sun
	var SR = 0.0;											// Longitude der Sonne
	var LA = 0.0;											// Die lokale siderische Zeit der RA des Aufgangs
																	//  The local sidereal time of the RA of the rise
																	// Longitude of the Sun
	var S = "";												// Sichtbarkeitsstatus der Sonne als String
																	// Visibility of the Sun as string
	var XX = 0.0;											// Returnwerte ist die dez. Zeit des Aufgangs
																	// Return value is the sunrise time
	var DI	= 0.8333333;								// Das horizontale Shifting mit allen Korrekturen der Sonne
																	// The horizontal shifting with all corrections of the sun
	var RET = new Array();					// Returnarray fr die Rckgabewerte
	// Fehlerkriterium auf OK setzen ...
	// Error criterion set to OK . . .
	var S = "OK";													
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GD = localtime2GreenwichDay(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... berechne die Longitude der Sonne zum Berechnungszeitraum ...
	SR = sunLong(12, 0, 0, DS, ZC, GD, GM, GY);
	// ... Springe zur Unterfunktion und berechne die Werte fr 12:00 Uhr am
	// Berechnungstag ...
	// ... Jump to the subfunction and calculate the values for 12:00 on ...
 	// Calculation day
	sub3710();
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... Ist bei der Konvertierung der lokalen siserischen Zeit zu Greenwich siderischer
	// Zeit ein Fehler aufgetreten, dann Abbruch ...
	// . . . is sidereal when converting local siseric time to Greenwich more sidereal
	// Time an error occurred, then abort . . .
	if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GD, GM, GY)) != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// ... und jetzt mit der UT in Greenwich, ohne DST und TZ, die Longitude
	// der Sonne neu berechnen ...
	// ...  and now with the UT in Greenwich, without DST and TZ, the longitude
 	// recalculate the Sun ...
	SR = 0.0;
	SR = sunLong(UT, 0, 0, 0, 0, GD, GM, GY);
	// ... und gehe dann zur Unterfunktion sub3710() ...
	// ... and then go to function sub3710() ...
	sub3710();
	
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// ... nun berechne den Azimuth des Sonnenaufgangs am Standort ...
	// . . . now find the azimuth of the sunrise at the location . . .
	XX = riseLocalSiderealDegreesAZ(X/15.0, 0, 0, Y, 0, 0, DI, GP);
	// ... und ans Ende der Funktion springen ...
	// ... and jump to the end of the function ...
	sub3740();
	
	function sub3710(){
		A = SR + nutationLong(GD, GM, GY) - 0.005694;
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for the RA in degrees ...
		X = eclipticCoords2EquatorialRA(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die Deklination in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for declination in degrees ...
		Y = eclipticCoords2EquatorialDec(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		//  Calculate the local sidereal time of the RA of the rise ...
		LA = riseLocalSiderealTime((X/15.0), 0, 0, Y, 0, 0, DI , GP);
		// ... und dann die Sichtbarkeit der Sonne berprfen, ob er Auf- und Untergeht, niemals
		// zu sehen ist, oder immer zu sehen ist ... kann auch fr die Sonne verwendet werden ...
		// ...  and then check the visibility of the sun to see if it rises and sets, never
 		// can be seen, or can always be seen ... can also be used for the Sun ...
		S = visibleStateMoonRiseSet((X / 15.0), 0, 0, Y, 0, 0, DI, GP);
		// ... zurck zum Aufrufer.
		// ... back to the caller
		return;
	}
	// Abbruchfunktion ...
	// Abort function . . .
	function sub3740(){
		// ... und mit Fehler raus aus der Funktion ...
		// ... and with error out of function ...
	}
	// In XX[0] steht der dezimale Azimut ...
	//  In XX[0] the decimal azimut is ...
	RET.push(XX);
	// In XX[1] steht die Hhe der Sonne ber dem Horizont in , ', '' als Horizontal Koordinate  ...
	//  In XX[1] the height of the sun is above the horizon in , ',  as horizontal  Coordinates...
	RET.push(decDegrees2AngleOfHours(objectHeight(XX, Y, GP )));
	// In XX[2] steht die Hhe der Sonne ber dem Horizont in dez. Grad als Horizontal Koordinate  ...
	//  In XX[2] the height of the sun is above the horizon in decimal degrees as horizontal  Coordinates...
	RET.push(objectHeight(XX, Y, GP ));
		// In XX[3] steht ob die Zeitkonvertierung OK war oder Warnung ...
	//  In XX[13 it is written whether the time conversion was OK or warning ...
	RET.push(eGSTUT);
	// In XX[4] steht der Status des Sonnenaufgangs, ob OK,
	// Immer sichtbar oder Nie sichtbar ...
	//  In XX[4] the state of the sunrise, whether OK,
 	//  Always visible or Never visible ...
	RET.push(S);
	// ... und zurck zum Aufrufer.					
	// ... and back to the caller.
	return(RET);
 
 }// End of sunriseAZ()




 /* *********************************************************************************
 * Die Funktion berechnet das lokale Datum des Sonnen Aufgangs in
 * kompakter Form als Dezimalwert.
 *	
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0]		=	Der Sonnenaufgang in Dezimaler Form und Number
 *		RET[1]		=	Status der Konvertierung von GST nach UT. Werte sind "Warnung" + "OK"
 *		RET[2]		=	Status des Sonnenaufgangs . Werte sind "OK",
								"Nie Sichtbar" und "Immer Sichtbar" ...
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the local date of the sun rise in
 * compact form in an array. See at Return values.
 * 
 * Transfer value: 
 * 		day 		= The calculation day 
 * 		month 	= The calculation month 
 * 		year 	= The calculation year 
 * 		dst 		= Daylight saving time 
 * 		tz 		= Time zone 
 * 		glong 	= Geographical longitude of the location 
 * 		glat 		= Geographical latitude of the location 
 * 
 * Return value: 
 *		RET[0]		 =	 The Sunrise in Decimal Form and Number
 *		 RET[1]		 =	 Convert status from GST to UT.  Values are Warning + OK
 *		 RET[2]		 =	 Status of sunrise .  Values are OK,
 *								"Never Visible and Always Visible ...
 *
  *************************************************************************************** */
 function sunriseLocalTime(day, month, year, dst, tz, glong, glat){
 	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LD = parseInt(day);					// Tag, Monat, Jahr Berechnungsdatum
 	var LM = parseInt(month);				// Day, month, year Calculation date
	var LY= parseInt(year);
	var DS = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var ZC = parseInt(tz);						// Zeitzone - Time zone
	var GL = Number(glong);				// Lngengrad des Orts - Longitude of the place
	var GP = Number(glat);					// Breitengrad des Orts - Latitude of the place
	var GD = 0;											// Grennwich Datum aus lokalem Datum
	var GM = 0;
	var GY = 0;
	var UT = 0.0;											// Grennwich UT
	var eGSTUT = "";									// Fehler bei der Konvertierung 
	var X = 0.0;											// Greenwich Sternzeit - Greenwich siderial time
	var Y = 0.0;											// quatoriale Deklination der Sonne
																	//  Equatorial declination of the Sun
	var A = 0.0;											// Korregierte Longitude der Sonne
																	// Corrected longitude of the sun
	var SR = 0.0;											// Longitude der Sonne
	var LA = 0.0;											// Die lokale siderische Zeit der RA des Aufgangs
																	//  The local sidereal time of the RA of the rise
																	// Longitude of the Sun
	var S = "";												// Sichtbarkeitsstatus der Sonne als String
																	// Visibility of the Sun as string
	var XX = 0.0;											// Returnwerte ist die dez. Zeit des Aufgangs
																	// Return value is the sunrise time
	var DI	= 0.8333333;								// Das horizontale Shifting mit allen Korrekturen der Sonne
																	// The horizontal shifting with all corrections of the sun
	var RET = new Array();					// Returnarray fr die Rckgabewerte
	// Fehlerkriterium auf OK setzen ...
	// Error criterion set to OK . . .
	var S = "OK";													
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GD = localtime2GreenwichDay(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... berechne die Longitude der Sonne zum Berechnungszeitraum ...
	SR = sunLong(12, 0, 0, DS, ZC, GD, GM, GY);
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... Ist bei der Konvertierung der lokalen siserischen Zeit zu Greenwich siderischer
	// Zeit ein Fehler aufgetreten, dann Abbruch ...
	// . . . is sidereal when converting local siseric time to Greenwich more sidereal
	// Time an error occurred, then abort . . .
	if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GD, GM, GY)) != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// ... und jetzt mit der UT in Greenwich, ohne DST und TZ, die Longitude
	// der Sonne neu berechnen ...
	// ...  and now with the UT in Greenwich, without DST and TZ, the longitude
 	// recalculate the Sun ...
	SR = 0.0;
	SR = sunLong(UT, 0, 0, 0, 0, GD, GM, GY);
	//console.log("sunriseLocalTime SR2=" + SR);
	// ... und gehe dann zur Unterfunktion sub3710() ...
	// ... and then go to function sub3710() ...
	sub3710();
	
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	// .  .  .  and calculate the Universal Time (UT) from the Greenwich Stardate .  .  .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... dann noch die lokale Zeit aus der UT und Datum, DST und TZ berechnen ...
	//  ... then calculate the local time from the UT and date, DST and TZ ...
	XX = ut2LocalTime(UT, 0, 0, DS, ZC, GD, GM, GY);
	// ... und ans Ende der Funktion springen ...
	// ... and jump to the end of the function ...
	sub3740();
	
	function sub3710(){
		A = SR + nutationLong(GD, GM, GY) - 0.005694;
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for the RA in degrees ...
		X = eclipticCoords2EquatorialRA(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die Deklination in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for declination in degrees ...
		Y = eclipticCoords2EquatorialDec(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		//  Calculate the local sidereal time of the RA of the rise ...
		LA = riseLocalSiderealTime((X/15.0), 0, 0, Y, 0, 0, DI , GP);
		// ... und dann die Sichtbarkeit des Mondes berprfen, ob er Auf- und Untergeht, niemals
		// zu sehen ist, oder immer zu sehen ist ... kann auch fr die Sonne verwendet werden ...
		// ...  and then check the visibility of the moon to see if it rises and sets, never
 		// can be seen, or can always be seen ... can also be used for the Sun ...
		S = visibleStateMoonRiseSet((X / 15.0), 0, 0, Y, 0, 0, DI, GP);
		// ... zurck zum Aufrufer.
		// ... back to the caller
		return;
	}
	
	// Abbruchfunktion ...
	// Abort function . . .
	function sub3740(){
		// ... und mit Fehler raus aus der Funktion ...
		// ... and with error out of function ...
	}
	// In XX[0] steht die dezimale Aufgangszeit ...
	//  In XX[0] the decimal rise time is ...
	RET.push(XX);
	// In XX[1] steht ob die Zeitkonvertierung OK war oder Warnung ...
	//  In XX[1] it is written whether the time conversion was OK or warning ...
	RET.push(eGSTUT);
	// In XX[2] steht der Status des Sonnenaufgangs, ob OK,
	// Immer sichtbar oder Nie sichtbar ...
	//  In XX[2] the state of the sunrise, whether OK,
 	//  Always visible or Never visible ...
	RET.push(S);
	// ... und zurck zum Aufrufer.					
	// ... and back to the caller.
	return(RET);
 
 }// End of sunriseLocalTime()
 
 
 
  /* *********************************************************************************
 * Die Funktion berechnet das lokale Datum des Sonnen Aufgangs in
 * kompakter Form als Dezimalwert.
 *	
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0]		=	Der Sonnenaufgang in Dezimaler Form und Number
 *		RET[1]		=	Status der Konvertierung von GST nach UT. Werte sind "Warnung" + "OK"
 *		RET[2]		=	Status des Sonnenaufgangs . Werte sind "OK",
								"Nie Sichtbar" und "Immer Sichtbar" ...
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the local date of the sun rise in
 * compact form in an array. See at Return values.
 * 
 * Transfer value: 
 * 		day 		= The calculation day 
 * 		month 	= The calculation month 
 * 		year 	= The calculation year 
 * 		dst 		= Daylight saving time 
 * 		tz 		= Time zone 
 * 		glong 	= Geographical longitude of the location 
 * 		glat 		= Geographical latitude of the location 
 * 
 * Return value: 
 *		RET[0]		 =	 The Sunrise in Decimal Form and Number
 *		 RET[1]		 =	 Convert status from GST to UT.  Values are Warning + OK
 *		 RET[2]		 =	 Status of sunrise .  Values are OK,
 *								"Never Visible and Always Visible ...
 *
  *************************************************************************************** */
 function sunsetLocalTime(day, month, year, dst, tz, glong, glat){
 	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LD = parseInt(day);					// Tag, Monat, Jahr Berechnungsdatum
 	var LM = parseInt(month);				// Day, month, year Calculation date
	var LY= parseInt(year);
	var DS = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var ZC = parseInt(tz);						// Zeitzone - Time zone
	var GL = Number(glong);				// Lngengrad des Orts - Longitude of the place
	var GP = Number(glat);					// Breitengrad des Orts - Latitude of the place
	var GD = 0;											// Grennwich Datum aus lokalem Datum
	var GM = 0;
	var GY = 0;
	var UT = 0.0;											// Grennwich UT
	var eGSTUT = "";									// Fehler bei der Konvertierung 
	var X = 0.0;											// Greenwich Sternzeit - Greenwich siderial time
	var Y = 0.0;											// quatoriale Deklination der Sonne
																	//  Equatorial declination of the Sun
	var A = 0.0;											// Korregierte Longitude der Sonne
																	// Corrected longitude of the sun
	var SR = 0.0;											// Longitude der Sonne
	var LA = 0.0;											// Die lokale siderische Zeit der RA des Aufgangs
																	//  The local sidereal time of the RA of the rise
																	// Longitude of the Sun
	var S = "";												// Sichtbarkeitsstatus der Sonne als String
																	// Visibility of the Sun as string
	var XX = 0.0;											// Returnwerte ist die dez. Zeit des Aufgangs
																	// Return value is the sunrise time
	var DI	= 0.8333333;								// Das horizontale Shifting mit allen Korrekturen der Sonne
																	// The horizontal shifting with all corrections of the sun
	var RET = new Array();					// Returnarray fr die Rckgabewerte
	// Fehlerkriterium auf OK setzen ...
	// Error criterion set to OK . . .
	var S = "OK";													
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GD = localtime2GreenwichDay(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... berechne die Longitude der Sonne zum Berechnungszeitraum ...
	SR = sunLong(12, 0, 0, DS, ZC, GD, GM, GY);
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... Ist bei der Konvertierung der lokalen siserischen Zeit zu Greenwich siderischer
	// Zeit ein Fehler aufgetreten, dann Abbruch ...
	// . . . is sidereal when converting local siseric time to Greenwich more sidereal
	// Time an error occurred, then abort . . .
	if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GD, GM, GY)) != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// ... und jetzt mit der UT in Greenwich, ohne DST und TZ, die Longitude
	// der Sonne neu berechnen ...
	// ...  and now with the UT in Greenwich, without DST and TZ, the longitude
 	// recalculate the Sun ...
	SR = 0.0;
	SR = sunLong(UT, 0, 0, 0, 0, GD, GM, GY);
	//console.log("sunriseLocalTime SR2=" + SR);
	// ... und gehe dann zur Unterfunktion sub3710() ...
	// ... and then go to function sub3710() ...
	sub3710();
	
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	// .  .  .  and calculate the Universal Time (UT) from the Greenwich Stardate .  .  .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... dann noch die lokale Zeit aus der UT und Datum, DST und TZ berechnen ...
	//  ... then calculate the local time from the UT and date, DST and TZ ...
	XX = ut2LocalTime(UT, 0, 0, DS, ZC, GD, GM, GY);
	// ... und ans Ende der Funktion springen ...
	// ... and jump to the end of the function ...
	sub3740();
	
	function sub3710(){
		A = SR + nutationLong(GD, GM, GY) - 0.005694;
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for the RA in degrees ...
		X = eclipticCoords2EquatorialRA(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die Deklination in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for declination in degrees ...
		Y = eclipticCoords2EquatorialDec(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		//  Calculate the local sidereal time of the RA of the rise ...
		LA = setLocalSiderealTime((X/15.0), 0, 0, Y, 0, 0, DI , GP);
		// ... und dann die Sichtbarkeit des Mondes berprfen, ob er Auf- und Untergeht, niemals
		// zu sehen ist, oder immer zu sehen ist ... kann auch fr die Sonne verwendet werden ...
		// ...  and then check the visibility of the moon to see if it rises and sets, never
 		// can be seen, or can always be seen ... can also be used for the Sun ...
		S = visibleStateMoonRiseSet((X / 15.0), 0, 0, Y, 0, 0, DI, GP);
		// ... zurck zum Aufrufer.
		// ... back to the caller
		return;
	}
	
	// Abbruchfunktion ...
	// Abort function . . .
	function sub3740(){
		// ... und mit Fehler raus aus der Funktion ...
		// ... and with error out of function ...
	}
	// In XX[0] steht die dezimale Aufgangszeit ...
	//  In XX[0] the decimal rise time is ...
	RET.push(XX);
	// In XX[1] steht ob die Zeitkonvertierung OK war oder Warnung ...
	//  In XX[1] it is written whether the time conversion was OK or warning ...
	RET.push(eGSTUT);
	// In XX[2] steht der Status des Sonnenaufgangs, ob OK,
	// Immer sichtbar oder Nie sichtbar ...
	//  In XX[2] the state of the sunrise, whether OK,
 	//  Always visible or Never visible ...
	RET.push(S);
	// ... und zurck zum Aufrufer.					
	// ... and back to the caller.
	return(RET);
 
 }// End of sunsetLocalTime()
 
 
 
 
  /* *********************************************************************************
 * Die Funktion berechnet den Azimut und die Hhe der Sonne am Horizont
 *	bei Sonnenuntergang. Ebenso wird der Status der Berechnung ermittelt.
 *	
 *	bergabewert:
 * 		day			=	Der Berechnungstag
 *		month		=	Der Berechnungsmonat
 *		year		=	Das Berechnungsjahr
 *		dst			=	Daylight saving time
*		tz				=	Zeitzone
*		glong		= Geografischer Lngengrad des Orts
*		glat			= Geografischer Breitengrad des Orts
 *	
 *	Returnwert:
 * 		RET[0]	=	Der Azimut in Dezimaler Form und Number
 *		RET[1]	=	Die Hhe der Sonne am Horizont in , ', ''
 *		RET[2]	=	Die Hhe der Sonne am Horizont in dezimalen Graden und als Number
 *		RET[3]	=	Status der Konvertierung von GST nach UT. Werte sind "Warnung" + "OK"
 *							und als String
 *		RET[4]	=	Status des Sonnenaufgangs als String . Werte sind "OK",
 *							"Nie Sichtbar" und "Immer Sichtbar" ...
 *	---------------------------------------------------------------------------------------------------	
 *  The function calculates the azimuth and the height of the Sun at the horizon
 *	 at sunset.  The status of the calculation is also determined.
 *	
 *	 Transfer value:
 *			day			=	 The day of calculation
 *		 	month		=	 The month of calculation
 *		 	year		=	 The year of calculation
 *		 	dst			=	 Daylight saving  time
 *		 	tz				=	 Time zone
 *		 	glong		=  Geographical longitude of the place
 *		 	glat			=  Geographical latitude of the place
 *	
 *	 Return value:
 * 			RET[0]	=	 The azimuth in decimal form and number
 *			RET[1]	=  The height of the Sun at the horizon in , ',  
 *			RET[2]	=  The height of the Sun at the horizon in decimal degrees and as a number
 *			RET[3]	=	 Convert status from GST to UT.  Values are "Warning" + "OK"
 *								and as a string
 *			RET[4 ]	=	 Status of sunrise as a string .  Values are "OK",
								"Never Visible"  and  "Always Visible"  ...
  *************************************************************************************** */
 function sunsetAZ(day, month, year, dst, tz, glong, glat){
 	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LD = parseInt(day);					// Tag, Monat, Jahr Berechnungsdatum
 	var LM = parseInt(month);				// Day, month, year Calculation date
	var LY= parseInt(year);
	var DS = parseInt(dst);					// Sommer/Winterzeit - Daylight saving time
	var ZC = parseInt(tz);						// Zeitzone - Time zone
	var GL = Number(glong);				// Lngengrad des Orts - Longitude of the place
	var GP = Number(glat);					// Breitengrad des Orts - Latitude of the place
	var GD = 0;											// Grennwich Datum aus lokalem Datum
	var GM = 0;
	var GY = 0;
	var UT = 0.0;											// Grennwich UT
	var eGSTUT = "";									// Fehler bei der Konvertierung 
	var X = 0.0;											// Greenwich Sternzeit - Greenwich siderial time
	var Y = 0.0;											// quatoriale Deklination der Sonne
																	//  Equatorial declination of the Sun
	var A = 0.0;											// Korregierte Longitude der Sonne
																	// Corrected longitude of the sun
	var SR = 0.0;											// Longitude der Sonne
	var LA = 0.0;											// Die lokale siderische Zeit der RA des Aufgangs
																	//  The local sidereal time of the RA of the rise
																	// Longitude of the Sun
	var S = "";												// Sichtbarkeitsstatus der Sonne als String
																	// Visibility of the Sun as string
	var XX = 0.0;											// Returnwerte ist die dez. Zeit des Aufgangs
																	// Return value is the sunrise time
	var DI	= 0.8333333;								// Das horizontale Shifting mit allen Korrekturen der Sonne
																	// The horizontal shifting with all corrections of the sun
	var RET = new Array();					// Returnarray fr die Rckgabewerte
	// Fehlerkriterium auf OK setzen ...
	// Error criterion set to OK . . .
	var S = "OK";													
	// Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 12:00 . . .
	GD = localtime2GreenwichDay(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(12, 0, 0,  DS, ZC, LD, LM, LY);
	// ... berechne die Longitude der Sonne zum Berechnungszeitraum ...
	SR = sunLong(12, 0, 0, DS, ZC, GD, GM, GY);
	// ... Springe zur Unterfunktion und berechne die Werte fr 12:00 Uhr am
	// Berechnungstag ...
	// ... Jump to the subfunction and calculate the values for 12:00 on ...
 	// Calculation day
	sub3710();
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// Dezimale Greenwich Sternzeit in Stunden aus der lokalen Sternzeit ermitteln ...
	// Decimal Greenwich Determine stardate in hours from the local stardate . . .
	X = lokalST2GreenwichST(LA, 0, 0, GL);
	// ... und aus der Greenwich Sternzeit die Universal Time (UT) berechnen ...
	// . . . and calculate the Universal Time (UT) from the Greenwich Stardate . . .
	UT =  greenwichST2UT(X, 0, 0, GD, GM, GY);
	// ... Ist bei der Konvertierung der lokalen siserischen Zeit zu Greenwich siderischer
	// Zeit ein Fehler aufgetreten, dann Abbruch ...
	// . . . is sidereal when converting local siseric time to Greenwich more sidereal
	// Time an error occurred, then abort . . .
	if ((eGSTUT = errorGreenwichST2UT(X, 0, 0, GD, GM, GY)) != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// ... und jetzt mit der UT in Greenwich, ohne DST und TZ, die Longitude
	// der Sonne neu berechnen ...
	// ...  and now with the UT in Greenwich, without DST and TZ, the longitude
 	// recalculate the Sun ...
	SR = 0.0;
	SR = sunLong(UT, 0, 0, 0, 0, GD, GM, GY);
	// ... und gehe dann zur Unterfunktion sub3710() ...
	// ... and then go to function sub3710() ...
	sub3710();
	
	// ... wenn S einen Fehler enthlt, dann Abbruch der Funktion mit einem Fehler ...
	//  ... if S contains an error, then abort the function with an error ...
    if (S != "OK"){
		XX = -99.0;
		// ... rufe die Beendenfunktion auf, da Fehler ...
		// ... call the stop function, since error ...
		sub3740();
	}
	// ... nun berechne den Azimuth des Sonnenaufgangs am Standort ...
	// . . . now find the azimuth of the sunrise at the location . . .
	XX = setLocalSiderealDegreesAZ(X/15.0, 0, 0, Y, 0, 0, DI, GP);
	// ... und ans Ende der Funktion springen ...
	// ... and jump to the end of the function ...
	sub3740();
	
	function sub3710(){
		A = SR + nutationLong(GD, GM, GY) - 0.005694;
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die RA in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for the RA in degrees ...
		X = eclipticCoords2EquatorialRA(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// ... und dann konvertiere ekliptische Koordinaten zu quatorialen Koordinaten fr die Deklination in Grad ...
		// ...  and then convert ecliptic coordinates to equatorial coordinates for declination in degrees ...
		Y = eclipticCoords2EquatorialDec(A, 0, 0, 0, 0, 0, GD, GM, GY);
		// Die lokale siderische Zeit der RA des Aufgangs berechnen ...
		//  Calculate the local sidereal time of the RA of the rise ...
		LA = setLocalSiderealTime((X/15.0), 0, 0, Y, 0, 0, DI , GP);
		// ... und dann die Sichtbarkeit des Mondes berprfen, ob er Auf- und Untergeht, niemals
		// zu sehen ist, oder immer zu sehen ist ... kann auch fr die Sonne verwendet werden ...
		// ...  and then check the visibility of the moon to see if it rises and sets, never
 		// can be seen, or can always be seen ... can also be used for the Sun ...
		S = visibleStateMoonRiseSet((X / 15.0), 0, 0, Y, 0, 0, DI, GP);
		// ... zurck zum Aufrufer.
		// ... back to the caller
		return;
	}
	// Abbruchfunktion ...
	// Abort function . . .
	function sub3740(){
		// ... und mit Fehler raus aus der Funktion ...
		// ... and with error out of function ...
	}
	// In XX[0] steht der dezimale Azimut ...
	//  In XX[0] the decimal azimut is ...
	RET.push(XX);
	// In XX[1] steht die Hhe der Sonne ber dem Horizont in , ', '' als Horizontal Koordinate  ...
	//  In XX[1] the height of the sun is above the horizon in , ',  as horizontal  Coordinates...
	RET.push(decDegrees2AngleOfHours(objectHeight(XX, Y, GP )));
	// In XX[2] steht die Hhe der Sonne ber dem Horizont in dez. Grad als Horizontal Koordinate  ...
	//  In XX[2] the height of the sun is above the horizon in decimal degrees as horizontal  Coordinates...
	RET.push(objectHeight(XX, Y, GP ));
		// In XX[3] steht ob die Zeitkonvertierung OK war oder Warnung ...
	//  In XX[13 it is written whether the time conversion was OK or warning ...
	RET.push(eGSTUT);
	// In XX[4] steht der Status des Sonnenaufgangs, ob OK,
	// Immer sichtbar oder Nie sichtbar ...
	//  In XX[4] the state of the sunrise, whether OK,
 	//  Always visible or Never visible ...
	RET.push(S);
	// ... und zurck zum Aufrufer.					
	// ... and back to the caller.
	return(RET);
 
 }// End of sunsetAZ()