/* ************************************************************************************
 *	Das Modul beinhaltet Funktionen zur Berechnung bzw. Konvertierung von Datum und 
 *	Uhrzeiten, Schaltjahr, Fhrende Nullen ..... 
 *	Die Funktionen und Algorithmen haben keinen Anspruch auf
 *	hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module contains functions for calculation and Conversion of date and 
 * clock times, leap year, leading zeros . . . . . 
 * The functions and algorithms do not claim 
 * high accuracy, completeness and correctness.
 *
 *	Funktionsliste:
 *
 *  - ut2LocalTime()							- 	Konvertiert ein Datum im UT Format in die lokale Uhrzeit
 * 													  	  	als Dezimalstunden
 *														- 	Converts a date in UT format to the local time 
 *														  	as decimal hours
 *	- percentageOfDecimal2Time()		-	Konvertiert eine dezimale Stundenangabe von Nachkommastellen,
 *													  	  	wie z.B. 0,5421666 in eine Stundenangabe hh:mm.ss.ms
 *														-	Converts a decimal hour specification from decimal places,
 * 														  	like z. B. 0. 5421666 into an hour hh:mm. ss. ms
 * - ut2TT()										-	Wandelt die Universal Time (UT)  in die Terrestic
 *													      	Time (TT), die 66.7 s hinter der UT liegt.
 *														-	Convert Universal Time (UT) to Terrestic
 * 														  	Time (TT), which is 66. 7 seconds behind UT.
 * - convertDateToDaycount ()			-	Ermittelt die Anzahl der Tage des angegebenen
 *														  	Datums von Jahresanfang bis zu dem Datum.
 *														-	Returns the number of days of the specified 
 * 														  	date from the beginning of the year to the date.
 *	- isLeapYear()								-	Ermittelt ob ein Jahr ein Schaltjahr ist.	
 * 															Determines if a year is a leap year.
 *	- LPad( )										-	Fllt eine Zeichenketten vorne	mit n-Zeichen auf.
 *															Fills a strings at the front with n-signs.
 *	- dectime2Time()							-	Konvertiert eine dezimale Stundenangabe,
 *														  	wie z.B. 18,5421666, also eine Dezimalzahl > 0.0  in
 *													      	eine Stundenangabe hh:mm.ss.ms .
 *															Converts a decimal hour specification, 
 * 														  	like z. B. 18. 5421666, i. e. a decimal number > 0. 0 in 
 * 														  	an hour hh:mm. ss. ms .
 *	- time2Dectime()							-	Konvertiert die als h, m, s, ms zu bergebenden Zeit-
 *														  	werte in eine Gleitkommazahl.
 *															Converts the time to be passed as h, m, s, ms
 * 														  	Values in a floating point number.
 * - dezHours2dezDeg()						-	Konvertiert dezimale Stunden von Lngengraden zu dezimalen Graden
 *															Converts decimal hours of longitudes to decimal degrees
 * - dezDegs2dezHours()					-	Konvertiert dezimale Grade der Lngengrade zu dezimalen Stunden 
 *															Converts decimal degrees of longitudes to decimal hours
 * - localtime2GreenwichYear()			-	Berechnet aus lokaltem Datum das Greenwich Jahr.
 *															Calculates from local date the Greenwich year.
 * - localtime2GreenwichMonth()		-	Berechnet aus lokaltem Datum den Greenwich Monat.
 *															Calculates from local date the Greenwich month.
 * - localtime2GreenwichDay()			-	Berechnet aus lokaltem Datum den Greenwich Tag.
 *															Calculates from local date the Greenwich day.
 * - localtime2UT()								-	Berechnet aus lokaltem Datum die Greenwich UT.
 *															Calculates from local date the Greenwich UT.
 * - localDate2GreenwichDate()			-	Berechnet aus dem lokalen Datum das komplette Greenwich Datum.
 *															Calculates from local date the complete Greenwich date.
 * - ut2LocalDate()							-	Berechnet aus dem UT Datum das lokale Datum
 *															Calculates from UT date the complete local date.
 *
 *	Last update:
 *	Update			13.04.2021				Dieter Steffan
 *
 *********************************************************************************** */




 /* *********************************************************************************
 * Die Funktion berechnet aus den dezimalen Lngengraden die dezimalen
 * Stunden. Zum Beispiel aus 15 Grad stlicher Lnge 1 Stunde stlicher
 * Lnge.
 *
 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
 *	
 *	bergabewert:
 * 		dd				= Grad als Dezimalwert und Number
 *	
 *	Returnwert:
 *		DD 				= Stunden als Dezimalwert und Number
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates from the decimal longitude the decimal longitude
 *  Hours .  For example, from 15 degrees east longitude 1 hour east
 *  Length .
 *
 *	 The transfer values are converted to the correct data type.
 *	
 *	 Transfer value:
 * 		 dd				 =  Degree as decimal value and number
 *	
 *	 Return value:
 *		 DD 				 =  Hours as decimal value and number
 *************************************************************************************** */
 function dezDegs2dezHours(dd){
	 var DD = Number(dd);		// Dezimale Grade - Decimal degrees
	 // Ergebnis zurck zum Aufrufer.
	 // Result back to the caller.
	 return(DD / 15.0);
	 
 }// end of dezDegs2dezHours()
 
 
 
  /* *********************************************************************************
 * Die Funktion berechnet aus dezimalen Stunden die dezimale Gradzahl
 * des Lngengrades. Zum Beispiel aus 1 Stunde stlicher Lnge 15,0 Grad
 * stlicher Lnge.
 *
 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
 *	
 *	bergabewert:
 * 		dh				= Stunden als Dezimalwert und Number
 *	
 *	Returnwert:
 *		DH 				= Korrespondierende Grad als Dezimalwert und Number
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the decimal degree number from decimal hours
 * the degree of longitude.  For example, from 1 hour E 15.0 degrees E.
 *
 *	 The transfer values are converted to the correct data type.
 *	
 *	 Transfer value:
 * 		 dh				 =  Hours as decimal value and number
 *	
 *	 Return value:
 *		 DH 				 =  Corresponding degrees as decimal value and number
 *************************************************************************************** */
 function dezHours2dezDeg(dh){
	 var DH = Number(dh);		// Dezimale Stunden - Decimal hours
	 // Ergebnis zurck zum Aufrufer.
	 // Result back to the caller.
	 return(DH * 15.0);
	 
 }// end of dezimalHour2DezimalHours()
 
 
 
 
 /* *********************************************************************************
	 * Die Funktion berechnet aus dem lokalen Datum mit Uhrzeit, DST und
	 *	Zeitzone das Greenwich Jahr. Dies mag erst sinnlos erscheinen. Es
	 * kann aber sein, das bei Jahresbergngen, bedingt durch die Zeitzone
	 * und DSTdas berechnete Greenwich Jahr davor oder dahinter liegt.
	 *
	 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
	 *	
	 *	bergabewert:
	 * 		hour				= Lokale Stunde als Dezimalwert und Number
	 *		minute			= Lokale Minute als Dezimalwert und Number
	 *		second		= Lokale sekunde als Dezimalwert und Number
	 *		dst				= Daylight saving Time (DST)Sommer/Winterzeit
	 *		tz					= Zeitzone -12 bis +12
	 *		Day				= Lokaler Tag als Dezimalwert und Number
	 *		Month			= Lokaler Monat als Dezimalwert und Number
	 *		Year			= Lokales jahr als Dezimalwert und Number
	 *	
	 *	Returnwert:
	 *		year 		=	Returnwert ist das Greenwich Jahr	als Dezimalwert
								und Integer.
 *	---------------------------------------------------------------------------------------------------
	* The function is calculated from the local date with time, DST and
	* Time zone is the Greenwich year. This may seem pointless at first. It is
	* but it may be that at year-round transitions, due to the time zone
	* and DSTthe calculated Greenwich year is before or after.
	*
	* The transfer values are converted to the correct data type.
	*
	* Transfer value:
	* 		hour 		= Local hour as decimal value and number
	* 		minute 	= Local minute as decimal value and number
	* 		second 	= Local second as decimal value and number
	* 		dst 			= Daylight saving Time (DST) Summer/Winter Time
	* 		tz 			= Time zone -12 to +12
	* 		Day 		= Local day as decimal value and number
	* 		Month 		= Local month as decimal value and number
	* 		Year 		= Local year as decimal value and number
	*
	* Return value:
	* 		year 		= Return value is the Greenwich year as Integer.
  *************************************************************************************** */
 function localtime2GreenwichYear(hour, minute, second, dst,  tz, Day, Month, Year){
 	var HOUR = Number(hour);			// Stunde der Berechnung
																	// Hour of calculation
	var MINUTE = Number(minute);	// Minute der Berechnung
																	// Minute of calculation
	var SECOND = Number(second);	// Sekunde der Berechnung
																	// Second of calculation
	var TZ = parseInt(tz);						// Zeitzone der Berechnung
																	// Time zone of calculation
	var DST = parseInt(dst);					// Daylight saving time der Berechnung
																	// Daylight saving time of calculation
	var DAY = Number(Day);				// Tag der Berechnung
																	// Day of calculation
	var MONTH = parseInt(Month);		// Monat der Berechnung
																	// Month of calculation
	var YEAR = parseInt(Year);			// Jahr der Berechnung
																	// Year of calculation
	var year = 0;											// Greenwich Jahr als Ergebnis
																	// Greenwich Jahr as result
	// Aus der Uhrzeit eine Dezimalzahl berechnen ...
	// Calculate a decimal number from the time . . .
    var A = time2Dectime(HOUR, MINUTE, SECOND, 0);
	//console.log("localtime2GreenwichDay A=" + A);
	// ... davon die Zeitzone und Daylight Savings Time abziehen ...
	// . . . subtract the time zone and Daylight Savings Time . . . 
    var B = A - TZ - DST;
	//console.log("localtime2GreenwichDay B=" + B);
	// ... und dann zum Berechnungstag mit 1/24-igstel dazuaddieren ...
	// . . . and then add 1/24th to the calculation day. . .
    var C = DAY + (B / 24);
	//console.log("localtime2GreenwichDay C=" + C);
	// ... mit dem Tag + Monat und Jahr das julianische Datum berechnen ...
	// . . . with the day + month and year calculate the Julian date . . .
    var D = calDate2JulDate(C, MONTH, YEAR);
	//console.log("localtime2GreenwichDay D=" + D);
	// ... und aus diesem julianischen Datum das Jahr zurckrechnen ...
	// . . . and calculate the day back from this Julian date. . .
    year = julianDate2CalYear(D);
	//... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(year);
	
 } // End of localtime2GreenwichYear()
 
 
 
 /* *********************************************************************************
	 * Die Funktion berechnet aus dem lokalen Datum mit Uhrzeit, DST und
	 *	Zeitzone den Greenwich Monat. Dies mag erst sinnlos erscheinen. Es
	 * kann aber sein, das bei Monats/Jahresbergngen, bedingt durch die Zeitzone
	 * und DSTder berechnete Greenwich Monatdavor oder dahinter liegt.
	 *
	 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
	 *	
	 *	bergabewert:
	 * 		hour				= Lokale Stunde als Dezimalwert und Number
	 *		minute			= Lokale Minute als Dezimalwert und Number
	 *		second		= Lokale sekunde als Dezimalwert und Number
	 *		dst				= Daylight saving Time (DST)Sommer/Winterzeit
	 *		tz					= Zeitzone -12 bis +12
	 *		Day				= Lokaler Tag als Dezimalwert und Number
	 *		Month			= Lokaler Monat als Dezimalwert und Number
	 *		Year			= Lokales jahr als Dezimalwert und Number
	 *	
	 *	Returnwert:
	 *		month 		=	Returnwert ist das Greenwich Monat	als Dezimalwert
	 *							und Integer.
 *	---------------------------------------------------------------------------------------------------
	* The function is calculated from the local date with time, DST and
	* Time zone is the Greenwich month. This may seem pointless at first. It is
	* but it may be that at month/year transitions, due to the time zone
	* and DSTthe calculated Greenwich is before or after the month.
	*
	* The transfer values are converted to the correct data type.
	*
	* Transfer value:
	* 		hour 		= Local hour as decimal value and number
	* 		minute 	= Local minute as decimal value and number
	* 		second 	= Local second as decimal value and number
	* 		dst 			= Daylight saving Time (DST) Summer/Winter Time
	* 		tz 			= Time zone -12 to +12
	* 		Day 		= Local day as decimal value and number
	* 		Month 		= Local month as decimal value and number
	* 		Year 		= Local year as decimal value and number
	*
	* Return value:
	* 		ymonth 	= Return value is the Greenwich month as Integer.
  *************************************************************************************** */
 function localtime2GreenwichMonth(hour, minute, second, dst,  tz, Day, Month, Year){
 	var HOUR = Number(hour);			// Stunde der Berechnung
																	// Hour of calculation
	var MINUTE = Number(minute);	// Minute der Berechnung
																	// Minute of calculation
	var SECOND = Number(second);	// Sekunde der Berechnung
																	// Second of calculation
	var TZ = parseInt(tz);						// Zeitzone der Berechnung
																	// Time zone of calculation
	var DST = parseInt(dst);					// Daylight saving time der Berechnung
																	// Daylight saving time of calculation
	var DAY = Number(Day);				// Tag der Berechnung
																	// Day of calculation
	var MONTH = parseInt(Month);		// Monat der Berechnung
																	// Month of calculation
	var YEAR = parseInt(Year);			// Jahr der Berechnung
																	// Year of calculation
	var month = 0;										// Greenwich Monat als Ergebnis
																	// Greenwich month as result
	// Aus der Uhrzeit eine Dezimalzahl berechnen ...
	// Calculate a decimal number from the time . . .
    var A = time2Dectime(HOUR, MINUTE, SECOND, 0);
	// ... davon die Zeitzone und Daylight Savings Time abziehen ...
	// . . . subtract the time zone and Daylight Savings Time . . . 
    var B = A - TZ - DST;
	// ... und dann zum Berechnungstag mit 1/24-igstel dazuaddieren ...
	// . . . and then add 1/24th to the calculation day. . .
    var C = DAY + (B / 24);
	// ... mit dem Tag + Monat und Jahr das julianische Datum berechnen ...
	// . . . with the day + month and year calculate the Julian date . . .
    var D = calDate2JulDate(C, MONTH, YEAR);
	// ... und aus diesem julianischen Datum den Tag zurckrechnen ...
	// . . . and calculate the day back from this Julian date. . .
    month = julianDate2CalMonth(D);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(month);
	
 } // End of localtime2GreenwichMonth()
 
 
 
 
 
 /* *********************************************************************************
	 * Die Funktion berechnet aus dem lokalen Datum mit Uhrzeit, DST und
	 *	Zeitzone den Greenwich Tag. Dies mag erst sinnlos erscheinen. Es
	 * kann aber sein, das bei Jahres/Monatsbergngen, bedingt durch die Zeitzone
	 * und DSTdas berechnete Greenwich Tag davor oder dahinter liegt.
	 *
	 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
	 *	
	 *	bergabewert:
	 * 		hour				= Lokale Stunde als Dezimalwert und Number
	 *		minute			= Lokale Minute als Dezimalwert und Number
	 *		second		= Lokale sekunde als Dezimalwert und Number
	 *		dst				= Daylight saving Time (DST)Sommer/Winterzeit
	 *		tz					= Zeitzone -12 bis +12
	 *		Day				= Lokaler Tag als Dezimalwert und Number
	 *		Month			= Lokaler Monat als Dezimalwert und Number
	 *		Year			= Lokales jahr als Dezimalwert und Number
	 *	
	 *	Returnwert:
	 *		day	 		=	Returnwert ist das Greenwich Tag	als Dezimalwert
								und Integer.
 *	---------------------------------------------------------------------------------------------------
	* The function is calculated from the local date with time, DST and
	* Time zone is the Greenwich day. This may seem pointless at first. It is
	* but it may be that at day-round transitions, due to the time zone
	* and DSTthe calculated Greenwich day is before or after.
	*
	* The transfer values are converted to the correct data type.
	*
	* Transfer value:
	* 		hour 		= Local hour as decimal value and number
	* 		minute 	= Local minute as decimal value and number
	* 		second 	= Local second as decimal value and number
	* 		dst 			= Daylight saving Time (DST) Summer/Winter Time
	* 		tz 			= Time zone -12 to +12
	* 		Day 		= Local day as decimal value and number
	* 		Month 		= Local month as decimal value and number
	* 		Year 		= Local year as decimal value and number
	*
	* Return value:
	* 		day 			= Return value is the Greenwich day as Integer.
  *************************************************************************************** */
 function localtime2GreenwichDay(hour, minute, second, dst,  tz, Day, Month, Year){
 	var HOUR = Number(hour);			// Stunde der Berechnung
																	// Hour of calculation
	var MINUTE = Number(minute);	// Minute der Berechnung
																	// Minute of calculation
	var SECOND = Number(second);	// Sekunde der Berechnung
																	// Second of calculation
	var TZ = parseInt(tz);						// Zeitzone der Berechnung
																	// Time zone of calculation
	var DST = parseInt(dst);					// Daylight saving time der Berechnung
																	// Daylight saving time of calculation
	var DAY = Number(Day);				// Tag der Berechnung
																	// Day of calculation
	var MONTH = parseInt(Month);		// Monat der Berechnung
																	// Month of calculation
	var YEAR = parseInt(Year);			// Jahr der Berechnung
																	// Year of calculation
	var day = 0;											// Greenwich Tag als Ergebnis
																	// Greenwich day as result
	// Aus der Uhrzeit eine Dezimalzahl berechnen ...
	// Calculate a decimal number from the time . . .
    var A = time2Dectime(HOUR, MINUTE, SECOND, 0);
	// ... davon die Zeitzone und Daylight Savings Time abziehen ...
	// . . . subtract the time zone and Daylight Savings Time . . . 
    var B = A - TZ - DST;
	// ... und dann zum Berechnungstag mit 1/24-igstel dazuaddieren ...
	// . . . and then add 1/24th to the calculation day. . .
    var C = DAY + (B / 24);
	// ... mit dem Tag + Monat und Jahr das julianische Datum berechnen ...
	// . . . with the day + month and year calculate the Julian date . . .
    var D = calDate2JulDate(C, MONTH, YEAR);
	// ... und aus diesem julianischen Datum den Tag zurckrechnen ...
	// . . . and calculate the day back from this Julian date. . .
    var E = julianDate2CalDay(D);
    day = Math.floor(E);
	//... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(day);
	
 } // End of localtime2GreenwichDay()
 
 
 
 
  /* *********************************************************************************
	 * Die Funktion Berechne aus der lokalen Zeit, Datum und DST, TZ die 
	 * Weltzeit UT in Greenwich.
	 * 
	 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
	 *	
	 *	bergabewert:
	 * 		hour				= Lokale Stunde als Dezimalwert und Number
	 *		minute			= Lokale Minute als Dezimalwert und Number
	 *		second		= Lokale sekunde als Dezimalwert und Number
	 *		dst				= Daylight saving Time (DST)Sommer/Winterzeit
	 *		tz					= Zeitzone -12 bis +12
	 *		Day				= Lokaler Tag als Dezimalwert und Number
	 *		Month			= Lokaler Monat als Dezimalwert und Number
	 *		Year			= Lokales jahr als Dezimalwert und Number
	 *	
	 *	Returnwert:
	 *		UT	 		=	Returnwert ist die UT als Dezimalwert
								und Number.
 *	---------------------------------------------------------------------------------------------------
	* The function Computate from the local time, date and DST, TZ
	* World Time UT in Greenwich.
	*
	* The transfer values are converted to the correct data type.
	*
	* Transfer value:
	* 		hour 		= Local hour as decimal value and number
	* 		minute 	= Local minute as decimal value and number
	* 		second 	= Local second as decimal value and number
	* 		dst 			= Daylight saving Time (DST) Summer/winter time
	* 		tz 			= Time zone -12 to +12
	* 		Day 		= Local tag as decimal value and number
	* 		Month 		= Local month as decimal value and number
	* 		Year 		= Local year as decimal value and number
	*
	* Return value:
	* 		UT 			= Return value is the UT as decimal value
	*							and Number.
  *************************************************************************************** */
 function localtime2UT(hour, minute, second, dst,  tz, Day, Month, Year){
 	var HOUR = Number(hour);			// Stunde der Berechnung
																	// Hour of calculation
	var MINUTE = Number(minute);	// Minute der Berechnung
																	// Minute of calculation
	var SECOND = Number(second);	// Sekunde der Berechnung
																	// Second of calculation
	var TZ = parseInt(tz);						// Zeitzone der Berechnung
																	// Time zone of calculation
	var DST = parseInt(dst);					// Daylight saving time der Berechnung
																	// Daylight saving time of calculation
	var DAY = Number(Day);				// Tag der Berechnung
																	// Day of calculation
	var MONTH = parseInt(Month);		// Monat der Berechnung
																	// Month of calculation
	var YEAR = parseInt(Year);			// Jahr der Berechnung
																	// Year of calculation
	var UT = 0.0;											// Weltzeit als Ergebnis
																	// Universal time as result
	// Aus der Uhrzeit eine Dezimalzahl berechnen ...
	// Calculate a decimal number from the time . . .
    var A = time2Dectime(HOUR, MINUTE, SECOND, 0);
	// ... davon die Zeitzone und Daylight Savings Time abziehen ...
	// . . . subtract the time zone and Daylight Savings Time . . . 
    var B = A - TZ - DST;
	// ... und dann zum Berechnungstag mit 1/24-igstel dazuaddieren ...
	// . . . and then add 1/24th to the calculation day. . .
    var C = DAY + (B / 24);
	// ... mit dem Tag + Monat und Jahr das julianische Datum berechnen ...
	// . . . with the day + month and year calculate the Julian date . . .
    var D = calDate2JulDate(C, MONTH, YEAR);
	// ... und aus diesem julianischen Datum den Tag zurckrechnen ...
	// . . . and calculate the day back from this Julian date. . .
    var E = julianDate2CalDay(D);
	// ... und den Ganzzahlanteil ders Tages daraus ermitteln ...
	// . . . and determine the integer part of the day from it . . .
    var E1 = Math.floor(E);
	// ... und mit dem Nachkommaanteil des Tages die UT berechnen ...
	// . . . and use the decimal part of the day to calculate the UT . . .
    UT = 24.0 * (E - E1);
	// ... und zurck an den Aufrufer.
	// ... and back to the caller.
	return(UT);
	
 }// End of localtime2UT()
 
 
 
 
   /* *********************************************************************************
	 * Die Funktion Berechnet aus lokalem Datum und Uhrzeit die
	 * Greenwich Daten Tag, Monat, Jahr.
	 * Die Funktion bernimmt quasi die einzelnen Funktionen aus:
	 * 	localtime2GreenwichDay()
	 * localtime2GreenwichMonth()
	 * localtime2GreenwichYear()
	 * 
	 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
	 *	
	 *	bergabewert:
	 * 		hour				= Lokale Stunde als Dezimalwert und Number
	 *		minute			= Lokale Minute als Dezimalwert und Number
	 *		second		= Lokale sekunde als Dezimalwert und Number
	 *		dst				= Daylight saving Time (DST)Sommer/Winterzeit
	 *		tz					= Zeitzone -12 bis +12
	 *		Day				= Lokaler Tag als Dezimalwert und Number
	 *		Month			= Lokaler Monat als Dezimalwert und Number
	 *		Year			= Lokales jahr als Dezimalwert und Number
	 *	
	 *	Returnwert:
	 *		Array mit 3 Werten
	 *		DATE[0]	 	=	Greenwich Tag als Integer.
	 *		DATE[1]	 	=	Greenwich Monat als Integer.
	 *		DATE[2]	 	=	Greenwich Jahr als Integer.
 *	---------------------------------------------------------------------------------------------------
	* The function Calculates from local date and time
	* Greenwich data day, month, year.
	* The function basically takes over the individual functions from:
	* localtime2GreenwichDay ( )
	* localtime2GreenwichMonth ( )
	* localtime2GreenwichYear ( )
	*
	* The transfer values are converted to the correct data type.
	*
	* Transfer value:
	* 		hour 			= Local hour as decimal value and number
	* 		minute 		= Local minute as decimal value and number
	* 		second 		= Local second as decimal value and number
	* 		dst 				= Daylight saving Time (DST) Summer/winter time
	* 		tz 				= Time zone -12 to +12
	* 		Day 			= Local tag as decimal value and number
	* 		Month 			= Local month as decimal value and number
	* 		Year 			= Local year as decimal value and number
	*
	* Return value:
	* 		Array with 3 values
	* 		DATE[0] 	= Greenwich tag as an integer.
	* 		DATE[1 ] 	= Greenwich month as integer.
	* 		DATE[2] 	= Greenwich year as an integer.
  *************************************************************************************** */
  function localDate2GreenwichDate(hour, minute, second, dst,  tz, Day, Month, Year){
	var DATE = new Array();					// Rckgabewert im Array - Return value in the array
	var arrDateJD = new Array();			// Ergebnis JD zu Datum - Result JD to date
	var HOUR = Number(hour);				// Lokale Stunde - Local hour
	var MINUTE = Number(minute);		// Lokale Minute - Local minute
	var SECOND = Number(second);		// Lokale Sekunde - Local second
	var TZ = parseInt(tz);							// Zeitzone - Time zone
	var DST = parseInt(dst);						// DST Sommer/Winterzeit
	var DAY = Number(Day);					// Lokaler Tag - Local day
	var MONTH = parseInt(Month);			// Lokaler Monat - Local month
	var YEAR = parseInt(Year);				// Lokales Jahr - Local year
	// Rechne die lokale Zeit in dezimale Zeit um ...
	// Recalcate local time to decimals . . .
	var A =  time2Dectime(HOUR, MINUTE, SECOND, 0);
	// ... subtrahiere dann Zeitzone und DST von der Uhrzeit ...
	// . . . then subtract time zone and DST from the time . . .
	var B = A - TZ - DST;
	// ... addiere zum lokalen Tag den Tagesanteil von B ...
	// . . . add to the local day the daily share of B . . .
	var c = DAY + (B / 24);
	// ... und berchne daraus ein neues JD  mit TZ = 0, DST = 0 um 00:00:00.000 Uhr...
	// . . . and calculate a new JD with TZ = 0, DST = 0 at 00:00:00,000. . .
	 D = calcJD(c, MONTH , YEAR, 0, 0, 0, 0, 0, 0);
	 // ... und konvertiere das JD wiederum in ein normales Datum ...
	 // Array[0] = Der Tag des Datums als Gleitkommazahl. Nachkommastellen sind die Uhrzeit. 
	 // Array[1] = Der Monat des Datums als Ganzzahl.
	 //	Array[2] = Das Jahr des Datums als Ganzzahl.
	// . . . and convert the JD to a normal date . . .
	// Array[0] = The day of the date as a floating-point number. Decimals are the time.
	// Array[1 ] = The month of the date as an integer.
	// Array[2] = The year of the date as an integer.
	arrDateJD = calcJD2Date(D);
	// ... nun noch die Uhrzeit im Tag Index 0 elimenieren ...
	// . . . now eliminate the time in the day index 0 . . .
	var E = arrDateJD[0];
	var E1 = Math.floor(E);
	// ... Den Tag im Array speichern ...
	// . . . Save the day in the array ...
	DATE.push(E1);
	// ... Den Monat im Array speichern ...
	// . . . Save the month in the array ...
	DATE.push(arrDateJD[1]);
	// ... Das jahr im Array speichern ...
	// . . . Save the yearin the array ...
	DATE.push(arrDateJD[2]);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(DATE);
						
}// End of localDate2GreenwichDate()
 
 
 
// 
   /* *********************************************************************************
	 * Die Funktion Berechnet aus dem UT Datum das lokale Datum 
	 * anhand der Zeitzone und dem DST.
	 * 
	 *	Die bergabewerte werden in den richtigen Datentyp gewandelt.
	 *	
	 *	bergabewert:
	 * 		hour				= UT Stunde als Dezimalwert und Number
	 *		minute			= UT Minute als Dezimalwert und Number
	 *		second		= UT sekunde als Dezimalwert und Number
	 *		dst				= Daylight saving Time (DST)Sommer/Winterzeit
	 *		tz					= Zeitzone -12 bis +12
	 *		Day				= UT Tag als Dezimalwert und Number
	 *		Month			= UT Monat als Dezimalwert und Number
	 *		Year			= UT jahr als Dezimalwert und Number
	 *	
	 *	Returnwert:
	 *		Array mit 3 Werten
	 *		DATE[0]	 	=	Lokaler Tag als Integer.
	 *		DATE[1]	 	=	Lokaler Monat als Integer.
	 *		DATE[2]	 	=	Lokales Jahr als Integer.
 *	---------------------------------------------------------------------------------------------------
	* The function Calculates the local date from the UT date
	* based on the time zone and the DST.
	*
	* The transfer values are converted to the correct data type.
	*
	* Transfer value:
	* 		hour 		= UT hour as decimal value and number
	* 		minute 	= UT minute as decimal value and number
	* 		second 	= UT second as decimal value and number
	* 		dst 			= Daylight saving Time (DST) Summer/winter time
	* 		tz 			= Time zone -12 to +12
	* 		Day 		= UT tag as decimal value and number
	* 		Month 		= UT month as decimal value and number
	* 		Year 		= UT year as decimal value and number
	*
	* Return value:
	* 		Array with 3 values
	* 		DATE[0] = Local day as integer.
	* 		DATE[1 ] = Local month as integer.
	* 		DATE[2] = Local year as integer.
  *************************************************************************************** */
 function ut2LocalDate(hour, minute, second, dst,  tz, Day, Month, Year){
	var DATE = [];										// Rckgabewert im Array - Return value in the array
	var HOUR = Number(hour);			// UT Stunde - LUT hour
	var MINUTE = Number(minute);	// UT Minute - LUT minute
	var SECOND = Number(second);	// UT Sekunde - LUT second
	var TZ = parseInt(tz);						// Zeitzone - Time zone
	var DST = parseInt(dst);					// Sommer/Winterzeit DST - Summer/Winter time DST
	var DAY = Number(Day);				// UT Tag - UT day
	var MONTH = parseInt(Month);		// UT Monat - UT month
	var YEAR = parseInt(Year);			// UT Jahr - UT year
	
	// UT Zeit in dezimalen Wert konvertieren ...
	// Convert UT time to decimal value . . .
	var A =  time2Dectime(HOUR, MINUTE, SECOND, 0);
	// ... dazu kommt die Zeitzone ...
	// . . . then there is the time zone . . .
	var B = A + TZ;
	// ... und die Sommer/Winterzeit ...
	// . . . and the summer/winter time . . .
	var c = B + DST;
	// ... und dann aus dem UT Datum + die Zeit als Teil des Tages das JD errechnen ...
	// ... das hin- und her Rechnen ist wegen der zeitbergnge notwendig ...
	// . . . and then calculate the JD from the UT date + the time as part of the day . . .
	// . . . the back and forth calculation is necessary because of the time transitions . . .
	var D = calDate2JulDate(Math.floor(DAY), MONTH , YEAR)+ c / 24.0;
	// ... und daraus dann wieder das Datum berechnen ...
	 // Array[0] = Der Tag des Datums als Gleitkommazahl. Nachkommastellen sind die Uhrzeit. 
	 // Array[1] = Der Monat des Datums als Ganzzahl.
	 //	Array[2] = Das Jahr des Datums als Ganzzahl.
	 // . . . and then calculate the date again . . .
	// Array[0] = The day of the date as a floating-point number. Decimals are the time.
	// Array[1 ] = The month of the date as an integer.
	// Array[2] = The year of the date as an integer.
	var arrDateJD = calcJD2Date(D);
	// Den Kalendertag aus dem julianischen Datum ermitteln.
	// Achtung, die Nachkommastellen beinhalten die Uhrzeit
	// des Tages ...
	// Determine the calendar day from the Julian date.
	// Attention, the decimal points contain the time
	// of the day . . .
	var E = julianDate2CalDay(D);
	// ... daher die Uhrzeit entfernen...
	// . . . therefore remove the time...
	var E1 = Math.floor(E);
	// Den Tag im Array speichern ...
	// Save the day in the array . . .
	DATE.push(E1);
	// Den Monat im Array speichern ...
	// Save the month in the array . . .
	DATE.push(arrDateJD[1]);
	// Das Jahr im Array speichern ...
	// Save theyear in the array . . .
	DATE.push(arrDateJD[2]);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(DATE);
						
}// End of ut2LocalDate()
 
 
 

/* *********************************************************************************
 * Die Funktion konvertiert ein Datum im UT Format in die lokale Uhrzeit
 * als Dezimalstunden. Das Hin- und Herrechnen ist wegend er bergnge
 * an Tagen, Monaten, Jahren notwendig.
 *	
 *	bergabewert:
 * 		Dezimale Stunde 		= Die Stunden des Taes als Number
 * 		Minute						= Minuten des Tages als Number
 * 		Second						= Sekunden des Tages als Number
 * 		DST							= Sommer/Winterzeit (DST) als Integer
 * 		TZ							= Zeitzone (Timezone) als Integer
 * 		Berechnungstag		= Tag als Number
 * 		Berechnungsmonat 	= Monat als Number
 * 		Berechnungsjahr		= Jahr als Number
 *	
 *	Returnwert:
 *										= 	Returnwert ist die lokale Zeit aus dem Greenwich UT
 *											als Dezimalwert und Number
 *	---------------------------------------------------------------------------------------------------
 * The function converts a date in UT format to the local time 
 * as decimal hours. The back and forth calculation is necessary away he transitions 
 * on days, months, years. 
 *
 * Transfer value: 
 * 		Decimal hour 			= The hours of the day as number 
 * 		Minute 						= Minutes of the day as number 
 * 		Second 					= Seconds of the day as number 
 * 		DST 						= Summer/Winter time (DST) as integer 
 * 		TZ 							= Time zone (Timezone) as integer 
 * 		Calculated day 			= Day as number 
 * 		Calculated month 		= Month as number 
 * 		Calculated year 		= Year as number 
 * 
 * Return value: 
 * 										= Return value is the local time from the Greenwich UT 
 * 										   	as decimal value and number
  *************************************************************************************** */
function ut2LocalTime(hour, minute, second, dst,  tz, newMoonDay, newMoonMonth, newMoonYear){
	var HOUR = Number(hour);											// UT Zeit Stunde, Minute, Sekunde
	var MINUTE = Number(minute);									// UT time hour, minute, second
	var SECOND = Number(second);
	var TZ = parseInt(tz);														// Zeitzone - Time zone
	var DST = parseInt(dst);													// Sommer/Winterzeit - Summer/Winter time
	var NEWMOONDAY = Number(newMoonDay);			// UT Datum Tag, Monat, Jahr
	var NEWMOONMONTH = parseInt(newMoonMonth);// UT date day, month, year
	var NEWMOONYEAR = parseInt(newMoonYear);
	// Die UT Stunden ...
	// The UT hours ...
	var A =  HOUR;
	// ... + die Zeitzone ...
	// ... add time zone ...
	var B = A + TZ;
	// ... und Sommer/Winterzeit ...
	// ... and summer/winter time ...
	var c = B + DST;
	// ... mit dem Datum + den Tagesanteil der Zeit zu dem JD umrechnen ...
	// . . . with the date + the daily share of the time to the JD . . .
	var D = calcJD(Math.floor(NEWMOONDAY), NEWMOONMONTH , NEWMOONYEAR, 0, 0, 0, 0)+ c / 24.0;
	// ... dann wieder in ein normales Datum verwandeln ...
	// . . . then turn it back into a normal date . . .
	var arrDateJD = calcJD2Date(D);
	// ... und aus dem Tag  mit der Uhrzeit als Nachkommastellen ...
	// . . . and from the day with the time as decimal place . . .
	var E = arrDateJD[0];
	// ... den ganzzahligen Anteil ermitteln ...
	// . . . determine the integer portion . . .
	var E1 = Math.floor(E);
	// ... die dezimale Zeit ergibt sich dann aus 24 Stunden/Tag mal dem Nachkommaanteil ...
	// . . . the decimal time is then determined by 24 hours/day times the decimal part . . .
	var UTLCT= 24 * (E - E1);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(UTLCT);
						
}// End of ut2LocalTime()
 
 
 
 /* *********************************************************************************
	Die Funktion arbeitet hnlich dectime2Time()und konvertiert 
	eine dezimale Stundenangabe von Nachkommastellen,
	wie z.B. 0,5421666 in eine Stundenangabe hh:mm.ss.ms oder
	ein Array mit den einzelnen Werten.
	Es muss eine 0,1..... Dezimalzahl sein. Es gibt ja auch die
	Dezimalen Stunden als 18,4782156.
	Die einzelnen Werte des Rckgabewertes werden links
	mit ein "0" aufgefllt, wenn der Wert < 10 ist.
					
	bergabewert:					
	dectime 			= Stundenwert als Dezimalzahl
	what    				= Was zurckgegeben werden soll, ein String = "hh:mm:ss.ms"
								oder ein Array mit Index von 0 - 4 der einzelnen Werte.
								Wert: "array" oder "string"
				
	Returnwert:
	String z.B. als 18:01:45:005 oder
	Array mit 4 Werten an den einzelnen Indexen von 0 - 4 fr
	hh mm ss ms
					
	---------------------------------------------------------------------------------------------------
	The function works similarly to dectime2Time () and converts
	a decimal hour of decimal places,
	e. g. B. 0,5421666 in an hourly rate hh:mm. ss. ms or
	an array with the individual values. It's got to be a 0. 1. . . decimal. There are also the
	Decimal hours as 18,4782156. The individual values of the return value are displayed on the left
	replenished with a "0" if the value is < 10.
					
	Transfer value:
	dectime = hour value as decimal number
	what = What should be returned, a string = hh:mm:ss.ms
	or an array with an index of 0-4 of the individual values.
	Value: array or string

	Return value:
	String z. e. g. as 18:01:45:005 or
	Array with 4 values on the individual indexes from 0-4 for
	hh mm ss ms.
				
  *************************************************************************************** */
 function percentageOfDecimal2Time(dect, what){
	var DECT = Number(dect);		// Sicherheitshalber nach Number konvertieren
	var rest = 0.0;								// Restzeiten
	var hour = 0;								// Stunde
	var minute = 0;								// Minute
	var second = 0;							// Sekunde
	var msecond = 0;							// Millisekunde
	var RETTIME ;								// Ergebniswert - Result value
	// Ermittle was der Rckgabewert sein soll? ...
	// Determine what the return value should be? ...
	var WHAT = what.toLowerCase();
	// ... ein Array mit den einzelnen Werten von h, m, s, ms ...
	// ... an array with the individual values of h, m, s, ms ...
	if (WHAT == "array")
		RETTIME = [];
	// ... oder eine zusammengesetzte Zeichenkette wie hh:mm:ss.ms.
	// . . . or a compound string like hh:mm:ss. ms.
	else
		RETTIME = "";
	// Stunden berechnen ...
	// Calculate hours...
	hour = Math.floor(DECT * 24.0 );
	// ... und die Nachkommawerte ...
	// ... and the decimals ...
	rest = (DECT * 24) -  hour;
	// ... und Stunden zum Ergebnis hinzufgen ...
	// ... and add hours to the result ...
	if (WHAT == "array"){
		// ... wenn Stunde auf 24:00 Uhr herauskommt, dann auf 00:00 Uhr setzen ...
		// . . . if hour comes out 24:00, set to 00:00. . .
		if (parseInt(hour )== 24) {
			hour = 0;
		}
		RETTIME.push(LPad(hour, 2, "0"));
	}else{
		// ... wenn Stunde auf 24:00 Uhr herauskommt, dann auf 00:00 Uhr setzen ...
		// . . . if hour comes out 24:00, set to 00:00. . .
		if (parseInt(hour )== 24) {
			hour = 0;
		}
		RETTIME += LPad(hour, 2, "0") + ":";		
	}
	// Die Minuten berechnen ...
	// Calculate the minutes ...
	minute = Math.floor(rest * 60.0);
	// ... und deren Nachkommastellen ...
	// ... and their decimal places ...
	rest = (rest * 60.0) - minute;	
	// ... und zum Ergebnis hinzufgen ...
	// ... and add to the result ...
	if (WHAT == "array")
		RETTIME.push(LPad(minute, 2, "0"));
	else
		RETTIME += LPad(minute, 2, "0") + ":";			
	// Die Sekunden berechnen
	// Calculate the seconds ...
	second = Math.floor(rest * 60.0);
	// ... und deren Nachkommastellen ...
	// ... and their decimal places ...
	rest = (rest * 60.0) - second;			
	// ... und zum Ergebnis hinzufgen ...
	// ... and add to the result ...
	if (WHAT == "array")
		RETTIME.push(LPad(second, 2, "0"));
	else
		RETTIME += LPad(second, 2, "0") + ".";				
	// fr die Milli-Sekundenberechnung ist ...
	// for the milli-second calculation is . . .
	msecond = Math.trunc(rest * 1000.0);			
	// ... und zum Ergebnis hinzufgen ...
	// ... and add to the result ...
	if (WHAT == "array")
		RETTIME.push(LPad(msecond, 3, "0"));
	else
		RETTIME += LPad(msecond, 3, "0");
	// und Wert an den Caller zurckgeben.
	// and return the value to the caller.
	return(RETTIME);
	
 }// End of percentageOfDecimal2Time()

 
 
 /* ******************************************************
	Die Funktion wandelt die Universal Time (UT)  in die Terrestic
	Time (TT), die 66.7 s hinter der UT liegt.
						
	bergabewert:
	UT		= Die Universal Time als Number
						
	Rckgabewert:
	TT als Number		
	----------------------------------------------------			
	The function converts Universal Time (UT) into Terrestic
	Time (TT), which is 66. 7 s behind the UT. 
	
	Transfer value:
	UT = The Universal Time as Number

	Return value:
	TT as Number
*************************************************** */
 function ut2TT(ut){
	// TT - UT = deltaT = 66.07 s
	// TT = deltaT + UT
	return(Number(ut) + (66.07/(3600.0 * 24.0)));
	
 }// End of ut2TT()
 
 

/* ******************************************************
	Die Funktion ermittelt die Anzahl der Tage des angegebenen
	Datums von Jahresanfang bis zu dem Datum. Dabei werden
	Schaltjahre bercksichtigt.
	Es wird keine Plausibilittsprfung der bergabedaten vor-
	genommen.
						
	bergabewert:
	day		= Integer und der Tag des Datums
	month	= Integer und der Monat des Datums
	year	= Integer und das Jahr des Datums
						
	Rckgabewert:
	Anzahl der Tage des Jahres bis zu dem Datum als Integer.
						
	----------------------------------------------------
						
	Date from the beginning of the year to the date. This involves:
	Schedule years are taken into account. No verification of the 
	plausibility of the surrender data is provided.
	taken.
						
	Transmission value:
	day = integer and date day
	month = integer and the month of the date
	year = integer and the year of the date

	Return value:
	Number of days of the year up to the date as integer.

*************************************************** */
function convertDateToDaycount (day, month, year){
	var DAY = parseInt(day);					// Den Tag des Monats bernehmen
	var MONTH = parseInt(month);		// Den Monat bernehmen
	var YEAR = parseInt(year);				// Das Jahr bernehmen
	var daysTillDate = 0.0;							// Returnwert ist Dezimalzahl
	// Fr Monate grsser Februar
	if (MONTH > 2){
		// Addiere 1 zum Monat hinzu ...
		// Add 1 to the month . . .
		MONTH += 1;
		// ... und multipliziere mit 30,6 ...
		// . . . and multiply by 30,6 . . .
		daysTillDate = MONTH * 30.6;
		// ... nehme den ganzzahligen Anteil davon ...
		//. . . take the integer part of it ...
		daysTillDate = Math.floor(daysTillDate);
		// Ist das Jahr ein Schaltjahr ...
		// Is the year a leap year . . .
		if (isLeapYear(YEAR)){
			// ... dann subtrahiere 62 ...
			// . . . then subtract 62 . . .
			daysTillDate = daysTillDate - 62;
		// Kein Schaltjahr ...
		// No leap year . . .
		} else {
			// ... und bei normalem Jahr 63
			// ... and in normal year 63
			daysTillDate = daysTillDate - 63;				
		}
		// ... und dazu kommt der Tag des Monats.
		// Dies ist die Anzahl der Tage des Jahres bis zum Datum.
		// . . . and on top of that comes the day of the month.
		// This is the number of days of the year up to the date.
		daysTillDate = parseInt(daysTillDate) + parseInt(DAY);		
		// Fr die Monate Januar und Februar
		// For the months of January and February
	} else {
		// Subtrahiere 1 vom Monat ...
		// Subtract 1 from the month . . .
		MONTH -= 1;				
		// Ist das Jahr ein Schaltjahr ...
		// Is the year a leap year . . .
		if (isLeapYear(YEAR)){
			// ... dann multipliziere den Monat
			// mit 62 und dividiere durch 2 ...
			//. . . then multiply the month
			// with 62 and divide by 2 . . .
			daysTillDate = (MONTH * 62) / 2;
		// Kein Schaltjahr ...
		// no Leap Year ...
		} else {
			// ... dann multipliziere den Monat
			// mit 63 und dividiere durch 2 ...
			//. . . then multiply the month
			// with 63 and divide by 2 . . .
			daysTillDate = (MONTH * 63) / 2;
		}
		// ... und davon den ganzzahligen Anteil ...
		// ... and the integer part of it ...
		daysTillDate = Math.floor(daysTillDate);
		// ... und dazu die Tage des Monats und das ist die Anzahl
		// der Tage im Jahr bis zu dem Datum ...
		// . . . and the days of the month and that's the number
		// the days in the year up to the date . . .
		daysTillDate = parseInt(daysTillDate) + parseInt(DAY);			
	}
	// und Wert an den Caller zurckgeben.
	// and return the value to the caller.
	return(daysTillDate);
					
}// End of convertDateToDaycount ()
					
					
					
/* ******************************************************
	Die Funktion ermittelt ob ein Jahr ein Schaltjahr ist.
	Ein Schaltjahr liegt vor, wenn das Jahr durch 4 und nicht
	durch 100 teilbar ist.
	Ist ein Jahr durch 4 und durch 100 teilbar, dann muss es
	durch 400 teilbar sein, um ein Schaltjahr zu sein.
				
	bergabewert:
	year = Ganzzahl, z.B. 1988
						
	Rckgabewert:
	true 	= Schaltjahr
	false 	= Kein Schaltjahr
						
	----------------------------------------------------
						
	The function determines whether a year is a leap year.
	A leap year is given if the year by 4 and not
	is divisible by 100. If a year is divisible by 4 and by 100,
	then it must be divisible by 400 to add one leap year.
						
	Transfer value:
	year = integer, e. g. B. 1988

	Return value:
	true = leap year
	false = No leap year
*************************************************** */		
function isLeapYear(year){
	ret = false;
	var fourhundred = Math.floor(year / 400.0); 	// Ganzzahl der 400 - Entire number of 400
	var yearFourhundred = year / 400.0					// Dezimalzahl der 400 - Decimal number of 400
	var four = Math.floor(year / 4.0);					// Ganzahl der 4 - Integer of the 4
	var yearFour = year / 4.0;									// Dezimalzahl der 4 - Decimal number of 4
	var hundred = Math.floor(year / 100.0);			// Ganzzahl 100  - Integer of the 100
	var yearHundred = year / 100.0;						// Dezimalzahl 100 - Decimal number of 100
	// Ist das Jahr durch 4 und nicht durch 100 teilbar, dann
	// Schaltjahr ...
	// If the year is divisible by 4 and not by 100, then
	// Leap year. . .
	if ( ((yearHundred - hundred) != 0) &&  ( (yearFour - four) == 0) ){
		ret = true;
	// Ist es durch 4 und durch 100 teilbar, muss es durch
	// 400 teilbar sein.
	// If it is divisible by 4 and by 100, it must be
	// be divisible.
	} else if ( (yearFourhundred - fourhundred) == 0 ) {
		ret = true;
	}
	// ... und zurck zum Caller.
	// ... and back to the caller.
	return(ret);
	
}// End of isLeapYear()
					
					
/* ******************************************************
	Die Funktion fllt eine Zeichenketten vorne
	mit n-Zeichen auf.
	Z.B. 1 -> 01
	izahl = 		Die Zahl ohne fhrende Nullen
	ianzStellen = 	Die gewnschte Anzahl von Stellen
	sFuelzeichen =	Das gewnschet fhrende Zeichen
						
	Rckgabewert =	Z.B. "01" oder "0001"
						
	----------------------------------------------------
						
	The function fills a string with number at the front
	with n-signs.
	E. g. 1 -> 01
						
	izahl = The number without leading zeros
	ianzStellen = The desired number of digits
	sFuel character = The desired leading character

	Return value = E. g. "01" or "0001"
   *************************************************** */
function LPad( iZahl, iAnzStellen, sFuellzeichen ){
	// Die aufzufllende Zahl als Zeichenkette Deklarieren.
	// Declare the number to be filled as a string.
	var sZahl = iZahl + "";
	// Vorne immer ein Fllzeichen anfgen.
	// Always add a fill character to the front.
	while( sZahl.length < iAnzStellen )
		sZahl = sFuellzeichen + sZahl;
	// ... und zurck zum Caller.
	// ... and back to the caller.
	return sZahl;
	
}// End of LPad()


				
/* *******************************************************
	Die Funktion konvertiert eine dezimale Stundenangabe,
	wie z.B. 18,5421666, also eine Dezimalzahl > 0.0  in
	eine Stundenangabe hh:mm.ss.ms oder
	ein Array mit den einzelnen Werten.
	Die Funktion percentageOfDecimal2Time(dect, what)arbeitet
	hnlich, nimmt aber als Wert eine Dezimalzahl < 0.0.
	
	Die einzelnen Werte des Rckgabewertes werden links
	mit ein "0" aufgefllt, wenn der Wert < 10 ist.
					
	bergabewert:					
	dectime 				= Stundenwert als Dezimalzahl
	what    				= Was zurckgegeben werden soll, ein String = "hh:mm:ss.ms"
									oder ein Array mit Index von 0 - 4 der einzelnen Werte.
									Wert: "array" oder "string"
				
	Returnwert:
	String z.B. als 18:01:45:005 oder
	Array mit 4 Werten an den einzelnen Indexen von 0 - 4 fr
	hh mm ss ms
					
	-------------------------------------------------------

	The function converts a decimal hour,
	e. g. B. 18,5421666, i. e. one decimal > 0. 0 in
	an hour reference hh:mm. ss. ms or
	an array with the individual values. The function percentageOfDecimal2Time (dect, what) works
	similar, but takes as a value a decimal number < 0. 0.
					
	Transfer value:
	dectime = hour value as decimal number
	what = What should be returned, a string = hh:mm:ss.ms
	or an array with an index of 0-4 of the individual values.
	Value: array or string

	Return value:
	String z. e. g. as 18:01:45:005 or
	Array with 4 values on the individual indexes from 0-4 for
	hh mm ss ms.
				
  ****************************************************** */
function dectime2Time(dectime, what){
	// Deklaration und Definition der Variablen.
	// Declaration and definition of the variables.	
	var rest = 0.0;
	var hour = 0;
	var minute = 0;
	var second = 0;
	var msecond = 0;
	
	// Ermittle was der Rckgabewert sein soll? ...
	// Determine what the return value should be? ...
	var WHAT = what.toLowerCase();
	// ... ein Array mit den einzelnen Werten von h, m, s, ms ...
	// ... an array with the individual values of h, m, s, ms ...
	if (WHAT == "array")
		var time = [];
	// ... oder eine zusammengesetzte Zeichenkette wie hh:mm:ss.ms.
	// . . . or a compound string like hh:mm:ss. ms.
	else
		var time = "";
	// Stunden berechnen ...
	// Calculate hours...
	hour = Math.floor(dectime );
	// ... und die Nachkommawerte ...
	// ... and the decimals ...
	rest = dectime  -  hour;
	// ... und Stunden zum Ergebnis hinzufgen ...
	// ... and add hours to the result ...
	if (WHAT == "array")
		time.push(LPad(hour, 2, "0"));
	else
		time += LPad(hour, 2, "0") + ":";				
	// Die Minuten berechnen ...
	// Calculate the minutes ...
	minute = Math.floor(rest * 60.0);
	// ... und deren Nachkommastellen ...
	// ... and their decimal places ...
	rest = (rest * 60.0) - minute;	
	// ... und zum Ergebnis hinzufgen ...
	// ... and add to the result ...
	if (WHAT == "array")
		time.push(LPad(minute, 2, "0"));
	else
		time += LPad(minute, 2, "0") + ":";			
	// Die Sekunden berechnen
	// Calculate the seconds ...
	second = Math.floor(rest * 60.0);
	// ... und deren Nachkommastellen ...
	// ... and their decimal places ...
	rest = (rest * 60.0) - second;			
	// ... und zum Ergebnis hinzufgen ...
	// ... and add to the result ...
	if (WHAT == "array")
		time.push(LPad(second, 2, "0"));
	else
		time += LPad(second, 2, "0") + ".";				
	// fr die Milli-Sekundenberechnung ist ...
	// for the milli-second calculation is . . .
	msecond = Math.trunc(rest * 1000.0);			
	// ... und zum Ergebnis hinzufgen ...
	// ... and add to the result ...
	if (WHAT == "array")
		time.push(LPad(msecond, 3, "0"));
	else
		time += LPad(msecond, 3, "0");
	// ...	und zurck an den Caller.		
	//. . . and back to the caller.
	return(time);
	
}// End of dectime2Time()
				
				
				
/* *******************************************************
	Die Funktion konvertiert die vier zu bergebenden Zeit-
	werte in eine Gleitkommazahl vom Typ Number.
					
	bergabewerte als 24h Zeitdaten:
					
	HH = Stunden als Ganzahl
	MM = Minuten als Ganzzahl
	SS = Sekunden als Ganzzahl
	MS = Millisekunden als Ganzzahl
				
	Returnwert: Stundenwert als Decimalzahl
					
	-------------------------------------------------------
	
	The function converts the four time-
	values to a floating point number of type Number.
					
	Transfer values as 24h Time data:

	HH = hours as number of hours
	MM = minutes as an integer
	SS = seconds as an integer
	MS = milliseconds as an integer

	Return value: Hour value as decimal number
				
  ****************************************************** */
function time2Dectime(HH, MM, SS, MS){
	// Initialisierung der Variablen
	// Initialization of the variables
	var dectime = 0.0;
	var val = 0.0;
	var hh = Number(HH); var mm = Number(MM); var ss = Number(SS); var ms = Number(MS);
	// Die Millisekunden : 1000, da 1 Sek. = 1000 ms ...
	// The milliseconds: 1000, da 1 sec. = 1000 ms ...
	val =  Math.abs(Number(ms)) / 1000.0;		
	// ... und zu den Sekunden addieren ...
	// ... and add to the seconds ...
	ss =  Math.abs(Number(ss)) + val;	
	// ... die Sek. : 60, da 1 Minute = 60 Sek. ...
	// ... the second. 60. by 1 minute = 60 seconds. ...
	val = Math.abs(Number(ss)) / 60.0;		
	// ... und dann zu den Minuten addieren ...
	// ... and then add to the minutes ...
	mm =  Math.abs(Number(mm)) + val;		
	// Die Minuten : 60, da 1 Stunden = 60 Minuten ...
	// the minutes: 60, since 1 hours = 60 minutes ...
	val =  Math.abs(Number(mm)) / 60.0;			
	// ... und dann nur noch die Stunden dazu addieren ...
	// ... and then only add the hours to it ...
	dectime = (val + Math.abs(Number(hh)));
	// Bei negativen Zeiten das Ergebnis mal -1
	if ((HH < 0.0) ||  (MM < 0.0) || (SS <0.0) || (MS < 0.0) )
		dectime *= -1;

	// ... und zurck an den Caller.
	// ... and back to the caller.
	return(dectime);
	
}// End of time2Dectime()